#include "thumbnailmanager.h"

#include <QStandardPaths>
#include <QFile>
#include <QBitmap>
#include <QGraphicsEffect>
#include <QPainter>
//#include <QThread>
#include <QTimer>
#include <QDBusInterface>
#include <QDBusError>

#include <QGuiApplication>
#include <QThread>
#include <QDebug>
#include <config-plasma.h>
#include <config-x11.h>
#include <malloc.h>
// X11
#if HAVE_XCB_COMPOSITE
#include <xcb/composite.h>
#include <xcb/damage.h>
#endif
#include <QX11Info>
#include <X11/Xutil.h>

//图片格式装换
QImage qimageFromXImage(void* __image)
{
    XImage* ximage = (XImage*)__image;
    QImage::Format format = QImage::Format_ARGB32_Premultiplied;
    if (ximage->depth == 24)
        format = QImage::Format_RGB32;
    else if (ximage->depth == 16)
        format = QImage::Format_RGB16;

    QImage image = QImage(reinterpret_cast<uchar*>(ximage->data),
                          ximage->width, ximage->height,
                          ximage->bytes_per_line, format).copy();

    // 大端还是小端?
    if ((QSysInfo::ByteOrder == QSysInfo::LittleEndian && ximage->byte_order == MSBFirst)
            || (QSysInfo::ByteOrder == QSysInfo::BigEndian && ximage->byte_order == LSBFirst)) {

        for (int i = 0; i < image.height(); i++) {
            if (ximage->depth == 16) {
                ushort* p = reinterpret_cast<ushort*>(image.scanLine(i));
                ushort* end = p + image.width();
                while (p < end) {
                    *p = ((*p << 8) & 0xff00) | ((*p >> 8) & 0x00ff);
                    p++;
                }
            } else {
                uint* p = reinterpret_cast<uint*>(image.scanLine(i));
                uint* end = p + image.width();
                while (p < end) {
                    *p = ((*p << 24) & 0xff000000) | ((*p << 8) & 0x00ff0000)
                         | ((*p >> 8) & 0x0000ff00) | ((*p >> 24) & 0x000000ff);
                    p++;
                }
            }
        }
    }

    // 修复alpha通道
    if (format == QImage::Format_RGB32) {
        QRgb* p = reinterpret_cast<QRgb*>(image.bits());
        for (int y = 0; y < ximage->height; ++y) {
            for (int x = 0; x < ximage->width; ++x)
                p[x] |= 0xff000000;
            p += ximage->bytes_per_line / 4;
        }
    }

    return image;
}

//图片亮度处理
QImage adjustBrightness(QImage Img, int iBrightValue)
{
    int red, green, blue;
    int pixels = Img.width() * Img.height();
    unsigned int *data = (unsigned int *)Img.bits();

    for (int i = 0; i < pixels; ++i)
    {
        red= qRed(data[i])+ iBrightValue;
        red = (red < 0x00) ? 0x00 : (red > 0xff) ? 0xff : red;
        green= qGreen(data[i]) + iBrightValue;
        green = (green < 0x00) ? 0x00 : (green > 0xff) ? 0xff : green;
        blue= qBlue(data[i]) + iBrightValue;
        blue =  (blue  < 0x00) ? 0x00 : (blue  > 0xff) ? 0xff : blue ;
        data[i] = qRgba(red, green, blue, qAlpha(data[i]));
    }

    return Img;
}
//图片模糊处理
QT_BEGIN_NAMESPACE
extern Q_WIDGETS_EXPORT void qt_blurImage(QPainter *p, QImage &blurImage, qreal radius, bool quality, bool alphaOnly, int transposed = 0);
QT_END_NAMESPACE
QImage adjustBlur(QImage image, int blurV)
{
    QImage destImage(image.size(), QImage::Format_ARGB32);
    destImage.fill(QColor(255,255,255,240));
    if(!image.isNull())
    {
        QPainter painter(&destImage);
        qt_blurImage(&painter, image, blurV, true, false);
    }
    return destImage;
}

//copy自ndetaskbar.cpp
bool acceptWindow(WId window)
{
    QFlags<NET::WindowTypeMask> ignoreList;
    ignoreList |= NET::DesktopMask;
    ignoreList |= NET::DockMask;
    ignoreList |= NET::SplashMask;
    ignoreList |= NET::ToolbarMask;
    ignoreList |= NET::MenuMask;
    ignoreList |= NET::UtilityMask;
    ignoreList |= NET::PopupMenuMask;
    ignoreList |= NET::NotificationMask;

    KWindowInfo info(window, NET::WMWindowType | NET::WMState, NET::WM2TransientFor);
    if (!info.valid())
        return false;

    if (NET::typeMatchesMask(info.windowType(NET::AllTypesMask), ignoreList))
        return false;

    if (info.state() & NET::SkipTaskbar)
        return false;

    // WM_TRANSIENT_FOR hint not set - normal window
    WId transFor = info.transientFor();
    if (transFor == 0 || transFor == window || transFor == (WId) QX11Info::appRootWindow())
        return true;

    info = KWindowInfo(transFor, NET::WMWindowType);

    QFlags<NET::WindowTypeMask> normalFlag;
    normalFlag |= NET::NormalMask;
    normalFlag |= NET::DialogMask;
    normalFlag |= NET::UtilityMask;

    return !NET::typeMatchesMask(info.windowType(NET::AllTypesMask), normalFlag);
}

ThumbnailManager::ThumbnailManager(QObject *parent) : QObject(parent),QAbstractNativeEventFilter(),mCurTime(QDateTime::currentDateTime())
{
    //

    connect(KWindowSystem::self(), &KWindowSystem::windowAdded, this, &ThumbnailManager::onKwinWindowAdd);
    connect(KWindowSystem::self(), &KWindowSystem::windowRemoved, this, &ThumbnailManager::onKwinWindowRemove);

    connect(KWindowSystem::self(),
            static_cast<void (KWindowSystem::*)(WId, NET::Properties, NET::Properties2)>(&KWindowSystem::windowChanged)
            , this, &ThumbnailManager::onKwinWindowChanged);

    mWatcher.addPath(getBackgroundSettingPath());
    connect(&(mWatcher), &QFileSystemWatcher::fileChanged, this, &ThumbnailManager::onBackgroundChanged);
    connect(&(mWatcher), &QFileSystemWatcher::directoryChanged, this, &ThumbnailManager::onBackgroundChanged);

    onBackgroundChanged();

    m_waitforPreviewTimer = new QTimer(this);
    m_waitforPreviewTimer->setSingleShot(true);
    m_waitforPreviewTimer->setInterval(10);
    m_waitforQuitPreviewTimer = new QTimer(this);
    m_waitforQuitPreviewTimer->setSingleShot(true);
    m_waitforQuitPreviewTimer->setInterval(500);

    connect(m_waitforPreviewTimer, &QTimer::timeout, this, &ThumbnailManager::waitforPreviewTimeout);
    connect(m_waitforQuitPreviewTimer, &QTimer::timeout, this, &ThumbnailManager::waitforQuitPreviewTimeout);

    if (QGuiApplication *gui = dynamic_cast<QGuiApplication *>(QCoreApplication::instance())) {
        m_xcb = (gui->platformName() == QLatin1String("xcb"));
        if (m_xcb) {
            gui->installNativeEventFilter(this);
#if HAVE_XCB_COMPOSITE
            xcb_connection_t *c = QX11Info::connection();
            xcb_prefetch_extension_data(c, &xcb_composite_id);
            const auto *compositeReply = xcb_get_extension_data(c, &xcb_composite_id);
            m_composite = (compositeReply && compositeReply->present);

            xcb_prefetch_extension_data(c, &xcb_damage_id);
            const auto *reply = xcb_get_extension_data(c, &xcb_damage_id);
            m_damageEventBase = reply->first_event;
            if (reply->present) {
                xcb_damage_query_version_unchecked(c, XCB_DAMAGE_MAJOR_VERSION, XCB_DAMAGE_MINOR_VERSION);
            }
#endif
        }
    }
    m_bPreviewing = false;
}

ThumbnailManager *ThumbnailManager::getIns()
{
    static ThumbnailManager tm;
    return &tm;
}

QPixmap ThumbnailManager::getThumbnail(WId id)
{
    QPixmap pixmap;
    auto it = m_thumbnails.find(id);
    if(m_thumbnails.contains(id) && it != m_thumbnails.end())
    {
        if(it.value())
        {
            pixmap = QPixmap::fromImage(qimageFromXImage((XImage*)it.value()));
        }
        else {
            pixmap = getWallPaper();
        }
    }else {
        //这里如果获取不到，就用背景替代
        pixmap = getWallPaper();
    }
    return pixmap;
}

QPixmap ThumbnailManager::getWallPaper()
{
    return m_wallPaper;
}

QPixmap ThumbnailManager::getAdjustWallPaper()
{
    return m_adjustWallPaper;
}

void ThumbnailManager::onKwinWindowAdd(WId id)
{
    updateThumbnail(id);
    Q_EMIT windowAdded(id);
}

void ThumbnailManager::onKwinWindowRemove(WId id)
{
    QMap<WId, QDateTime>::iterator it;
    it = m_freReduc.find(id);
    if (it != m_freReduc.end()){
        m_freReduc.erase(it);
    }

    removeThumbnail(id);
    Q_EMIT windowRemoved(id);
}

void ThumbnailManager::onKwinWindowChanged(WId id, NET::Properties p1, NET::Properties2 p2)
{
//    do{
//        if(p1 & NET::WMState)
//        {
//            KWindowInfo info(id, NET::WMState);
//            if(!(info.state() & NET::Hidden)) //如果从最小化恢复直接截图
//            {
//                break;
//            }
//        }
//        QDateTime now = QDateTime::currentDateTime();
//        if (m_freReduc.contains(id))
//        {
//            QDateTime last = m_freReduc.value(id);
//            if (last.msecsTo(now) < 200)
//            {
//                Q_EMIT windowChanged(id,p1,p2);
//                return ;
//            }
//        }
//        m_freReduc.insert(id, now);
//    }while (false);

    //    QThread::msleep(200);

//    QTimer::singleShot(10, [=](){updateThumbnail(id);});
    Q_EMIT windowChanged(id,p1,p2);
}

void ThumbnailManager::onBackgroundChanged()
{
    QString filePath = getBackgroundSettingPath() + "settings.conf";
    QString wallpaper;

    //优先使用配置文件中的配置
    if(QFile::exists(filePath))
    {
        QSettings m_wallPaperSetting(filePath, QSettings::IniFormat, this);
        m_wallPaperSetting.beginGroup("Desktop");
        wallpaper = m_wallPaperSetting.value("Wallpaper", "").toString();
        m_wallPaperSetting.endGroup();
    }
    //其次使用cache文件
    if(!QFile::exists(wallpaper))
    {
        QString cacheFileName = QStandardPaths::writableLocation(QStandardPaths::HomeLocation) % QLatin1String("/.cache");
        cacheFileName += QLatin1String("/nde-fileman/nde/wallpaper.cache");
        if(QFile::exists(cacheFileName))
            wallpaper = cacheFileName;
    }
    //最后使用默认文件
    if(!QFile::exists(wallpaper))
    {
        wallpaper = "/usr/share/backgrounds/default.png";
    }

    if(wallpaper.compare(m_wallPaperPath) == 0) return;

    m_wallPaperPath = wallpaper;
    m_wallPaper.load(wallpaper);
    //第一次开机时，该图片可能读取出来为空，几率很小
    if(m_wallPaper.isNull())
    {
        m_wallPaper.load("/usr/share/backgrounds/default.png");
    }

    QImage image = adjustBlur(m_wallPaper.toImage(), 400);
    image = adjustBrightness(image, -30);
    m_adjustWallPaper = QPixmap::fromImage(image);
    //第一次开机时，该图片可能读取出来为空，几率很小
    if(m_adjustWallPaper.isNull())
    {
        m_adjustWallPaper.load("/usr/share/backgrounds/default-blur.png");
    }
}

void ThumbnailManager::waitforPreviewTimeout()
{
    if(!m_waitforPreviewTimer->isActive())
    {
        m_bPreviewing = true;
        QDBusInterface interface_kwin("org.kde.KWin", "/KWin");

        interface_kwin.call("previewWindows", QVariant::fromValue(QList<uint>({(uint)m_previewWId})));

        if (interface_kwin.lastError().type() == QDBusError::NoError) {
            return;
        }
    }
}

void ThumbnailManager::waitforQuitPreviewTimeout()
{
    updateThumbnail(m_previewWId);
    m_previewWId = 0;
    QDBusInterface interface_kwin("org.kde.KWin", "/KWin");

    interface_kwin.call("quitPreviewWindows");
    m_bPreviewing = false;
    if (interface_kwin.lastError().type() == QDBusError::NoError) {
        return;
    }
}

void ThumbnailManager::removeThumbnail(WId id)
{
    m_thumbnailsLock.lock();
    auto it = m_thumbnails.find(id);
    if(it != m_thumbnails.end())
    {
        if(it.value())
            XDestroyImage((XImage*)it.value());
        m_thumbnails.erase(it);
    }
    m_thumbnailsLock.unlock();
}

void ThumbnailManager::updateThumbnail(WId id)
{
    if(!acceptWindow(id)) return;

    const auto display = QX11Info::display();

    XWindowAttributes attr;
    XGetWindowAttributes(display, id, &attr);

    XImage *ximage = XGetImage(display, id, 0, 0, attr.width, attr.height, ~0, ZPixmap);

//    if(display)
//    {
//        XCloseDisplay(display);
//    }

    if(ximage)
    {
//        QImage qImage = QImage((const uchar*)(ximage->data), ximage->width, ximage->height, ximage->bytes_per_line, QImage::Format_RGB32);
//        QImage thumbnail = qimageFromXImage(ximage);

//        m_thumbnailsLock.lock();
//        m_thumbnails.insert(id, QPixmap::fromImage(thumbnail));
//        m_thumbnailsLock.unlock();

//        XDestroyImage(ximage);
        insertThumbnail(id, ximage);
    }
}

void ThumbnailManager::insertThumbnail(WId id, void *img)
{
    m_thumbnailsLock.lock();
    auto it = m_thumbnails.find(id);
    if(it != m_thumbnails.end())
    {
        if(it.value())
            XDestroyImage((XImage*)it.value());
        m_thumbnails.erase(it);
    }
    m_thumbnails.insert(id, img);
    m_thumbnailsLock.unlock();
}

void *ThumbnailManager::getXimage(WId id)
{
    QMutexLocker locker(&m_thumbnailsLock);
    return m_thumbnails.value(id);
}

QString ThumbnailManager::getBackgroundSettingPath()
{
    QString dirName = QStandardPaths::writableLocation(QStandardPaths::ConfigLocation);
//    dirName += "/nde-fileman/nde/settings.conf";
    dirName += "/nde-fileman/nde/";

    QDir dir(dirName);
    if(!dir.exists())
    {
        dir.mkpath(dirName);
    }

    return dirName;
}

void ThumbnailManager::onDesktopChanged(int desktop)
{
    const auto display = QX11Info::display();

    Window root = DefaultRootWindow(display);
    XWindowAttributes attr;
    XGetWindowAttributes(display, root, &attr);

    XImage *ximage = XGetImage(display, root, 0, 0, attr.width, attr.height, ~0, ZPixmap);

//    if(display)
//    {
//        XCloseDisplay(display);
//    }

    if(ximage)
    {
        QImage thumbnail = qimageFromXImage(ximage);
        m_screenShotLock.lock();
        m_screenShot.insert(desktop, QPixmap::fromImage(thumbnail));
        m_screenShotLock.unlock();
        XDestroyImage(ximage);
    }
}

void ThumbnailManager::onDesktopDelete(int desktop)
{
    m_screenShotLock.lock();
    if (m_screenShot.contains(desktop)){
        m_screenShot.remove(desktop);
    }

    m_screenShotLock.unlock();
}

QPixmap ThumbnailManager::getWallPaper(int desktop)
{
    QMutexLocker locker(&m_screenShotLock);
    QPixmap pix;
    if (m_screenShot.contains(desktop)  && !m_screenShot.value(desktop).isNull()){
        pix = m_screenShot.value(desktop);
    }else{
        pix = getWallPaper();
    }

    return pix;
}

void ThumbnailManager::refreshScreenShot(int deskNum)
{
    m_screenShotLock.lock();
    QMap<int, QPixmap>::iterator iter = m_screenShot.begin();
     while (iter != m_screenShot.end())
     {
         if (iter.key() > deskNum){
             m_screenShot.insert(iter.key()-1, m_screenShot.value(iter.key()));
             iter = m_screenShot.erase(iter);
         }else{
             iter++;
         }
     }
     m_screenShotLock.unlock();
}

void ThumbnailManager::previewWindow(WId id)
{
    if(m_bPreviewing)
        updateThumbnail(m_previewWId);
    m_previewWId = id;
    m_waitforQuitPreviewTimer->stop();
    m_waitforPreviewTimer->start();
}

void ThumbnailManager::quitPreviewWindow(bool force)
{
    if(force)
    {
        waitforQuitPreviewTimeout();
        m_waitforQuitPreviewTimer->stop();
        m_waitforPreviewTimer->stop();
        return;
    }
    m_waitforQuitPreviewTimer->start();
//    m_waitforPreviewTimer->stop();
}

bool ThumbnailManager::nativeEventFilter(const QByteArray &eventType, void *message, long *result)
{
    Q_UNUSED(result)
    if (!m_xcb || !m_composite || eventType != QByteArrayLiteral("xcb_generic_event_t")) {
        // currently we are only interested in XCB events
        return false;
    }
#if HAVE_XCB_COMPOSITE
    xcb_generic_event_t *event = static_cast<xcb_generic_event_t *>(message);
    const uint8_t responseType = event->response_type & ~0x80;
    if (responseType == m_damageEventBase + XCB_DAMAGE_NOTIFY) {
        Q_EMIT windowDamaged(reinterpret_cast<xcb_damage_notify_event_t *>(event)->drawable, XCB_DAMAGE);
        WId id = reinterpret_cast<xcb_damage_notify_event_t *>(event)->drawable;
//        QDateTime now = QDateTime::currentDateTime();
//        if (m_freReduc.contains(id))
//        {
//            QDateTime last = m_freReduc.value(id);
//            if (last.msecsTo(now) < 100)
//            {
//                return false;
//            }
//            updateThumbnail(id);
//            malloc_trim(0);
//        }
//        m_freReduc.insert(id, now);
        if(!m_bPreviewing)
            updateThumbnail(id);
    } else if (responseType == XCB_CONFIGURE_NOTIFY) {
//        Q_EMIT windowDamaged(reinterpret_cast<xcb_configure_notify_event_t *>(event)->window, XCB_CONFIGURE);
    } else if (responseType == XCB_MAP_NOTIFY) {
        Q_EMIT windowDamaged(reinterpret_cast<xcb_configure_notify_event_t *>(event)->window, XCB_MAP);
        //窗口最大化，截图
//        QTimer::singleShot(100, [=](){//延时防止窗口还没渲染完全就截图
//            updateThumbnail(reinterpret_cast<xcb_configure_notify_event_t *>(event)->window);
//        });
//        m_freReduc.insert(reinterpret_cast<xcb_configure_notify_event_t *>(event)->window, QDateTime::currentDateTime());
    }
    else if(responseType == XCB_UNMAP_NOTIFY)
    {
        Q_EMIT windowDamaged(reinterpret_cast<xcb_configure_notify_event_t *>(event)->window, XCB_UNMAP);
//        auto it = m_freReduc.find(reinterpret_cast<xcb_configure_notify_event_t *>(event)->window);
//        if(it != m_freReduc.end())
//            m_freReduc.erase(it);//最小化取消计时
    }
#else
    Q_UNUSED(message)
#endif
    // do not filter out any events, there might be further WindowThumbnails for the same window
    return false;
}

