#ifndef THUMBNAILMANAGER_H
#define THUMBNAILMANAGER_H

#include <QObject>
#include <KWindowSystem>
#include <QPixmap>
#include <QMap>
#include <QMutex>
#include <QSettings>
#include <QFileSystemWatcher>
#include <QDateTime>
#include <QDir>
#include <QAbstractNativeEventFilter>

QImage qimageFromXImage(void* __image);

class ThumbnailManager : public QObject, public QAbstractNativeEventFilter
{
    Q_OBJECT
private:
    explicit ThumbnailManager(QObject *parent = nullptr);

public:
    typedef enum EventType
    {
        XCB_DAMAGE = 0x00,
        XCB_CONFIGURE = 0x01,
        XCB_MAP = 0x02, //最小化
        XCB_UNMAP = 0x03 //最大化
    }EventType;
    static ThumbnailManager * getIns();

    QPixmap getThumbnail(WId id);
    void* getXimage(WId id);

    QPixmap getWallPaper();
    QPixmap getWallPaper(int desktop);

    //获取模糊处理之后的桌面壁纸
    QPixmap getAdjustWallPaper();

    void refreshScreenShot(int deskNum);

    void previewWindow(WId id);
    void quitPreviewWindow(bool force = false);

protected:
    bool nativeEventFilter(const QByteArray &eventType, void *message, long int *result) override;

Q_SIGNALS:
    void windowAdded(WId id);
    void windowRemoved(WId id);
    void windowChanged(WId window, NET::Properties prop, NET::Properties2 prop2);
    void windowDamaged(WId id, const uint8_t responseType);
public Q_SLOTS:
    void onKwinWindowAdd(WId id);
    void onKwinWindowRemove(WId id);
    void onKwinWindowChanged(WId id, NET::Properties p1, NET::Properties2 p2);
    void onDesktopChanged(int desktop);
    void onDesktopDelete(int desktop);

private Q_SLOTS:
    void onBackgroundChanged();

    void waitforPreviewTimeout();
    void waitforQuitPreviewTimeout();

private:
    void removeThumbnail(WId id);
    void updateThumbnail(WId id);
    void insertThumbnail(WId id, void* img);
    QString getBackgroundSettingPath();

private:
    QFileSystemWatcher mWatcher;
    QMutex m_thumbnailsLock;
    QMutex m_screenShotLock;
    QMap<WId, void*> m_thumbnails; //需要渲染时才图像处理转换格式
    QMap<int, QPixmap> m_screenShot;

    QString m_wallPaperPath;
    QPixmap m_wallPaper;
    QPixmap m_adjustWallPaper;
    QDateTime mCurTime;
    QMap<WId, QDateTime> m_freReduc;

    QTimer* m_waitforPreviewTimer;
    QTimer* m_waitforQuitPreviewTimer;
    WId m_previewWId;
    bool m_bPreviewing;

private:
    bool m_xcb;
    bool m_composite;
    uint8_t m_damageEventBase;
};
#endif // THUMBNAILMANAGER_H
