#ifndef DESKTOPITEM_H
#define DESKTOPITEM_H


#include <QObject>
#include <QPoint>
#include <QHash>
#include <QtWidgets>
#include <QWindow>
#include <KF5/KWindowSystem/KWindowSystem>
#include <QDebug>
#include <QtDBus/QDBusConnection>
#include <QtDBus/QDBusMessage>
#include <QtDBus/QDBusArgument>
#include <QtDBus/QDBusUnixFileDescriptor>
#include "../panel/thumbnailmanager.h"

namespace NewStart {

bool variantToString(const QVariant &arg, QString &out);
bool argToString(const QDBusArgument &busArg, QString &out);
QString argumentToString(const QVariant &arg);


class windowitem : public QObject
{
    Q_OBJECT
public:
    windowitem();
    WId window() { return window_id; }
    void setwindow(WId id);
    QImage desktopimg() {
        return m_desktopimg;
    }
    QImage workplaceimg() {
        return m_workplaceimg;
    }
    void settitle(QString str){
        m_title = str;
    }
    QString title(){
        return m_title;
    }

private:
    WId window_id;
    QImage m_workplaceimg;
    QImage m_desktopimg;
    QString m_title;

};


class desktopitem: public QObject
{
    Q_OBJECT
public:
    desktopitem();
    void initdesktop();
    int desktop() const { return cur_desktop; }
    void setDesktop(int d);
    void deletewindow(WId id,bool close = false,bool refresh = true);
    void addwindow(QSharedPointer<windowitem> item);
    void settitle(QString name);
    QSharedPointer<windowitem> findwindowbywid(WId id);
    QMap<WId,QSharedPointer<windowitem>> getwindowslist(){
        return m_windowlist;
    }

Q_SIGNALS:
    void desktopChanged();
    void windowsChanged(int);
    void backgroundadd(WId);
    void backgrounddel(WId);
    void rectChanged();
    void titleChanged();

private Q_SLOTS:
    void removewindow(WId id) { deletewindow(id,true); }
private:
    int cur_desktop {0};
    QPixmap m_bgimg;
    QRect m_rect;
    QString m_desktopname;
    QMap<WId,QSharedPointer<windowitem>> m_windowlist;
};

//class ImageManager : public QObject
//{
//    Q_OBJECT
//public:
//    static ImageManager& instance(){
//        static ImageManager* _self = nullptr;
//        if (!_self) {
//            _self = new ImageManager();
//        }
//        return *_self;
//    }

//    void addImage(WId id) {
//        QImage img;
//        bool result = BackgroundManager::instance().getwindowimage(id,img);
//        if (result){
//            auto iter = m_imagelist.find(id);
//            if(iter != m_imagelist.end()){
//                m_imagelist.remove(id);
//            }

//            m_imagelist.insert(id,img);
//        }

//    }
//    void addImage(WId id,QImage img) {
//        auto iter = m_imagelist.find(id);
//        if(iter != m_imagelist.end()){
//            m_imagelist.remove(id);
//        }
//        m_imagelist.insert(id,img);
//    }
//    bool getImage(WId id , QImage & img){
//        auto iter = m_imagelist.find(id);
//        if(iter != m_imagelist.end()){
//            img = iter.value();
//            return true;
//        }
//        return false;
//    }

//    void removeImage(WId id){
//        auto iter = m_imagelist.find(id);
//        if(iter != m_imagelist.end()){
//            m_imagelist.remove(id);
//        }
//    }

//private:
//    ImageManager() {
//    }

//private:
//    QMap<int,QImage> m_imagelist;
//};
class desktopmanager : public QObject
{
    Q_OBJECT
public:
    static desktopmanager& instance(){
        static desktopmanager* _self = nullptr;
        if (!_self) {
            _self = new desktopmanager();
        }
        return *_self;
    }
    int currentdesktop() {
        return KWindowSystem::currentDesktop();
    }
    int desktopnum() {
        return m_managerlist.count();
    }
    void Init();
    void clear() {
        m_managerlist.clear();
    }
    int getSelectedDesktop(){
        return selectdesktopnum;
    }
    void setSelectedDesktop(int num){
        selectdesktopnum = num;
    }

    void movewindowfromto(WId id,int from ,int to );
    QSharedPointer<desktopitem> getdesktopbyid(int id);
    QMap<int,QSharedPointer<desktopitem>> getdesktops() {
        return m_managerlist;
    }

Q_SIGNALS:
    void Refresh();

private Q_SLOTS:
    void onNumberOfDesktopsChanged(int id){
        clear();
        Init();
        Q_EMIT Refresh();
    }
//    void windowAdded(WId id){ ImageManager::instance().addImage(id);}
//    void windowRemoved(WId id){ImageManager::instance().removeImage(id);}
    void desktopNamesChanged(){}
    void deletedesktop(int id);
    void removeitem(WId id,bool desktop);
    void adddesktop();


private:
    desktopmanager() {
        Init();
    }

private:
    QMap<int,QSharedPointer<desktopitem>> m_managerlist;
    int selectdesktopnum;
};

class KWinOpt : public QObject
{
    Q_OBJECT
public:
    static KWinOpt& instance(){
        static KWinOpt* _self = nullptr;
        if (!_self) {
            _self = new KWinOpt();
        }
        return *_self;
    }
    bool removewindow(WId id);
    bool removedesktop(int id);
    int createdesktop();
    int getdesktopnum();
private Q_SLOTS:
    void dumpMessage(const QDBusMessage &message);
private:
    KWinOpt() : objectPathRegExp(QLatin1String("\\[ObjectPath: (.*)\\]")) {}
    QRegExp objectPathRegExp;
};

}
#endif // DESKTOPITEM_H
