#include "desktopscreen.h"
#include "ui_desktopscreen.h"
#include "desktopitem.h"
#include <QPainterPath>

DesktopScreen* selectdesktop = nullptr;

DesktopScreen::DesktopScreen(QWidget *parent,int id ) :
    QWidget(parent),
    ui(new Ui::DesktopScreen),
    m_selected(false),
    m_desktop(id)
{
    ui->setupUi(this);
    setAcceptDrops(true);

    ui->image->setFixedSize(PREVIEWSIZE);
    ui->image->setAlignment( Qt::AlignCenter);
}

DesktopScreen::~DesktopScreen()
{
    if (selectdesktop) {
        selectdesktop = nullptr;
    }

    delete ui;
}
void DesktopScreen::setPixmap(const QPixmap pixmap)
{
    m_pixmap = pixmap;
//    ui->image->setPixmap(pixmap.scaled(ui->image->size(), Qt::IgnoreAspectRatio,Qt::SmoothTransformation));
}

void DesktopScreen::setselected()
{
    if(selectdesktop) {
        if (this == selectdesktop) {
            return;
        }
        selectdesktop->cancleseelect();
    }
    selectdesktop = this;
    m_selected = true;
    setProperty("Select", true);
    style()->polish(this);
    style()->polish(ui->image);
    update();

    NewStart::desktopmanager::instance().setSelectedDesktop(m_desktop);
    Q_EMIT desktopChanged(m_desktop);
//    repaint();
}
void DesktopScreen::cancleseelect()
{
    if (this == selectdesktop) {
        selectdesktop = nullptr;
    }

    m_selected = false;

    setProperty("Select",false);
    style()->polish(this);
    style()->polish(ui->image);
    update();

}

void DesktopScreen::initwindowspic()
{
    QSize desktopsize(QApplication::desktop()->screenGeometry().width(), QApplication::desktop()->screenGeometry().height());
    auto windowlist = NewStart::desktopmanager::instance().getdesktopbyid(m_desktop)->getwindowslist();
    for( auto it=windowlist.constBegin(); it!=windowlist.constEnd(); ++it)
    {
        QPixmap pic = QPixmap::fromImage(it.value()->desktopimg());
        if(pic.isNull()) continue;

        int width = pic.width() * ICON_WIDTH / desktopsize.width() ;
        int height = width * pic.height() / pic.width();

        QSize tsize(width,height);
        m_piclist.insert(it.key(),pic.scaled(tsize,Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
    }
    updatebackground();
}



void DesktopScreen::setText(const QString text)
{
    m_title = text;
    ui->title->setText(m_title);
}

void DesktopScreen::backgrounddel(WId wid) {
    auto iter = m_piclist.find(wid);
    if(iter != m_piclist.end()){
        m_piclist.remove(wid);
    }
    updatebackground();
//    repaint();

}

void DesktopScreen::backgroundadd(WId wid)
{

    QSize desktopsize(QApplication::desktop()->screenGeometry().width(), QApplication::desktop()->screenGeometry().height());
    auto window = NewStart::desktopmanager::instance().getdesktopbyid(m_desktop)->findwindowbywid(wid);

    QPixmap pic = QPixmap::fromImage(window->desktopimg());
    int width = pic.width() * ICON_WIDTH / desktopsize.width() ;
    int height = width * pic.height() / pic.width();
    QSize tsize(width,height);
    m_piclist.insert(wid,pic.scaled(tsize,Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
    updatebackground();
//    repaint();
}

void DesktopScreen::mouseDoubleClickEvent(QMouseEvent *event)
{
    Q_EMIT desktopselectd(m_desktop);
}

void DesktopScreen::mousePressEvent(QMouseEvent *event)
{
    if(selectdesktop) {
        if (this == selectdesktop) {
            return;
        }
        selectdesktop->cancleseelect();
    }
    selectdesktop = this;
    m_selected = true;
    setProperty("Select",true);
    style()->polish(this);
    style()->polish(ui->image);
    update();

    NewStart::desktopmanager::instance().setSelectedDesktop(m_desktop);
    Q_EMIT desktopChanged(m_desktop);
}

void DesktopScreen::mouseReleaseEvent(QMouseEvent *event)
{
    //Q_EMIT desktopselectd(m_desktop);
}

void DesktopScreen::dragEnterEvent(QDragEnterEvent  * e){
    //处理是否接受拖动事件。
    e->accept();  			//接受拖动进入事件
 }

 void DesktopScreen::dropEvent(QDropEvent  * e){
    //处理拖动中的数据
    QString str = e->mimeData()->text();
    QStringList list = str.split(",");
    QString a = list[0]; //desktop
    QString b = list[1]; //window

    int tmpdesktop = a.toInt();
    WId temwindow = b.toUInt();

    if (tmpdesktop == m_desktop) {
        e->setDropAction(Qt::IgnoreAction);
        e->accept();
        return ;
    }
    e->setDropAction(Qt::MoveAction);
    e->accept();
    NewStart::desktopmanager::instance().movewindowfromto(temwindow,tmpdesktop,m_desktop);
 }

void DesktopScreen::enterEvent(QEvent *)                      //进入QWidget瞬间事件
{
    shadow = false;
    //Q_EMIT mouseenter();
    setProperty("Hover", true);
    style()->polish(this);
    style()->polish(ui->image);
    update();
}
void DesktopScreen::leaveEvent(QEvent *)                      //离开QWidget瞬间事件
{
    shadow = true;
    setProperty("Hover", false);
    style()->polish(this);
    style()->polish(ui->image);
    update();
}

QRect DesktopScreen::getPixmapRect()
{
    QRect rc = this->rect();
    QRect imageRc = rc.adjusted(0,TEXT_HEIGHT,0,0);
    return imageRc;
}

QRect DesktopScreen::getpicRect()
{
    QRect rc = this->rect();
    QRect imageRc(startx,starty,ICON_WIDTH,ICON_LENGTH);
    startx = startx + 20;
    if (startx > 70) { startx = 30; }
    starty = starty + 20;
    if (starty > 70) { startx = 30; }
    return imageRc;
}

QRect DesktopScreen::getTextRect()
{
    QRect rc = this->rect();
    QRect textrc(4,2,rc.width()-10,TEXT_HEIGHT);
    return textrc;
}

void DesktopScreen::updatebackground()
{
    startx = 0;
    starty = 0;

    QPainterPath path;
    QImage image(PREVIEWSIZE,QImage::Format_ARGB32);
    //QPixmap image(PREVIEWSIZE);
    image.fill(Qt::transparent);
    QPainter painter(&image);
    QRect previewRect = ui->image->rect();
    previewRect.adjust(1,1,-1,-1);

    // 抗锯齿
    painter.setRenderHints(QPainter::Antialiasing, true);
    // 图片平滑处理
    painter.setRenderHints(QPainter::SmoothPixmapTransform, true);
    path.addRoundedRect(previewRect, 4, 4);
    painter.setClipPath(path);

    painter.drawImage(0,0,m_pixmap.toImage());

    QPixmap qb;
    qb = QPixmap::fromImage(image);
    ui->image->setPixmap(qb.scaled(ui->image->width(), ui->image->height(), Qt::KeepAspectRatio, Qt::SmoothTransformation));
}

void DesktopScreen::paintEvent(QPaintEvent *event)
{
    QPainter p(this);
    QStyleOption opt;
    opt.initFrom(this);
    style()->drawPrimitive(QStyle::PE_Widget, &opt, &p, this);
}

void DesktopScreen::on_pushButton_clicked()
{
    Q_EMIT desktopdelete(m_desktop);
}

void DesktopScreen::hideCloseBtn()
{
    ui->pushButton->hide();
}
