/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)LGPL2+
 *
 * Nde - a lightweight, Qt based, desktop toolset
 * https://lxqt.org
 *
 * Copyright: 2011 Razor team
 * Authors:
 *   Petr Vanek <petr@scribus.info>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include <QButtonGroup>
#include <QWheelEvent>
#include <QtDebug>
#include <QSignalMapper>
#include <QTimer>
#include <nde-globalkeys.h>
#include <QHBoxLayout>
#include <Nde/GridLayout>
#include <KWindowSystem/KWindowSystem>
#include <QX11Info>
#include <QLocale>
#include <cmath>
#include "../panel/indepanelplugin.h"
#include "desktopswitch.h"
#include "desktopswitchbutton.h"
#include "desktopswitchconfiguration.h"
#include "desktopitem.h"
#include "desktopwidget.h"

static const QString DEFAULT_SHORTCUT_TEMPLATE("Control+Alt+Tab");
static const QString DEFAULT_SHORTCUT_MULTI_VIEW("Meta+Tab"); //multitask view
static const QString DEFAULT_SHORTCUT_SWITCHDESKTOP("Control+Alt+W");

DesktopSwitch::DesktopSwitch(const INdePanelPluginStartupInfo &startupInfo) :
    QObject(),
    INdePanelPlugin(startupInfo),
    mVisible(true)
{

    mWidget = new DesktopSwitchWidget(this);

    //settingsChanged();
    connect(KWindowSystem::self(), SIGNAL(numberOfDesktopsChanged(int)), &NewStart::desktopmanager::instance(),SLOT(onNumberOfDesktopsChanged(int)));
    connect(KWindowSystem::self(), SIGNAL(desktopNamesChanged()), &NewStart::desktopmanager::instance(), SLOT(desktopNamesChanged()));
//    connect(KWindowSystem::self(), SIGNAL(windowAdded(WId)), &NewStart::desktopmanager::instance(), SLOT(windowAdded(WId)));
//    connect(KWindowSystem::self(), SIGNAL(windowRemoved(WId)), &NewStart::desktopmanager::instance(), SLOT(windowRemoved(WId)));
//    connect(KWindowSystem::self(), SIGNAL(activeWindowChanged(WId)), &NewStart::desktopmanager::instance(), SLOT(windowAdded(WId)));
//    connect(KWindowSystem::self(), SIGNAL(windowChanged(WId)), &NewStart::desktopmanager::instance(), SLOT(windowAdded(WId)));

    QTimer::singleShot(0, this, SLOT(registerShortcuts()));
    QTimer::singleShot(0,[this](){settingsChanged();});
}
void DesktopSwitch::registerShortcuts()
{
    // Register shortcuts to change desktop

    QString path;
    QString description;
    QString args =  settings()->group();
    mShortcut = GlobalKeyShortcut::Client::instance()->addAction(QString{}, QString("/panel/%1/%2").arg(args).arg("multi_view1"), tr("Show multiview"), this);
    if (mShortcut)
    {
        connect(mShortcut, &GlobalKeyShortcut::Action::registrationFinished, [this] {
            if (mShortcut->shortcut().isEmpty())
                mShortcut->changeShortcut(DEFAULT_SHORTCUT_TEMPLATE);
        });
        connect(mShortcut, &GlobalKeyShortcut::Action::activated, [this] {
            mWidget->setDesktop();
        });
    }
    m_mulit_view_Shortcut = GlobalKeyShortcut::Client::instance()->addAction(QString{}, QString("/panel/%1/%2").arg(args).arg("multi_view2"), tr("multitask view"), this);
    if(m_mulit_view_Shortcut)
    {
        connect(m_mulit_view_Shortcut, &GlobalKeyShortcut::Action::registrationFinished, [this] {
            if(m_mulit_view_Shortcut->shortcut().isEmpty())
                m_mulit_view_Shortcut->changeShortcut(DEFAULT_SHORTCUT_MULTI_VIEW);
        });
        connect(m_mulit_view_Shortcut, &GlobalKeyShortcut::Action::activated, [this] {
            mWidget->setDesktop();
        });
    }
    m_switchdesktop_Shortcut = GlobalKeyShortcut::Client::instance()->addAction(QString{}, QString("/panel/%1/%2").arg(args).arg("multi_view3"), tr("switch desktop"), this);
    if (m_switchdesktop_Shortcut)
    {
        connect(m_switchdesktop_Shortcut, &GlobalKeyShortcut::Action::registrationFinished, [this] {
            if (m_switchdesktop_Shortcut->shortcut().isEmpty())
                m_switchdesktop_Shortcut->changeShortcut(DEFAULT_SHORTCUT_SWITCHDESKTOP);
        });
        connect(m_switchdesktop_Shortcut, &GlobalKeyShortcut::Action::activated, [this] {
            nextdesktop();
        });
    }
}

void DesktopSwitch::shortcutRegistered()
{
    GlobalKeyShortcut::Action * const shortcut = qobject_cast<GlobalKeyShortcut::Action*>(sender());

    disconnect(shortcut, &GlobalKeyShortcut::Action::registrationFinished, this, &DesktopSwitch::shortcutRegistered);

    const int i = m_keys.indexOf(shortcut);
    Q_ASSERT(-1 != i);

    if (shortcut->shortcut().isEmpty())
    {
        shortcut->changeShortcut(DEFAULT_SHORTCUT_TEMPLATE.arg(i + 1));
    }
}

void DesktopSwitch::nextdesktop(){
    int number = KWindowSystem::numberOfDesktops();
    int curdesktp = KWindowSystem::currentDesktop();
    if(curdesktp < number)
    {
        KWindowSystem::setCurrentDesktop(curdesktp+1);
        NewStart::desktopmanager::instance().setSelectedDesktop(curdesktp+1);
    }
    else {
        KWindowSystem::setCurrentDesktop(1);
        NewStart::desktopmanager::instance().setSelectedDesktop(1);
    }
}

bool DesktopSwitch::isWindowHighlightable(WId window)
{
    QFlags<NET::WindowTypeMask> ignoreList;
    ignoreList |= NET::DesktopMask;
    ignoreList |= NET::DockMask;
    ignoreList |= NET::SplashMask;
    ignoreList |= NET::ToolbarMask;
    ignoreList |= NET::MenuMask;
    ignoreList |= NET::PopupMenuMask;
    ignoreList |= NET::NotificationMask;

    KWindowInfo info(window, NET::WMWindowType | NET::WMState, NET::WM2TransientFor);
    if (!info.valid())
        return false;

    if (NET::typeMatchesMask(info.windowType(NET::AllTypesMask), ignoreList))
        return false;

    if (info.state() & NET::SkipTaskbar)
        return false;

    // WM_TRANSIENT_FOR hint not set - normal window
    WId transFor = info.transientFor();
    if (transFor == 0 || transFor == window || transFor == (WId) QX11Info::appRootWindow())
        return true;

    info = KWindowInfo(transFor, NET::WMWindowType);

    QFlags<NET::WindowTypeMask> normalFlag;
    normalFlag |= NET::NormalMask;
    normalFlag |= NET::DialogMask;
    normalFlag |= NET::UtilityMask;

    return !NET::typeMatchesMask(info.windowType(NET::AllTypesMask), normalFlag);
}

DesktopSwitch::~DesktopSwitch()
{
}

void DesktopSwitch::realign()
{
    mWidget->realign();
}

QDialog *DesktopSwitch::configureDialog()
{
    return new DesktopSwitchConfiguration(settings());
}

void DesktopSwitch::settingsChanged()
{
    bool visible = this->settings()->value("visible","true").toBool();
    if(mVisible != visible)
    {
        mWidget->setVisible(visible);
        mVisible = visible;
    }
}

DesktopSwitchWidget::DesktopSwitchWidget(INdePanelPlugin *plugin):
    QFrame(),
    mPlugin(plugin)
{
    QHBoxLayout *layout = new QHBoxLayout(this);
    layout->setContentsMargins (0, 0, 0, 0);
    layout->setAlignment(Qt::AlignCenter);
    desktopwidget = new desktopWidget(plugin);
    desktopwidget->hide();
    multiview = new QToolButton();
    //multiview->setFixedSize(32,32);
    QAction * act = new QAction(this);
    act->setToolTip(tr("Multitask View"));

    connect(act, SIGNAL(triggered()), this, SLOT(setDesktop()));
    multiview->setDefaultAction(act);
    multiview->setObjectName("multiview");
    multiview->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    multiview->setAutoRaise(true);
    layout->addWidget(multiview);
}

void DesktopSwitchWidget::realign()
{
    desktopwidget->realign();
}

void DesktopSwitchWidget::setDesktop()
{
   if (!desktopwidget->isVisible()){
       NewStart::desktopmanager::instance().Init();
       desktopwidget->setbackground();
       desktopwidget->setdesktopitem();
       desktopwidget->setFocus();
       desktopwidget->setVisible(true);
       desktopwidget->activateWindow();
   }
   else{
       desktopwidget->clearFocus();
       desktopwidget->setVisible(false);
   }

}
