/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)LGPL2+
 *
 * Nde - a lightweight, Qt based, desktop toolset
 * https://lxqt.org
 *
 * Copyright: 2011 Razor team
 * Authors:
 *   Petr Vanek <petr@scribus.info>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */


#ifndef DESKTOPSWITCH_H
#define DESKTOPSWITCH_H

#include "../panel/indepanelplugin.h"
#include <QFrame>
#include <QScopedPointer>
#include <KWindowSystem/NETWM>

#include "desktopswitchbutton.h"
#include "desktopwidget.h"
class QSignalMapper;
class QButtonGroup;
class NETRootInfo;
namespace Nde {
class GridLayout;
}

class DesktopSwitchWidget: public QFrame
{
    Q_OBJECT
public:
    DesktopSwitchWidget(INdePanelPlugin * plugin);

    void realign();
public Q_SLOTS:
    void setDesktop();
private:
    INdePanelPlugin *mPlugin;
    QToolButton * multiview;
    desktopWidget * desktopwidget;
};

/**
 * @brief Desktop switcher. A very simple one...
 */
class DesktopSwitch : public QObject, public INdePanelPlugin
{
    Q_OBJECT
public:
    DesktopSwitch(const INdePanelPluginStartupInfo &startupInfo);
    ~DesktopSwitch();

    QString themeId() const { return "DesktopSwitch"; }
    QWidget *widget() { return mWidget; }
    virtual void realign();

    bool isSeparate() const { return true; }

    virtual INdePanelPlugin::Flags flags() const { return SingleInstance|NeedsHandle; }
    QDialog *configureDialog();

    void nextdesktop();

    //setting change
    virtual void settingsChanged();
private:
    QList<GlobalKeyShortcut::Action*> m_keys;
    GlobalKeyShortcut::Action * mShortcut;
    GlobalKeyShortcut::Action * m_mulit_view_Shortcut;
    GlobalKeyShortcut::Action * m_switchdesktop_Shortcut;
    QSignalMapper* m_pSignalMapper;
    DesktopSwitchWidget *mWidget;
    DesktopSwitchButton::LabelType mLabelType;
    bool mVisible;

    bool isWindowHighlightable(WId window);

private Q_SLOTS:
    void shortcutRegistered();
    void registerShortcuts();
};


class DesktopSwitchPluginLibrary: public QObject, public INdePanelPluginLibrary
{
    Q_OBJECT
    // Q_PLUGIN_METADATA(IID "lxqt.org/Panel/PluginInterface/3.0")
    Q_INTERFACES(INdePanelPluginLibrary)
public:
    INdePanelPlugin *instance(const INdePanelPluginStartupInfo &startupInfo) const { return new DesktopSwitch(startupInfo);}
};

#endif
