#include "desktopwidget.h"
#include "ui_desktopwidget.h"

#include <QVBoxLayout>
#include <QPushButton>
#include <QDir>
#include <QDebug>
#include <QPainter>
#include <QUrl>
#include <QFile>
#include <XdgIcon>
#include <QApplication>
#include <QDesktopWidget>
#include <QDBusInterface>
#include <nde-globalkeys.h>
#include <QThread>
#include "desktopscreen.h"
#include "adddesktop.h"
//#include "background.h"
#include "desktopswitchbutton.h"
#include "desktopitem.h"

#define DESKTOPHEIGHT 160

desktopWidget::desktopWidget(INdePanelPlugin *plugin, QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::desktopWidget),
      mPlugin(plugin),
      mCompositorOpen(true)
{
    ui->setupUi(this);
    this->setObjectName("DesktopWidget");

    setWindowFlag(Qt::FramelessWindowHint);
    setWindowFlag(Qt::WindowStaysOnTopHint);
    setWindowFlag(Qt::Tool);
    setWindowFlag(Qt::SubWindow);
    setWindowFlag(Qt::Popup);
    setWindowFlag(Qt::X11BypassWindowManagerHint);


    QRect rect = QApplication::desktop()->screenGeometry();
    setGeometry(rect);

    connect(&NewStart::desktopmanager::instance(),SIGNAL(Refresh()),this,SLOT(Refresh()));

    deskLayout = new QVBoxLayout();
    deskLayout->setSizeConstraint(QVBoxLayout::SetMinAndMaxSize);
    ui->desktopScrollAreaWidgetContents->setLayout(deskLayout);

    windowlayout = new QVBoxLayout(ui->windowScrollAreaWidgetContents);
    windowlayout->setSizeConstraint(QVBoxLayout::SetMinAndMaxSize);
    windowlayout->setSpacing(20);
    ui->windowScrollAreaWidgetContents->setLayout(windowlayout);

    ui->windowArea->setWidgetResizable(true);
    ui->windowArea->setAlignment(Qt::AlignCenter);
    ui->windowArea->setFrameShape(QFrame::NoFrame);

    m_scroolbar = ui->windowArea->verticalScrollBar();

    ui->desktopArea->setWidgetResizable(true);
    ui->desktopArea->setAlignment(Qt::AlignCenter);
    ui->desktopArea->setFrameShape(QFrame::NoFrame);

    m_hscroolbar = ui->desktopArea->horizontalScrollBar();

    NewStart::desktopmanager::instance().setSelectedDesktop(KWindowSystem::currentDesktop());
    connect(QApplication::desktop(),SIGNAL(resized(int)),this,SLOT(resizeScreen(int)));

    mCompositorOpen = mPlugin->panel()->compositorOpen();
    ui->desktopArea->setProperty("compositor", mCompositorOpen);

    //Init();
}

bool desktopWidget::event(QEvent * event)
{
    if (event->type() == QEvent::KeyPress){
        QKeyEvent *ke = static_cast<QKeyEvent*>(event);

        if(ke->modifiers() == Qt::ControlModifier){
            if(ke->key() == Qt::Key_Tab){
                selectDesktop(-1);
            }
        } else {
            switch (ke->key())
            {
            case Qt::Key_Left:
                selectWindowleftright(-1);
                break;
            case Qt::Key_Right:
                selectWindowleftright(1);
                break;
            case Qt::Key_Up:
                selectWindowupdown(-1);
                break;
            case Qt::Key_Down:
                selectWindowupdown(1);
                break;
            case Qt::Key_Tab:
                selectDesktop(1);
                break;
            case Qt::Key::Key_Enter:
            case Qt::Key_Return:
                enterWindow();
                break;
            case Qt::Key_Escape:
                this->releaseKeyboard();
                close();
                break;
            }

        }
        return true;

    }
    return QWidget::event(event);
}

void desktopWidget::enterWindow()
{
    if(cur_plubutton){
        cur_plubutton->Select();
        cur_plubutton = nullptr;
        return;
    }
    auto iter = ptr_windowlist.begin();
    while (iter != ptr_windowlist.end()) {
        auto windowlist = iter.value();
        int Count = windowlist.size();
        int i;
        for (i = 0; i<Count; i++){
            if(windowlist[i]->isselect()){
                WId id = windowlist[i]->getWindwonid();
                selectwindows(id);
                return ;
            }
        }
        iter++;
    }
    if (cur_desktop){
        selectdesktop(cur_desktop->getDesktopid());
    }

}

void desktopWidget::InitFirstWindow()
{
    auto iter = ptr_windowlist.begin();
    if (iter != ptr_windowlist.end()) {
        auto windowlist = iter.value();
        int Count = windowlist.size();
        if (Count > 0) {
            windowlist[0]->selectWindow();
        }
    }
}


void desktopWidget::selectDesktop(int direction)
{
    int Count = ptr_desktoplist.size();
    int i = 0;
    if(cur_plubutton){
        i = Count - 1;
    } else {
        for (i = 0; i<Count; i++){
            QString classname = ptr_desktoplist[i]->metaObject()->className();
            if(ptr_desktoplist[i]->metaObject()->className() != QString("DesktopScreen")){
                continue;
            }
            DesktopScreen * ptr = qobject_cast<DesktopScreen*>(ptr_desktoplist[i]);
            if(ptr->isSelected()){
                break;
            }
        }
    }

    int selectid = i + direction;
    if (selectid < 0) {
        selectid = Count-1;
    } else if (selectid >= Count) {
        selectid = 0;
    }
    if(ptr_desktoplist[selectid]->metaObject()->className() == QString("DesktopScreen")){
        DesktopScreen * ptr = qobject_cast<DesktopScreen*>(ptr_desktoplist[selectid]);
        ptr->setselected();
        cur_desktop = ptr;

        if(cur_plubutton){
            cur_plubutton->cancleSelect();
            cur_plubutton = nullptr;
        }
    } else if (ptr_desktoplist[selectid]->metaObject()->className() == QString("AddDesktop")){
        drawwindow(0);
        AddDesktop * ptr = qobject_cast<AddDesktop*>(ptr_desktoplist[selectid]);
        ptr->setSelect();
        cur_plubutton = ptr;
        DesktopScreen * dptr = qobject_cast<DesktopScreen*>(ptr_desktoplist[i]);
        dptr->cancleseelect();
    }

    int value;
    if (selectid == Count-1){
        value = m_maxhbar;
    } else {
        value = (selectid-1) / ((float)Count ) * m_maxhbar;
    }
    m_hscroolbar->setValue(value);
}

void desktopWidget::selectWindowupdown(int direction)
{
    int rowcount = ptr_windowlist.size();

    auto iter = ptr_windowlist.begin();
    int row = 1;
    while (iter != ptr_windowlist.end()) {
        auto windowlist = iter.value();
        int Count = windowlist.size();
        int i;
        for (i = 0; i<Count; i++){
            if(windowlist[i]->isselect()){
                if ((direction < 0) && (iter == ptr_windowlist.begin())){
                    return;
                }
                iter = iter + direction;
                if(direction > 0) {
                    row++;
                } else {
                    row--;
                }
                if(iter != ptr_windowlist.end()){
                    auto windowlist = iter.value();
                    int size = windowlist.size();
                    if (i >= size){
                        i = size-1;
                    }
                    int selectid = i;
                    if ((selectid < 0) ||(selectid >=Count)) {
                        return;
                    }
                    windowlist[selectid]->selectWindow();
                    int value = (row-1) / ((float)rowcount-1 ) * m_maxbar;
                    m_scroolbar->setValue(value);
                    return ;
                } else {
                    return;
                }
            }
        }
        iter++;
        row++;
    }
}
void desktopWidget::selectWindowleftright(int direction)
{
    auto iter = ptr_windowlist.begin();
    int row = 1;
    int rowcount = ptr_windowlist.size();
    while (iter != ptr_windowlist.end()) {
        auto windowlist = iter.value();
        int Count = windowlist.size();
        int i;
        for (i = 0; i<Count; i++){
            if(windowlist[i]->isselect()){
                int selectid = i + direction;
                if ((selectid < 0) ||(selectid >=Count)) {
                    if (selectid < 0) {
                        if(iter == ptr_windowlist.begin()){
                            return;
                        }
                        iter--;
                        row--;
                        windowlist = iter.value();
                        selectid = windowlist.size()-1;
                    } else {
                        iter++;
                        row++;
                        if(iter == ptr_windowlist.end()){
                            return;
                        }
                        windowlist = iter.value();
                        selectid = 0;
                    }
                }
                windowlist[selectid]->selectWindow();
                if (rowcount > 1){
                    int value = (row-1) / ((float)rowcount-1 ) * m_maxbar;
                    m_scroolbar->setValue(value);
                }

                return ;
            }
        }
        iter++;
        row++;
    }
}

void desktopWidget::Init()
{
    //    LXQt::Settings *setting =new LXQt::Settings(QLatin1String("panel"),this);
    //    setting->beginGroup("panel1");
    //    QColor color = setting->value("background-color").value<QColor>();
    //    int opacity = setting->value("opacity").toInt();
    //    QString colors = QString("%1, %2, %3, %4")
    //        .arg(color.red())
    //        .arg(color.green())
    //        .arg(color.blue())
    //        .arg(opacity / 100.0);
    //    qInfo()<<"color:" <<colors;
    //    setStyleSheet(QString("background-color:rgba(" + colors + ");"));
}

void desktopWidget::Refresh()
{
    InitSize();
    //getWindowsize();
    drawdesktop();
    drawwindow(NewStart::desktopmanager::instance().getSelectedDesktop());
}

void desktopWidget::resizeScreen(int)
{
    InitSize();
    //getWindowsize();
    drawdesktop();
    drawwindow(NewStart::desktopmanager::instance().getSelectedDesktop());
}

void desktopWidget::setbackground()
{
    QSize desktopsize(QApplication::desktop()->screenGeometry().width(), QApplication::desktop()->screenGeometry().height());
    QPixmap m_pixmap = ThumbnailManager::getIns()->getAdjustWallPaper();
    m_pixmap.scaled(desktopsize, Qt::KeepAspectRatioByExpanding,Qt::SmoothTransformation);
    if (!m_pixmap.isNull())
    {
        if (m_pixmap.size().width() > desktopsize.width()
                || m_pixmap.size().height() > desktopsize.height())
        {
            m_pixmap = m_pixmap.scaled(desktopsize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
        }
        QPalette pal = this->palette();
        pal.setBrush(QPalette::Background,QBrush(m_pixmap.scaled(this->size())));
        setPalette(pal);
    }
}

void desktopWidget::selectwindows(WId id)
{
    KWindowSystem::activateWindow(id);
    this->releaseKeyboard();
    close();
}

void desktopWidget::selectdesktop(int id)
{
    KWindowSystem::setCurrentDesktop(id);
    this->releaseKeyboard();
    close();
    QThread::msleep(100);
    ThumbnailManager::getIns()->onDesktopChanged(id);
}
void desktopWidget::addShadowEffect(QWidget* widget)
{
    QGraphicsDropShadowEffect *shadow_effect = new QGraphicsDropShadowEffect(this);
    shadow_effect->setOffset(0, 0);
    QColor color = QColor("#000000");
    color.setAlphaF(0.3);
    shadow_effect->setColor(color);
    shadow_effect->setBlurRadius(5);
    widget->setGraphicsEffect(shadow_effect);
}
void desktopWidget::drawwindow(int cur_desktop)
{
//    windowlayout->setEnabled(false);
    if (cur_desktop == 0){
        if(cur_plubutton){
            cur_plubutton->cancleSelect();
            cur_plubutton = nullptr;
        }
    }


    clearLayout(windowlayout);
    ptr_windowlist.clear();
    QHBoxLayout * lay = new QHBoxLayout();
    twindowlist tlist;
    lay->addSpacing(20);

    QSize size = this->size();
    int iwidth = 0;
    int i = 0;
    auto iter = NewStart::desktopmanager::instance().getdesktopbyid(cur_desktop);
    if(iter){
        auto windowlist = iter->getwindowslist();
        for( auto it=windowlist.constBegin(); it!=windowlist.constEnd(); ++it)
        {
            auto item = it.value();
            WindowScreen * window = new WindowScreen(it.key(),iter->desktop());
            connect(window,SIGNAL(activewindow(WId)),this,SLOT(selectwindows(WId)));
            connect(window,SIGNAL(deletewindow(WId)),iter.data(),SLOT(removewindow(WId)));
            window->setpixmap(QPixmap::fromImage(item->workplaceimg()),item->window(),item->title());

            if(iwidth + window->size().width() >= size.width()){
                iwidth = window->size().width();
                windowlayout->addItem(lay);
                ptr_windowlist.insert(i,tlist);
                i++;
                tlist.clear();
                lay = new QHBoxLayout();
                lay->addSpacing(20);

            } else {
                iwidth = iwidth + window->size().width()+20;
            }
            tlist.push_back(window);
            lay->addWidget(window);
            lay->addSpacing(20);
        }
    }

    windowlayout->addItem(lay);
    ptr_windowlist.insert(i,tlist);
    QWidget * widget = new QWidget();
    widget->setWindowOpacity(0.0);
    //widget->setAttribute(Qt::WA_TranslucentBackground);
    widget->setLayout(windowlayout);
    QWidget *widget1 = ui->windowArea->takeWidget();
    delete  widget1;
    ui->windowArea->setWidget(widget);

    m_maxbar = m_scroolbar->maximum();
    m_scroolbar->setValue(0);
    InitFirstWindow();
}

void desktopWidget::drawdesktop(bool init)
{
    ptr_desktoplist.clear();
    if(cur_plubutton){
        cur_plubutton = nullptr;
    }
//    deskLayout->setEnabled(false);

    clearLayout(deskLayout);

    QHBoxLayout * lay = new QHBoxLayout();
    lay->addSpacing(20);

    auto desklist = NewStart::desktopmanager::instance().getdesktops();
    int rcount = 0;
    int column = 0;

    if (init){
        ThumbnailManager::getIns()->onDesktopChanged(KWindowSystem::currentDesktop());
    }

    for( auto it=desklist.constBegin(); it!=desklist.constEnd(); ++it)
    {
        rcount++;
        DesktopScreen * itemWgt = new DesktopScreen(this,it.key());
        itemWgt->setText(KWindowSystem::desktopName(it.key()));
        QPixmap pixmap  = ThumbnailManager::getIns()->getWallPaper(it.key());
        itemWgt->setPixmap(pixmap.scaled(PREVIEWSIZE, Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
        itemWgt->initwindowspic();
        if (NewStart::desktopmanager::instance().getSelectedDesktop() == it.key()){
            itemWgt->setselected();
            cur_desktop = itemWgt;
            column = rcount;
        }
        connect(itemWgt, SIGNAL(desktopChanged(int)) ,this, SLOT(drawwindow(int)));
        connect(itemWgt, SIGNAL(desktopselectd(int)) ,this, SLOT(selectdesktop(int)));
        connect(itemWgt, SIGNAL(desktopdelete(int)) ,&NewStart::desktopmanager::instance(), SLOT(deletedesktop(int)));
        connect(it->data(), SIGNAL(windowsChanged(int)) ,this, SLOT(drawwindow(int)));
        connect(it->data(), SIGNAL(backgroundadd(WId)) ,itemWgt, SLOT(backgroundadd(WId)));
        connect(it->data(), SIGNAL(backgrounddel(WId)) ,itemWgt, SLOT(backgrounddel(WId)));
        lay->addWidget(itemWgt);
        lay->addSpacing(20);
        ptr_desktoplist.push_back(itemWgt);
        itemWgt->installEventFilter(this);
    }

    if (ptr_desktoplist.count() == 1){
        (qobject_cast<DesktopScreen *>(ptr_desktoplist.at(0)))->hideCloseBtn();
    }

    plubtn = new AddDesktop(this);
    connect(plubtn,SIGNAL(PluClick()),this,SLOT(adddesktop()));
    lay->addWidget(plubtn);

    ptr_desktoplist.push_back(plubtn);
    deskLayout->addItem(lay);
    QWidget * widget = new QWidget();

    widget->setWindowOpacity(0.0);
    //widget->setAttribute(Qt::WA_TranslucentBackground);
    widget->setLayout(deskLayout);
    QWidget *widget1 = ui->desktopArea->takeWidget();
    delete  widget1;
    ui->desktopArea->setWidget(widget);

    m_maxhbar = m_hscroolbar->maximum();
    m_maxhbar = m_hscroolbar->maximum();
    int value;
    if (column == rcount){
        value = m_maxhbar;
    } else {
        value = (column-1) / ((float)rcount ) * m_maxhbar;
    }
    m_hscroolbar->setValue(value);

}

void desktopWidget::adddesktop()
{
    NewStart::KWinOpt::instance().createdesktop();
}

void desktopWidget::clearLayout(QBoxLayout * layout)
{
    while(layout->count() != 0)
    {
        QLayoutItem *child = layout->takeAt(0);
        if(child->widget())
        {
            QWidget * tmpWgt = child->widget();
            delete tmpWgt;
        }
        if(child->layout())
        {
            QBoxLayout * tmpLayout = qobject_cast<QBoxLayout*>(child->layout());
            clearLayout(tmpLayout);
            delete  tmpLayout;
        }
        layout->removeItem(child);
    }

}

void desktopWidget::paintEvent(QPaintEvent *event)
{

}

void desktopWidget::resizeEvent(QResizeEvent *event)
{
    InitSize();
}

void desktopWidget::InitSize()
{
    QRect rect = QApplication::desktop()->screenGeometry();
    if (oldwidth != rect.width()){
        oldwidth = rect.width();
        setGeometry(rect);
        QSize size = this->size();
        ui->windowArea->widget()->setGeometry(0,0,size.rwidth(),size.rheight()-DESKTOPHEIGHT);
        ui->windowArea->setGeometry(0,0,size.rwidth(),size.rheight()-DESKTOPHEIGHT);
        ui->desktopArea->setGeometry(0,size.rheight()-DESKTOPHEIGHT,size.rwidth(),DESKTOPHEIGHT);
        setbackground();
    }

}

void desktopWidget::getWindowsize()
{
    for (auto wid: KWindowSystem::self()->windows())
    {
        bool minsized = false;
        KWindowInfo info(wid, NET::WMState | NET::XAWMState);
        if (info.valid())
            minsized =info.isMinimized();
        windowflaglist.insert(wid,minsized);
    }
}
void desktopWidget::setWindowsize()
{
    for( auto it=windowflaglist.constBegin(); it!=windowflaglist.constEnd(); ++it){
        WId id = it.key();
        bool minsized = it.value();
        if (minsized) {
            KWindowSystem::minimizeWindow(id);
        }
    }
}

void desktopWidget::mousePressEvent(QMouseEvent *event)
{
    QWidget::mousePressEvent(event);
}

void desktopWidget::mouseReleaseEvent(QMouseEvent *event)
{
    selectdesktop(NewStart::desktopmanager::instance().getSelectedDesktop());
    //setWindowsize();
    this->releaseKeyboard();
    this->close();
    QWidget::mouseReleaseEvent(event);
}

void desktopWidget::setdesktopitem()
{
    InitSize();
    //getWindowsize()
    drawdesktop(true);
    drawwindow(NewStart::desktopmanager::instance().getSelectedDesktop());
    this->grabKeyboard();
}

desktopWidget::~desktopWidget()
{
    delete ui;
}

void desktopWidget::realign()
{
    if (ui->windowArea && ui->desktopArea){
    	mCompositorOpen = mPlugin->panel()->compositorOpen();
    	ui->desktopArea->setProperty("compositor", mCompositorOpen);
        style()->polish(ui->windowArea);
        style()->polish(ui->desktopArea);
    }
}
