#include "windowscreen.h"
#include "ui_windowscreen.h"
#include <QGraphicsDropShadowEffect>
#include <QMouseEvent>
#include <QDrag>
#include <QMimeData>
#include <QPainter>
#include <QStyleOption>
#include <QDesktopWidget>
#include <QPainterPath>

WindowScreen * selectedWindow = nullptr;

WindowScreen::WindowScreen(WId wid,int d_id,QWidget *parent) :
    QWidget(parent),
    m_wid(wid),
    desktopid(d_id),
    mMaxWidth(600),
    mMinWidth(100),
    mTextHeight(20),
    mCloseBtnLen(16),
    m_select(false),
    ui(new Ui::WindowScreen)
{
    ui->setupUi(this);
    QDesktopWidget *dw = QApplication::desktop();
    QRect desktopRect = dw->screenGeometry();
    mHeight = desktopRect.height()*6/16;


    mMinWidth = mHeight/3;
    mMaxWidth = mHeight*4/3;

    ui->image->setAlignment( Qt::AlignTop);
}

WindowScreen::~WindowScreen()
{
    if (this == selectedWindow){
        selectedWindow = nullptr;
    }

    delete ui;
}
void WindowScreen::setpixmap(QPixmap pic,WId Wid,QString str)
{
    this->m_wid = Wid;
    this->m_title = str;

    //根据比例、最大值、最小值计算最终图片的长宽
    int mWidght;
    float scale = (float)pic.width() / (float)pic.height();
    float labelWidth = scale*mHeight;

    if (labelWidth > mMaxWidth){
        mWidght = mMaxWidth;
        this->setMaximumWidth(mMaxWidth);
        this->setMinimumWidth(mMaxWidth);
    }else if (labelWidth < mMinWidth){
        mWidght = mMinWidth;
        this->setMaximumWidth(mMinWidth);
        this->setMinimumWidth(mMinWidth);
    }else{
        mWidght = labelWidth;
        this->setMaximumWidth(labelWidth);
        this->setMinimumWidth(labelWidth);
    }

    //根据图片长宽固定各个控件的大小，注意间距
    m_map = pic.scaled(QSize(mWidght-60,mHeight-60),Qt::KeepAspectRatio,Qt::SmoothTransformation);
    int margin = 2;
    ui->image->setFixedSize(m_map.size());
    ui->widgetBorder->setFixedSize(m_map.size().width()+(margin*2),m_map.height()+(margin*2));
    ui->widgetTitle->setFixedSize(m_map.size().width()+(margin), mTextHeight);
    ui->pushButton->setFixedSize(QSize(mCloseBtnLen,mCloseBtnLen));
    this->setFixedSize(QSize(m_map.size().width()+(margin*4), m_map.size().height()+(margin*7)+mTextHeight));
    update();

    //利用paintevent更改图片圆角
    QPainterPath path;
    QImage image(m_map.size(),QImage::Format_ARGB32);
    //QPixmap image(PREVIEWSIZE);
    image.fill(Qt::transparent);
    QPainter painter(&image);
    QRect previewRect = ui->image->rect();
    painter.setRenderHints(QPainter::Antialiasing, true);
    painter.setRenderHints(QPainter::SmoothPixmapTransform, true);
    path.addRoundedRect(previewRect, 3, 3);
    painter.setClipPath(path);
    painter.drawImage(0,0,m_map.toImage());
    QPixmap qb;
    qb = QPixmap::fromImage(image);

    ui->image->setPixmap(qb);
    ui->title->setText(str);
}

void WindowScreen::mousePressEvent(QMouseEvent *event)
{
    pos=event->pos();
    setProperty("Click", true);
    style()->polish(this);
}

void WindowScreen::mouseMoveEvent(QMouseEvent *e)
{
    //若拖动的距离大于5个像素则启用拖放
    if((e->pos()-pos).x()>=5||(e->pos()-pos).y()>=5) {
        setVisible(false);
        //在该事件中启动拖放
        QDrag *dg=new QDrag(this);
        QPixmap m_pixmap = QPixmap(m_map);
        dg->setPixmap(m_pixmap.scaled(100,100,Qt::KeepAspectRatioByExpanding, Qt::SmoothTransformation));
        dg->setHotSpot(QPoint(50,50));

        QMimeData *md=new QMimeData;
        QString msg = QString::number(desktopid) + "," + QString::number(m_wid);
        md->setText(msg);    			//拖放时传递的数据
        dg->setMimeData(md);   			//设置拖动的数据
        Qt::DropAction action = dg->exec(Qt::MoveAction,Qt::MoveAction);

        if (action != Qt::MoveAction) {
            setVisible(true);
            repaint();
        }
    }
}

void WindowScreen::selectWindow()
{
    if(selectedWindow) {
        if (this == selectedWindow) {
//            return;
        }
        selectedWindow->selectChanged();
    }
    selectedWindow = this;

    m_select = true;
    setProperty("Hover",true);
    style()->polish(this);
    style()->polish(ui->title);
    style()->polish(ui->widgetBorder);
    style()->polish(ui->pushButton);
    update();
}

void WindowScreen::selectChanged(){
    m_select = false;
    setProperty("Hover",false);
    style()->polish(this);
    style()->polish(ui->title);
    style()->polish(ui->widgetBorder);
    style()->polish(ui->pushButton);
    update();
}

void WindowScreen::enterEvent(QEvent *)                      //进入QWidget瞬间事件
{
    selectWindow();
}

void WindowScreen::leaveEvent(QEvent *)                      //离开QWidget瞬间事件
{
    setProperty("Hover",false);
    style()->polish(this);
    style()->polish(ui->title);
    style()->polish(ui->widgetBorder);
    style()->polish(ui->pushButton);
    update();
}
void WindowScreen::mouseReleaseEvent(QMouseEvent *e)
{
    setProperty("Hover",false);
    style()->polish(this);
    style()->polish(ui->title);
    style()->polish(ui->widgetBorder);
    style()->polish(ui->pushButton);
    update();
    Q_EMIT activewindow(m_wid);
}

void WindowScreen::paintEvent(QPaintEvent *event)
{
    QPainter p(this);
    QStyleOption opt;
    opt.initFrom(this);
    style()->drawPrimitive(QStyle::PE_Widget, &opt, &p, this);
}

void WindowScreen::on_pushButton_clicked()
{
    Q_EMIT deletewindow(m_wid);
}
