#include "appmanager.h"
#include "pinyinhandle.h"
#include <QTimer>
#include <QDBusInterface>
#include <QDebug>

QString getDFID(const XdgDesktopFile d)
{
    QStringList list = d.fileName().split("/");
    if(list.isEmpty()) return QString();
    return list.takeLast();
}

//CommonAppData排序函数
bool compareCommonAppData(const CommonAppData & c1, const CommonAppData & c2)
{
    if(c1._count == c2._count)
    {
        int com = QString::compare(c1._desktopFile.name(), c2._desktopFile.name(), Qt::CaseInsensitive);
        if(com >= 0) return false;
        return true;
    }
    return c1._count > c2._count;
}

//CommonAppData排序函数
bool compareFixedAppData(const FixedAppData & f1, const FixedAppData & f2)
{
    return f1._fixedTime > f2._fixedTime;
}

AppManager::~AppManager()
{
    saveFixedSetting();
//    saveCommonSetting();
}

AppManager *AppManager::getIns()
{
    static AppManager  mgr;
    return &mgr;
}

void AppManager::switchMenuCategory()
{
    if(mMenuCategoryType == CategoryByFile) {
        this->setMenuCategoryType(CategoryByABC);
    }
    else {
        this->setMenuCategoryType(CategoryByFile);
    }
}

void AppManager::setMenuCategoryType(MenuCategoryType type)
{
    mMenuCategoryType = type;
    mMenuSetting->beginGroup("mainmenu");
    mMenuSetting->setValue("MenuCategoryType", (int)type);
    mMenuSetting->endGroup();
    Q_EMIT menuCategoryChanged(type);
}

void AppManager::restoreMenuDisplayMode()
{
    mMenuSetting->beginGroup("mainmenu");
    mMenuDisplayMode = (MenuDisplayMode) (mMenuSetting->value("MenuDisplayMode", 0).toInt());
    mMenuSetting->endGroup();
}

void AppManager::switchMenuDisplayMode()
{
    if(mMenuDisplayMode == DisplayNormal) {
        this->setMenuDisplayMode(DisplayFull);
    }
    else {
        this->setMenuDisplayMode(DisplayNormal);
    }
}

void AppManager::setMenuDisplayMode(MenuDisplayMode mode)
{
    mMenuDisplayMode = mode;
    mMenuSetting->beginGroup("mainmenu");
    mMenuSetting->setValue("MenuDisplayMode", (int)mode);
    mMenuSetting->endGroup();
    QDBusInterface* interface = new QDBusInterface(QString::fromLatin1("org.nde.systemsettings.Taskbar"),
                                                     QString::fromLatin1("/org/nde/Taskbar"),
                                                     QString::fromLatin1("org.nde.systemsettings.Taskbar"),
                                                     QDBusConnection::sessionBus());
    interface->call("toggleDisplayFullScreenModeChanged", (int)mode);
    delete interface;
    Q_EMIT menuDisplayModeChanged(mode);
}

bool AppManager::menuCommonShow()
{
    mMenuSetting->beginGroup("mainmenu");
    mMenuCommonShow = mMenuSetting->value("MenuCommonShow", true).toBool();
    mMenuSetting->endGroup();
    return mMenuCommonShow;
}

bool AppManager::menuCommonBtnShow()
{
    return mMenuCommonShow;
}
void AppManager::setMenuCommonShow(bool checked)
{
    mMenuCommonShow = checked;
    mMenuSetting->beginGroup("mainmenu");
    mMenuSetting->setValue("MenuCommonShow", checked);
    mMenuSetting->endGroup();
}

void AppManager::rebuildMenu(QDomDocument doc)
{
    mapNormalApps.clear();
    mapABCApps.clear();

    mMenuDoc = doc;
    QDomElement root = mMenuDoc.documentElement();

    QString title;
    if (!root.attribute(QLatin1String("title")).isEmpty()) {
        title = root.attribute(QLatin1String("title"));
    }
    else {
        title = root.attribute(QLatin1String("name"));
    }

    this->parseXdgMenu(root, title);

    //menu变化时需要重新读取一下常用配置
    parseFixedSetting();
//    parseCommonSetting();

    Q_EMIT menuChanged();
}

void AppManager::commonAppFixed(const XdgDesktopFile dFile)
{
    FixedAppData tmpF;
    tmpF._desktopFile = dFile;

    bool isFind = false;
    for(FixedAppData f : mFixedAppDatas)
    {
        if(tmpF == f)
        {
            mFixedAppDatas.removeOne(tmpF);
            isFind = true;
            break;
        }
    }
    if(!isFind)
    {
        mFixedAppDatas.append(tmpF);
    }

    qSort(mFixedAppDatas.begin(), mFixedAppDatas.end(), compareFixedAppData);

    Q_EMIT commonAppDataChanged();

    saveFixedSetting();
}

void AppManager::systemCommonAppFixed(const QString fileName)
{
    XdgDesktopFile df;
    df.load(fileName);
    commonAppFixed(df);
}

void AppManager::setMenuCommonBtnShow(bool checked)
{
    mMenuCommonShow = checked;
    mMenuSetting->beginGroup("mainmenu");
    mMenuSetting->setValue("MenuCommonShow", checked);
    mMenuSetting->endGroup();
}
//void AppManager::commonAppCount(const XdgDesktopFile dFile)
//{
//    CommonAppData tmpC;
//    tmpC._desktopFile = dFile;
//    bool isFind = false;
//    for(CommonAppData & c: mCommonAppDatas)
//    {
//        if(c == tmpC)
//        {
//            c.add();
//            isFind = true;
//            break;
//        }
//    }
//    if(!isFind)
//    {
//        tmpC.add();
//        mCommonAppDatas.append(tmpC);
//    }

//    //按CommonAppData的_count从大到小排序
//    qSort(mCommonAppDatas.begin(), mCommonAppDatas.end(), compareCommonAppData);

//    Q_EMIT commonAppDataChanged();

//    saveCommonSetting();
//}

bool AppManager::isCommonApp(const XdgDesktopFile dFile)
{
    bool isFind = false;

    for(FixedAppData f: mFixedAppDatas)
    {
        if(getDFID(f._desktopFile) == getDFID(dFile))
        {
            isFind = true;
            break;
        }
    }

    // 按频率使用的常用app暂时屏蔽
//    for(const CommonAppData & c : mCommonAppDatas)
//    {
//        if(getDFID(c._desktopFile) == getDFID(dFile))
//        {
//            isFind = true;
//            break;
//        }
//    }

    return isFind;
}

QList<XdgDesktopFile> AppManager::getCommonApps(MenuDisplayMode mode)
{
    QList<XdgDesktopFile> list;
    for(FixedAppData f: mFixedAppDatas)
    {
        if(!XdgDesktopFile::id(f._desktopFile.fileName()).isEmpty())
        {
            list.append(f._desktopFile);
        }
    }

    // 按频率使用的常用app暂时屏蔽
//    int limit = mLimitWindow;
//    if(mode == DisplayFull)  limit = mLimitFull;

//    for(CommonAppData c: mCommonAppDatas)
//    {
//        if(limit <= list.count()) break;
//        if(!XdgDesktopFile::id(c._desktopFile.fileName()).isEmpty())
//        {
//            list.append(c._desktopFile);
//        }
//    }

    return list;
}

QHash<QString, QList<XdgDesktopFile> > AppManager::getApps()
{
    if(mMenuCategoryType == CategoryByFile) return mapNormalApps;
    return mapABCApps;
}

void AppManager::addCommonAppFixed(QString name)
{
    XdgDesktopFile dFile;
    dFile.load(name);
    if(dFile.isValid())
    {
        commonAppFixed(dFile);
        qWarning() << "DesktopFile Load From D-BUS Successed:" << name;
    }
    else {
        qWarning() << "DesktopFile Load From D-BUS Filed:" << name;
    }
}

//void AppManager::addCommonAppCount(QString name)
//{
//    XdgDesktopFile dFile;
//    dFile.load(name);
//    if(dFile.isValid())
//    {
//        commonAppCount(dFile);
//        qWarning() << "DesktopFile Load From D-BUS Successed:" << name;
//    }
//    else {
//        qWarning() << "DesktopFile Load From D-BUS Filed:" << name;
//    }
//}

AppManager::AppManager(QObject *parent) :
    QObject(parent)
{
    mMenuSetting = new Nde::Settings(QLatin1String("panel"), this);
    mMenuSetting->beginGroup("mainmenu");
    mMenuDisplayMode = (MenuDisplayMode) (mMenuSetting->value("MenuDisplayMode", 0).toInt());
    mMenuCategoryType = (MenuCategoryType) (mMenuSetting->value("MenuCategoryType", 0).toInt());
    mMenuCommonShow = mMenuSetting->value("MenuCommonShow", true).toBool();
    mMenuSetting->endGroup();

    mCommonSetting = new Nde::Settings(QLatin1String("mainmenu"), this);

    if (!QDBusConnection::sessionBus().isConnected()) {
        qWarning() << "Cannot connect to the D-Bus session bus.";
    }

    if (!QDBusConnection::sessionBus().registerService(QStringLiteral("org.nde.AppManager"))) {
        qWarning() << QDBusConnection::systemBus().lastError().message();
    }
    QDBusConnection::sessionBus().registerObject(QStringLiteral("/AppManager"), QStringLiteral("org.nde.AppManager"), this);
}

void AppManager::parseFixedSetting()
{
    mFixedAppDatas.clear();
    //读取配置
    mCommonSetting->beginGroup("fixed_apps");
    QStringList keys = mCommonSetting->childKeys();
    for(QString key : keys)
    {
        FixedAppData f;
        QString v = mCommonSetting->value(key).toString();
        f._desktopFile.load(key);
        f._fixedTime = QDateTime::fromString(v,"yyyy/MM/dd HH:mm:ss");

        //判断该desktop所在的文件是否存在
        if(XdgDesktopFile::id(f._desktopFile.fileName()).isEmpty())
        {
            mCommonSetting->remove(key);
            continue;
        }

        mFixedAppDatas.append(f);
    }
    mCommonSetting->endGroup();

    //按时间从最近到到最早排序
    qSort(mFixedAppDatas.begin(), mFixedAppDatas.end(), compareFixedAppData);
}

void AppManager::saveFixedSetting()
{
    mCommonSetting->beginGroup("fixed_apps");

    QStringList keys = mCommonSetting->childKeys();
    for(QString key : keys)
    {
        mCommonSetting->remove(key);
    }

    for(FixedAppData f: mFixedAppDatas)
    {
        QString key = getDFID(f._desktopFile);
        mCommonSetting->setValue(key, f._fixedTime.toString("yyyy/MM/dd HH:mm:ss"));
    }
    mCommonSetting->endGroup();

    mCommonSetting->sync();
}

//void AppManager::parseCommonSetting()
//{
//    mCommonAppDatas.clear();

//    //读取配置
//    mCommonSetting->beginGroup("common_app_num");
//    mLimitWindow = mCommonSetting->value("limit_window", 3).toInt();
//    mLimitFull = mCommonSetting->value("limit_full", 10).toInt();
//    mCommonSetting->endGroup();

//    mCommonSetting->beginGroup("common_app_count");
//    QStringList keys = mCommonSetting->childKeys();
//    for(QString key : keys)
//    {
//        CommonAppData c;
//        QString v = mCommonSetting->value(key).toString();
//        c._desktopFile.load(key);
//        c._count = v.toInt();

//        //判断该desktop所在的文件是否存在
//        if(XdgDesktopFile::id(c._desktopFile.fileName()).isEmpty())
//        {
//            mCommonSetting->remove(key);
//            continue;
//        }
//        mCommonAppDatas.append(c);
//    }
//    mCommonSetting->endGroup();

//    //按CommonAppData的_count从大到小排序
//    qSort(mCommonAppDatas.begin(), mCommonAppDatas.end(), compareCommonAppData);
//}

//void AppManager::saveCommonSetting()
//{
//    mCommonSetting->beginGroup("common_app_num");
//    mCommonSetting->setValue("limit_window", mLimitWindow);
//    mCommonSetting->setValue("limit_full", mLimitFull);
//    mCommonSetting->endGroup();

//    mCommonSetting->beginGroup("common_app_count");
//    for(CommonAppData c: mCommonAppDatas)
//    {
//        QString key = getDFID(c._desktopFile);
//        mCommonSetting->setValue(key, QString::number(c._count));
//    }
//    mCommonSetting->endGroup();

//    mCommonSetting->sync();
//}

void AppManager::parseXdgMenu(QDomElement &root, QString category)
{
    QDomElement ele = root.firstChildElement();
    while(!ele.isNull())
    {
        //分组
        if(ele.tagName() == QLatin1String("Menu"))
        {
            //分组名称
            QString title;
            if (!ele.attribute(QLatin1String("title")).isEmpty()) {
                title = ele.attribute(QLatin1String("title"));
            }
            else {
                title = ele.attribute(QLatin1String("name"));
            }

            this->parseXdgMenu(ele, title);
        }
        //应用
        else if(ele.tagName() == QLatin1String("AppLink"))
        {
            XdgDesktopFile df;
            df.load(ele.attribute(QLatin1String("desktopFile")));

            //qWarning() << df.fileName() << XdgDesktopFile::id(df.fileName(),false);

            //去除重复的项目
            bool isRepeat = false;
            for(QHash<QString, QList<XdgDesktopFile> >::iterator it = mapNormalApps.begin();
                it != mapNormalApps.end(); it++)
            {
                for(XdgDesktopFile & tmpDf : it.value())
                {
                    bool condtion1 = (XdgDesktopFile::id(df.fileName(),false) == XdgDesktopFile::id(tmpDf.fileName(),false));
                    bool condtion2 = (tmpDf.fileName().split("/").last() == df.fileName().split("/").last());
                    if(condtion1 || condtion2)
                    {
                        isRepeat = true;
                        break;
                    }
                }
            }

            if(!isRepeat)
            {
                //默认分类
                QHash<QString, QList<XdgDesktopFile> >::iterator it = mapNormalApps.find(category);
                if(it != mapNormalApps.end())
                {
                    it.value().append(df);
                } else {
                    QList<XdgDesktopFile> tmpList;
                    tmpList.append(df);
                    mapNormalApps.insert(category, tmpList);
                }

                //首字母分类
                QString noSpaceAppName = PinyinHandle::removeAllSpace(df.name());
                QString appNameFirstLetter = PinyinHandle::ConvertToQuanpin(noSpaceAppName);
                QString categoryABC = appNameFirstLetter.isEmpty() ? "" : QString(appNameFirstLetter.at(0));
                categoryABC = categoryABC.toUpper();
                QHash<QString, QList<XdgDesktopFile> >::iterator it1 = mapABCApps.find(categoryABC);
                if(it1 != mapABCApps.end()) {
                    it1.value().append(df);
                } else {
                    QList<XdgDesktopFile> tmpList;
                    tmpList.append(df);
                    mapABCApps.insert(categoryABC, tmpList);
                }
            }
        }

        ele = ele.nextSiblingElement();
    }
}
