#ifndef APPMANAGER_H
#define APPMANAGER_H

#include <QObject>
#include <Nde/Settings>
#include <XdgDesktopFile>
#include <QDateTime>
#include <QHash>
#include <QStackedLayout>
#include <QDomDocument>
#include "common.h"

QString getDFID(const XdgDesktopFile d);

struct FixedAppData
{
    XdgDesktopFile _desktopFile; //desktop文件
    QDateTime _fixedTime;

    FixedAppData()
    {
        _fixedTime = QDateTime::currentDateTime();
    }
    FixedAppData(const FixedAppData & f)
    {
        _desktopFile = f._desktopFile;
        _fixedTime = f._fixedTime;
    }
    void operator=(const FixedAppData & f)
    {
        _desktopFile = f._desktopFile;
        _fixedTime = f._fixedTime;
    }
    bool operator==(const FixedAppData & f)
    {
        if(f._desktopFile.isValid() && _desktopFile.isValid())
        {
            return getDFID(f._desktopFile) == getDFID(_desktopFile);
        }
        return false;
    }
};

//桌面应用使用频率统计
struct CommonAppData
{
    XdgDesktopFile _desktopFile; //desktop文件
    int _count;                  //用户点击次数
    CommonAppData()
    {
        _count = 0;
    }
    CommonAppData(const CommonAppData & c)
    {
        _desktopFile = c._desktopFile;
        _count = c._count;
    }
    void operator=(const CommonAppData & c)
    {
        _desktopFile = c._desktopFile;
        _count = c._count;
    }
    bool operator==(const CommonAppData & c)
    {
        if(c._desktopFile.isValid() && _desktopFile.isValid())
        {
            return getDFID(c._desktopFile) == getDFID(_desktopFile);
        }
        return false;
    }
    void add(int count = 1)
    {
        _count+=count;
    }
};

class AppManager : public QObject
{
    Q_OBJECT
    Q_PROPERTY(MenuCategoryType menuCategoryType READ menuCategoryType WRITE setMenuCategoryType NOTIFY menuCategoryChanged)
    Q_PROPERTY(MenuDisplayMode menuDisplayMode READ menuDisplayMode WRITE setMenuDisplayMode NOTIFY menuDisplayModeChanged)

    Q_CLASSINFO("D-Bus Interface", "com.nde.panel.mainmenu")

public:
    ~AppManager();
    static AppManager * getIns();

    //@menuCategoryType
    void switchMenuCategory();
    MenuCategoryType menuCategoryType(){return mMenuCategoryType;}
    void setMenuCategoryType(MenuCategoryType type);

    //@menuDisplayMode
    void restoreMenuDisplayMode();
    void switchMenuDisplayMode();
    MenuDisplayMode menuDisplayMode(){return mMenuDisplayMode;}
    void setMenuDisplayMode(MenuDisplayMode mode);
    bool menuCommonShow();
    bool menuCommonBtnShow();
    void setMenuCommonShow(bool checked);

    //读取menu xml配置文件
    void rebuildMenu(QDomDocument doc);

    //设置为固定的常用应用
    void commonAppFixed(const XdgDesktopFile dFile);

    //用户点击应用，按照点击次数计算常用
//    void commonAppCount(const XdgDesktopFile dFile);

    //判断是否是常用app
    bool isCommonApp(const XdgDesktopFile dFile);

    //获取常用App的desktop文件
    QList<XdgDesktopFile> getCommonApps(MenuDisplayMode mode);

    //获取所有App的desktop文件
    QHash<QString, QList<XdgDesktopFile> > getApps();

Q_SIGNALS:
    //开始菜单配置文件变化
    void menuChanged();
    //菜单分类形式改变
    void menuCategoryChanged(MenuCategoryType type);
    //菜单显示模式改变
    void menuDisplayModeChanged(MenuDisplayMode mode);
    //常用app数量变化
    void commonAppNumChanged();
    //常用app变化
    void commonAppDataChanged();

public Q_SLOTS:
    //dbus 接口
    void addCommonAppFixed(QString name);
//    void addCommonAppCount(QString name);
    void systemCommonAppFixed(const QString fileName);
    void setMenuCommonBtnShow(bool checked);

private:
    explicit AppManager(QObject *parent = nullptr);
    void parseFixedSetting();
    void saveFixedSetting();
//    void parseCommonSetting();
//    void saveCommonSetting();
    void parseXdgMenu(QDomElement &root, QString category);

private:
    //@开始菜单相关配置
    QDomDocument mMenuDoc;
    Nde::Settings *mMenuSetting;      //菜单配置
    MenuDisplayMode mMenuDisplayMode;               //全屏或窗口模式显示
    MenuCategoryType mMenuCategoryType;             //当前分类
    QHash<QString, QList<XdgDesktopFile> > mapNormalApps; //默认分类的所有app
    QHash<QString, QList<XdgDesktopFile> > mapABCApps; //首字母分类的所有app
    bool mMenuCommonShow;          // 是否显示常用按钮

    //@常用应用相关配置
//    int mLimitWindow; //窗口模式下显示的常用应用数量
//    int mLimitFull; //全屏模式下显示的常用应用数量
    Nde::Settings *mCommonSetting;  //常用应用配置
//    QList<CommonAppData> mCommonAppDatas; //常用应用配置列表
    QList<FixedAppData> mFixedAppDatas; //用户固定到常用的应用
};

#endif // APPMANAGER_H
