#ifndef DISPLAYMAINMENU_H
#define DISPLAYMAINMENU_H

#include <QWidget>
#include <QVBoxLayout>
#include <QDomElement>
#include "mainmenuapplicationitem.h"
#include "mainmenucategoryitem.h"
#include "powermenu.h"
#include "common.h"
#include "../panel/ndepanel.h"

namespace Ui {
class DisplayMainMenu;
}

class DisplayMainMenu : public QWidget
{
    Q_OBJECT

public:
    explicit DisplayMainMenu(INdePanelPlugin * plugin, QWidget *parent = nullptr);
    ~DisplayMainMenu();

    void restoreUI();

    //检测Lineedit 上下键 event
    virtual bool eventFilter(QObject *watched, QEvent *event);

Q_SIGNALS:
    void signalAddToDesktop(XdgDesktopFile desktopFile);
    void signalAddToQuickLaunch(XdgDesktopFile desktopFile);

private Q_SLOTS:
    void onMenuChanged();
    void onCommonAppChanged();
    void onPowerMenuAnimFinished();
    void onSearchTextChanged(const QString & filter);
    void onLineeditReturnPressed();
    void onMenuCategoryChanged();

    void onSwitchClicked();
    void onSwitchCommonClicked();
    void onSoftwareCenterClicked();
    void onUserClicked();
    void onComputerClicked();
    void onSettingClicked();
    void onPowerClicked();

    void focusNextItem();
    void focusPrivItem();

private:
    void switchToMenuPage();
    void switchToCommonPage();
    void switchPowerMenu(bool showPowerMenu);
    void addToLayout();
    void clearLayout();
    QList<MainMenuApplicationItem*> getCurSortItems();
    void focusFirstItem();
    void focusLastItem();
    void updateUserIcon();

private:
    INdePanelPlugin * mPlugin;

    Ui::DisplayMainMenu *ui;

    bool mShowPowerMenu;
    PowerMenu *mPowerMenu;

    QVBoxLayout * mMenuLayout;          //应用列表布局
    MainMenuCategoryItem*    mCommonCategory;   //常用分类UI
    QHash<QString, MainMenuCategoryItem*> mAllCategories; //所有分类UI项

    bool mInputMethodActive; //输入法是否激活的状态
};

#endif // DISPLAYMAINMENU_H
