#include "displaymainmenufull.h"
#include "ui_displaymainmenufull.h"
#include <QDomDocument>
#include <QScreen>
#include <QTimer>
#include <QKeyEvent>
#include "pinyinhandle.h"
#include "appmanager.h"
#include "toolbutton.h"

#include <QDebug>

DisplayMainMenuFull::DisplayMainMenuFull(INdePanelPlugin *plugin, QWidget *parent) :
    QWidget(parent),
    mPlugin(plugin),
    ui(new Ui::DisplayMainMenuFull),
    mCommonCategory(nullptr),
    mInputMethodActive(false)
{
    ui->setupUi(this);

    mMenuLayout = new QVBoxLayout();
    mMenuLayout->setMargin(0);
    mMenuLayout->setContentsMargins(0,0,0,0);
    mMenuLayout->setSpacing(2);
    mMenuLayout->addStretch();
    ui->scrollAreaWidgetContents->setFocusPolicy(Qt::NoFocus);
    ui->scrollAreaWidgetContents->setLayout(mMenuLayout);

    mCategoryBtnsLayout = new QVBoxLayout();
    mCategoryBtnsLayout->setMargin(0);
    mCategoryBtnsLayout->setContentsMargins(0,0,0,0);
    mCategoryBtnsLayout->setSpacing(10);
    ui->scrollAreaWidgetContents_btns->setFocusPolicy(Qt::NoFocus);
    ui->scrollAreaWidgetContents_btns->setLayout(mCategoryBtnsLayout);

    connect(ui->btn_switch,&QToolButton::clicked, [this](){
        AppManager::getIns()->switchMenuCategory();
        if(AppManager::getIns()->menuCategoryType() == CategoryByFile){
            ui->btn_switch->setToolTip(tr("Letter Switch"));
        } else {
            ui->btn_switch->setToolTip(tr("Category Switch"));
        }
    });
    connect(ui->lineEdit_search,&QLineEdit::textChanged, this, &DisplayMainMenuFull::onSearchTextChanged);
    connect(ui->lineEdit_search, &QLineEdit::returnPressed, this, &DisplayMainMenuFull::onLineeditReturnPressed);
    connect(ui->btn_reduce,&QToolButton::clicked, this, [](){
        AppManager::getIns()->switchMenuDisplayMode();});

    ui->lineEdit_search->setContextMenuPolicy(Qt::NoContextMenu);
    ui->lineEdit_search->installEventFilter(this);

    connect(AppManager::getIns(), &AppManager::menuChanged, this, &DisplayMainMenuFull::onMenuChanged);
    connect(AppManager::getIns(), &AppManager::commonAppDataChanged, this, &DisplayMainMenuFull::onCommonAppChanged);
    connect(AppManager::getIns(), &AppManager::menuCategoryChanged, this, &DisplayMainMenuFull::onMenuCategoryChanged);
}

DisplayMainMenuFull::~DisplayMainMenuFull()
{
    delete ui;
}

void DisplayMainMenuFull::restoreUI()
{
    onSearchTextChanged("");
    //初始化
    ui->lineEdit_search->setText("");
    ui->lineEdit_search->setFocus();
    if(mCommonCategory) ui->scrollArea->ensureWidgetVisible(mCommonCategory);
    onMenuChanged();
    if(AppManager::getIns()->menuCategoryType() == CategoryByFile){
        ui->btn_switch->setToolTip(tr("Letter Switch"));
    } else {
        ui->btn_switch->setToolTip(tr("Category Switch"));
    }
}

bool DisplayMainMenuFull::eventFilter(QObject *watched, QEvent *event)
{
    if(watched == ui->lineEdit_search)
    {
        if(event->type() == QEvent::KeyPress && !mInputMethodActive)
        {
            QKeyEvent * keyEvent = static_cast<QKeyEvent*>(event);
            if(keyEvent && keyEvent->key() == Qt::Key_Up)
            {
                event->accept();
                focusLastItem();
                return true;
            }
            if(keyEvent && keyEvent->key() == Qt::Key_Down)
            {
                event->accept();
                focusFirstItem();
                return true;
            }
        }
        else if(event->type() == QEvent::InputMethod)
        {
            QInputMethodEvent * keyEvent = static_cast<QInputMethodEvent*>(event);

            //keyEvent->preeditString() 若该字符串不为空说明输入法被激活，否则没有激活输入法
            mInputMethodActive = !keyEvent->preeditString().isEmpty();
        }
    }
    return QWidget::eventFilter(watched, event);
}

void DisplayMainMenuFull::onMenuChanged()
{
    mMenuLayout->setEnabled(false);
    clearLayout();

    QList<XdgDesktopFile> commonApp =  AppManager::getIns()->getCommonApps(DisplayFull);
    QHash<QString, QList<XdgDesktopFile> > mapApps = AppManager::getIns()->getApps();
    if(AppManager::getIns()->menuCommonShow())
    {
        for(XdgDesktopFile common : commonApp)
        {
            MainMenuApplicationItemFull* item = new MainMenuApplicationItemFull(
                        mPlugin, common, ui->scrollAreaWidgetContents);
            connect(item, &MainMenuApplicationItemFull::signalClicked, [this](){
                this->topLevelWidget()->hide();});
            connect(item, &MainMenuApplicationItemFull::signalStartDrag, [this](){
                this->topLevelWidget()->hide();});
            connect(item, &MainMenuApplicationItemFull::signalFocusNext, [this](){this->focusNextItem();});
            connect(item, &MainMenuApplicationItemFull::signalFocusNextLine, [this](){this->focusNextItemLine();});
            connect(item, &MainMenuApplicationItemFull::signalFocusPrvi, [this](){this->focusPrivItem();});
            connect(item, &MainMenuApplicationItemFull::signalFocusPrviLine, [this](){this->focusPrivItemLine();});
            connect(item, &MainMenuApplicationItemFull::signalFocusSearch, [this](){ui->lineEdit_search->setFocus();});
            connect(item, &MainMenuApplicationItemFull::signalAddToDesktop,
                    this, &DisplayMainMenuFull::signalAddToDesktop);
            connect(item, &MainMenuApplicationItemFull::signalAddToQuickLaunch,
                    this, &DisplayMainMenuFull::signalAddToQuickLaunch);
            item->setVisible(true);

            if(!mCommonCategory)
            {
                QString category = tr("Common");
                mCommonCategory = new MainMenuCategoryItemFull(category, ui->scrollAreaWidgetContents);

                QToolButton * btn = new FocusToolButton(ui->scrollAreaWidgetContents_btns);
                btn->setText(category);
                btn->setFixedSize(ui->scrollArea_btns->width(), 30);
                connect(btn, &QToolButton::clicked, this, &DisplayMainMenuFull::onCategoryBtnClicked);
                mCommonCategory->setCategaryBtn(btn);

//                //由于style中的HighlightedText颜色默认为黑色，这里设置其默认颜色为白色
                QPalette pa = btn->palette();
                pa.setColor(QPalette::HighlightedText, QColor(Qt::white));
                btn->setPalette(pa);
            }
            mCommonCategory->addItem(item);
        }
    }

    for(QString category : mapApps.keys())
    {
        QList<XdgDesktopFile> list = mapApps.value(category);
        for(int i = 0; i < list.count(); i++)
        {
            XdgDesktopFile desktopFile = list.at(i);
            MainMenuApplicationItemFull* item = new MainMenuApplicationItemFull(
                        mPlugin, desktopFile, ui->scrollAreaWidgetContents);
            connect(item, &MainMenuApplicationItemFull::signalClicked, [this](){
                this->topLevelWidget()->hide();});
            connect(item, &MainMenuApplicationItemFull::signalStartDrag, [this](){
                this->topLevelWidget()->hide();});
            connect(item, &MainMenuApplicationItemFull::signalFocusNext, [this](){this->focusNextItem();});
            connect(item, &MainMenuApplicationItemFull::signalFocusNextLine, [this](){this->focusNextItemLine();});
            connect(item, &MainMenuApplicationItemFull::signalFocusPrvi, [this](){this->focusPrivItem();});
            connect(item, &MainMenuApplicationItemFull::signalFocusPrviLine, [this](){this->focusPrivItemLine();});
            connect(item, &MainMenuApplicationItemFull::signalFocusSearch, [this](){ui->lineEdit_search->setFocus();});
            connect(item, &MainMenuApplicationItemFull::signalAddToDesktop,
                    this, &DisplayMainMenuFull::signalAddToDesktop);
            connect(item, &MainMenuApplicationItemFull::signalAddToQuickLaunch,
                    this, &DisplayMainMenuFull::signalAddToQuickLaunch);
            item->setVisible(true);

            QHash<QString, MainMenuCategoryItemFull*>::iterator it = mAllCategories.find(category);
            if(it != mAllCategories.end())
            {
                it.value()->addItem(item);
            }
            else
            {
                MainMenuCategoryItemFull * categoryItem = new MainMenuCategoryItemFull(category, ui->scrollAreaWidgetContents);
                categoryItem->addItem(item);
                mAllCategories.insert(category, categoryItem);

                QToolButton * btn = new FocusToolButton(ui->scrollAreaWidgetContents_btns);
                btn->setText(category);
                btn->setFixedSize(ui->scrollArea_btns->width(), 30);
                connect(btn, &QToolButton::clicked, this, &DisplayMainMenuFull::onCategoryBtnClicked);
                categoryItem->setCategaryBtn(btn);

//                //由于style中的HighlightedText颜色默认为黑色，这里设置其默认颜色为白色
                QPalette pa = btn->palette();
                pa.setColor(QPalette::HighlightedText, QColor(Qt::white));
                btn->setPalette(pa);
            }
        }
    }

    addToLayout();
    mMenuLayout->setEnabled(true);
}

void DisplayMainMenuFull::onCommonAppChanged()
{
    onMenuChanged();
//    if(mCommonCategory)
//    {
//        QList<MainMenuApplicationItemFull*> items = mCommonCategory->getItems();
//        while(items.count())
//        {
//            MainMenuApplicationItemFull* item = items.takeFirst();
//            mMenuLayout->removeWidget(item);
//            mCommonCategory->removeItem(item);
//            delete item;
//        }
//    }
}

void DisplayMainMenuFull::onSearchTextChanged(const QString &filter)
{
    //先清除子布局
    QStringList categorys = mAllCategories.keys();
    while(categorys.count())
    {
        QString category = categorys.takeFirst();

        MainMenuCategoryItemFull * categoryItem = mAllCategories.value(category);
        mMenuLayout->removeWidget(categoryItem);
    }
    while(mMenuLayout->count())
    {
        delete mMenuLayout->takeAt(0);
    }


    if(mCommonCategory)
    {
        //搜索时不显示常用应用列表
        bool visible = filter.isEmpty();
        mMenuLayout->addWidget(mCommonCategory);
        mCommonCategory->setVisible(visible);
        mCommonCategory->getCategaryBtn()->setVisible(visible);

        QList<MainMenuApplicationItemFull*> items = mCommonCategory->getItems();
        for(MainMenuApplicationItemFull* item: items)
        {
            item->setVisible(visible);
        }
        QList<QHBoxLayout *> childLayouts = calItemLayout(items);
        for(QHBoxLayout * childLayout : childLayouts)
        {
            mMenuLayout->addLayout(childLayout);
        }
    }

    //按搜索结果重新排列
    categorys = mAllCategories.keys();
    categorys.sort(Qt::CaseInsensitive);
    while(categorys.count())
    {
        QString category = categorys.takeFirst();

        MainMenuCategoryItemFull * categoryItem = mAllCategories.value(category);
        categoryItem->setFilter(filter);
        mMenuLayout->addWidget(categoryItem);

        QList<MainMenuApplicationItemFull*> items = categoryItem->getItems();
        QList<QHBoxLayout *> childLayouts = calItemLayout(items);
        for(QHBoxLayout * childLayout : childLayouts)
        {
            mMenuLayout->addLayout(childLayout);
        }
    }

    mMenuLayout->addStretch();
}

void DisplayMainMenuFull::onLineeditReturnPressed()
{
    focusFirstItem();
}

void DisplayMainMenuFull::onMenuCategoryChanged()
{
    this->onMenuChanged();
}

void DisplayMainMenuFull::onCategoryBtnClicked()
{
    //
    if(mCommonCategory)
    {
        if(mCommonCategory->getCategaryBtn() == sender())
        {
            for(MainMenuApplicationItemFull * item : mCommonCategory->getItems())
            {
                ui->scrollArea->ensureWidgetVisible(item);
            }
            ui->scrollArea_btns->ensureWidgetVisible(mCommonCategory->getCategaryBtn());
            ui->scrollArea->ensureWidgetVisible(mCommonCategory);
            mCommonCategory->getCategaryBtn()->setFocus();
            return;
        }
    }
    QStringList categorys = mAllCategories.keys();
    while(categorys.count())
    {
        QString category = categorys.takeFirst();

        MainMenuCategoryItemFull * categoryItem = mAllCategories.value(category);
        if(categoryItem->getCategaryBtn() == sender())
        {
            for(MainMenuApplicationItemFull * item : categoryItem->getItems())
            {
                ui->scrollArea->ensureWidgetVisible(item);
            }
            ui->scrollArea_btns->ensureWidgetVisible(categoryItem->getCategaryBtn());
            ui->scrollArea->ensureWidgetVisible(categoryItem);
            categoryItem->getCategaryBtn()->setFocus();
            break;
        }
    }
}

void DisplayMainMenuFull::addToLayout()
{
    this->setTabOrder(ui->lineEdit_search, ui->btn_switch);
    QWidget * priv = ui->btn_switch;

    mCategoryBtnsLayout->addStretch(0);

    if(mCommonCategory)
    {
        mMenuLayout->addWidget(mCommonCategory);

        mCategoryBtnsLayout->addWidget(mCommonCategory->getCategaryBtn());
        this->setTabOrder(priv, mCommonCategory->getCategaryBtn());
        priv = mCommonCategory->getCategaryBtn();

        QList<MainMenuApplicationItemFull*> items = mCommonCategory->getItems();
        QList<QHBoxLayout *> childLayouts = calItemLayout(items);
        for(QHBoxLayout * childLayout : childLayouts)
        {
            mMenuLayout->addLayout(childLayout);
        }

        QList<MainMenuApplicationItemFull*>::iterator it = items.begin();
        while(it != items.end())
        {
            this->setTabOrder(priv, *it);
            priv = *it;
            it++;
        }
    }

    QStringList categorys = mAllCategories.keys();
    categorys.sort(Qt::CaseInsensitive);
    while(categorys.count())
    {
        QString category = categorys.takeFirst();

        MainMenuCategoryItemFull * categoryItem = mAllCategories.value(category);
        mMenuLayout->addWidget(categoryItem);

        mCategoryBtnsLayout->addWidget(categoryItem->getCategaryBtn());
        this->setTabOrder(priv, categoryItem->getCategaryBtn());
        priv = categoryItem->getCategaryBtn();

        QList<MainMenuApplicationItemFull*> items = categoryItem->getItems();
        QList<QHBoxLayout *> childLayouts = calItemLayout(items);
        for(QHBoxLayout * childLayout : childLayouts)
        {
            mMenuLayout->addLayout(childLayout);
        }

        QList<MainMenuApplicationItemFull*>::iterator it = items.begin();
        while(it != items.end())
        {
            this->setTabOrder(priv, *it);
            priv = *it;
            it++;
        }
    }

    this->setTabOrder(priv, ui->btn_reduce);
    this->setTabOrder(ui->btn_reduce, ui->lineEdit_search);

    mMenuLayout->addStretch();
    mCategoryBtnsLayout->addStretch(0);
}

void DisplayMainMenuFull::clearLayout()
{
    if(mCommonCategory)
    {
        mMenuLayout->removeWidget(mCommonCategory);

        QToolButton *btn = mCommonCategory->getCategaryBtn();
        mCategoryBtnsLayout->removeWidget(btn);
        delete btn;

        QList<MainMenuApplicationItemFull*> items = mCommonCategory->getItems();
        while(items.count())
        {
            MainMenuApplicationItemFull* item = items.takeFirst();
            mMenuLayout->removeWidget(item);
            mCommonCategory->removeItem(item);
            delete item;
        }
        delete mCommonCategory;
        mCommonCategory = nullptr;
    }

    QStringList categorys = mAllCategories.keys();
    categorys.sort(Qt::CaseInsensitive);
    while(categorys.count())
    {
        QString category = categorys.takeFirst();

        MainMenuCategoryItemFull * categoryItem = mAllCategories.take(category);
        mMenuLayout->removeWidget(categoryItem);

        QToolButton *btn = categoryItem->getCategaryBtn();
        mCategoryBtnsLayout->removeWidget(btn);
        delete  btn;

        QList<MainMenuApplicationItemFull*> items = categoryItem->getItems();
        while(items.count())
        {
            MainMenuApplicationItemFull* item = items.takeFirst();
            mMenuLayout->removeWidget(item);
            categoryItem->removeItem(item);
            delete item;
        }
        delete categoryItem;
    }

    while(mMenuLayout->count())
    {
        delete mMenuLayout->takeAt(0);
    }
    while(mCategoryBtnsLayout->count())
    {
        delete mCategoryBtnsLayout->takeAt(0);
    }

    ui->lineEdit_search->setText("");
}

QList<QHBoxLayout *> DisplayMainMenuFull::calItemLayout(QList<MainMenuApplicationItemFull *> list)
{
    QList<QHBoxLayout *> childLayouts;
    int width = 0;
    QHBoxLayout * childLayout = 0;

    QList<MainMenuApplicationItemFull*>::iterator it = list.begin();
    while(it != list.end())
    {
        if((*it)->isVisible()){
            int space = 48;
            if(!childLayout ||  (width + space + (*it)->width()) > ui->scrollArea->width())
            {
                width = (*it)->width() + 60;
                childLayout = new QHBoxLayout;
                childLayout->setContentsMargins(60,0,0,20);
                childLayout->setSpacing(space);
                childLayout->addStretch(0);
                childLayouts.append(childLayout);
            }
            else
            {
                width += (*it)->width() + space;
            }
            childLayout->insertWidget(childLayout->count()-1,*it);
        }
        it++;
    }

    return childLayouts;
}

QList<MainMenuApplicationItemFull *> DisplayMainMenuFull::getCurSortItems()
{
    QList<MainMenuApplicationItemFull *> list;
    if(mCommonCategory)
    {
        QList<MainMenuApplicationItemFull*> items = mCommonCategory->getItems();
        QList<MainMenuApplicationItemFull*>::iterator it = items.begin();
        while(it != items.end())
        {
            if((*it)->isVisible()) {
                list.append(*it);
            }
            it++;
        }
    }

    QStringList categorys = mAllCategories.keys();
    categorys.sort(Qt::CaseInsensitive);
    while(categorys.count())
    {
        QString category = categorys.takeFirst();

        MainMenuCategoryItemFull * categoryItem = mAllCategories.value(category);

        QList<MainMenuApplicationItemFull*> items = categoryItem->getItems();
        QList<MainMenuApplicationItemFull*>::iterator it = items.begin();
        while(it != items.end())
        {
            if((*it)->isVisible()){
                list.append(*it);
            }
            it++;
        }
    }
    return list;
}

void DisplayMainMenuFull::focusFirstItem()
{
    QList<MainMenuApplicationItemFull*>  list = getCurSortItems();
    if(list.count() > 0)
    {
        MainMenuApplicationItemFull *item = list.at(0);
        item->setFocus();
        ui->scrollArea->ensureWidgetVisible(item);
    }
}

void DisplayMainMenuFull::focusLastItem()
{
    QList<MainMenuApplicationItemFull*>  list = getCurSortItems();
    if(list.count() > 0)
    {
        MainMenuApplicationItemFull *item = list.at(list.count()-1);
        item->setFocus();
        ui->scrollArea->ensureWidgetVisible(item);
    }
}

void DisplayMainMenuFull::focusNextItem()
{
    QList<MainMenuApplicationItemFull*>  list = getCurSortItems();
    MainMenuApplicationItemFull * next = 0;
    MainMenuApplicationItemFull * cur = 0;
    QList<MainMenuApplicationItemFull*>::iterator it = list.begin();
    while (it != list.end()) {
        if(cur){
            next = *it;
            break;
        }
        if(this->focusWidget() == *it)
        {
            cur = *it;
        }
        it++;
    }

    if(next)
    {
        next->setFocus();
        ui->scrollArea->ensureWidgetVisible(next);
    }
    else
    {
        ui->lineEdit_search->setFocus();
    }
}

void DisplayMainMenuFull::focusNextItemLine()
{
    QList<MainMenuApplicationItemFull*>  list = getCurSortItems();
    MainMenuApplicationItemFull * cur = 0;
    QList<MainMenuApplicationItemFull*>::iterator it = list.begin();
    while (it != list.end()) {
        if(this->focusWidget() == *it)
        {
            cur = *it;
            break;
        }
        it++;
    }

    int index = 0;
    QHBoxLayout * curChildLayout = 0;
    QHBoxLayout * nextChildLayout = 0;
    for(int i =0 ; i < mMenuLayout->count(); i++)
    {
        QLayoutItem * itemI = mMenuLayout->itemAt(i);
        QHBoxLayout * childLayout = qobject_cast<QHBoxLayout*>(itemI->layout());
        if(childLayout)
        {
            if(curChildLayout)
            {
                nextChildLayout = childLayout;
                break;
            }
            for(int j =0 ; j < childLayout->count(); j++)
            {
                QLayoutItem * itemJ = childLayout->itemAt(j);
                MainMenuApplicationItemFull * appItem = qobject_cast<MainMenuApplicationItemFull*>(itemJ->widget());
                if(appItem)
                {
                    if(appItem == cur)
                    {
                        curChildLayout = childLayout;
                        index = j;
                        break;
                    }
                }
            }
        }
    }

    if(nextChildLayout)
    {
        index = qMin(index, nextChildLayout->count() - 2);
        MainMenuApplicationItemFull * next = qobject_cast<MainMenuApplicationItemFull*>(nextChildLayout->itemAt(index)->widget());
        next->setFocus();
        ui->scrollArea->ensureWidgetVisible(next);
    }
    else
    {
        ui->lineEdit_search->setFocus();
    }
}

void DisplayMainMenuFull::focusPrivItem()
{
    QList<MainMenuApplicationItemFull*>  list = getCurSortItems();
    MainMenuApplicationItemFull * prvi = 0;
    MainMenuApplicationItemFull * cur = 0;
    QList<MainMenuApplicationItemFull*>::iterator it = list.end();
    while (it != list.begin()) {
        it--;
        if(cur){
            prvi = *it;
            break;
        }
        if(this->focusWidget() == *it)
        {
            cur = *it;
        }
    }

    if(prvi)
    {
        prvi->setFocus();
        ui->scrollArea->ensureWidgetVisible(prvi);
    }
    else
    {
        ui->lineEdit_search->setFocus();
    }
}

void DisplayMainMenuFull::focusPrivItemLine()
{
    QList<MainMenuApplicationItemFull*>  list = getCurSortItems();
    MainMenuApplicationItemFull * cur = 0;
    QList<MainMenuApplicationItemFull*>::iterator it = list.begin();
    while (it != list.end()) {
        if(this->focusWidget() == *it)
        {
            cur = *it;
            break;
        }
        it++;
    }

    int index = 0;
    QHBoxLayout * curChildLayout = 0;
    QHBoxLayout * priChildLayout = 0;
    for(int i = mMenuLayout->count() - 1 ; i >= 0; i--)
    {
        QLayoutItem * itemI = mMenuLayout->itemAt(i);
        QHBoxLayout * childLayout = qobject_cast<QHBoxLayout*>(itemI->layout());
        if(childLayout)
        {
            if(curChildLayout)
            {
                priChildLayout = childLayout;
                break;
            }
            for(int j =0 ; j < childLayout->count(); j++)
            {
                QLayoutItem * itemJ = childLayout->itemAt(j);
                MainMenuApplicationItemFull * appItem = qobject_cast<MainMenuApplicationItemFull*>(itemJ->widget());
                if(appItem)
                {
                    if(appItem == cur)
                    {
                        curChildLayout = childLayout;
                        index = j;
                        break;
                    }
                }
            }
        }
    }

    if(priChildLayout)
    {
        index = qMin(index, priChildLayout->count() - 2);
        MainMenuApplicationItemFull * next = qobject_cast<MainMenuApplicationItemFull*>(priChildLayout->itemAt(index)->widget());
        next->setFocus();
        ui->scrollArea->ensureWidgetVisible(next);
    }
    else
    {
        ui->lineEdit_search->setFocus();
    }
}
