#ifndef DISPLAYWIDGET_H
#define DISPLAYWIDGET_H

#include <QWidget>
#include <Nde/Settings>
#include <QPropertyAnimation>
#include "../panel/ndepanel.h"
#include "../plugin-quicklaunch/ndequicklaunchplugin.h"
#include "../plugin-quicklaunch/ndequicklaunch.h"
#include "common.h"
#include "displaymainmenu.h"
#include "displaymainmenufull.h"

class DisplayWidget : public QWidget
{
    Q_OBJECT
public:
    explicit DisplayWidget(INdePanelPlugin * plugin, QWidget *parent = nullptr);

    void showMainMenu();

    void realign();

protected Q_SLOTS:
    void slotMainMenuModeSwitch(MenuDisplayMode mode);
    void slotModeSwitchAnimFinished();
    void slotAddToDesktop(XdgDesktopFile desktopFile);
    void slotAddToQuickLaunch(XdgDesktopFile desktopFile);

protected:
    bool event(QEvent *event) override;
    virtual void resizeEvent(QResizeEvent *event) override;
    virtual void paintEvent(QPaintEvent *event) override;

private:
    void showMainMenuByMode();
    QRect calGeometry();
    void updateBackground();

private:
    INdePanelPlugin * mPlugin;

    INdePanel::Position mCurPanelPos; //位置
    bool mCompositorOpen; //特效开关

    QPixmap mBlurBackground;

    QDomDocument mDoc;

    QPropertyAnimation * mAnimation;

    MenuDisplayMode mMenuDisplayMode;
    QWidget * mBackgroundWidget;
    DisplayMainMenu * mDisplayMainMenu;
    DisplayMainMenuFull * mDisplayMainMenuFull;
};

#endif // DISPLAYWIDGET_H
