#include "mainmenuapplicationitemfull.h"
#include <XdgIcon>
#include <QEvent>
#include <QMouseEvent>
#include <QStyleOption>
#include <QPainter>
#include <QApplication>
#include <QDrag>
#include <QMimeData>
#include <QTimer>
#include <QMenu>
#include "appmanager.h"
#include <QDebug>
#include <QDBusInterface>
#include "../plugin-quicklaunch/ndequicklaunch.h"


MainMenuApplicationItemFull::MainMenuApplicationItemFull(INdePanelPlugin *plugin, QWidget *parent) :
    QWidget(parent),
    mPlugin(plugin)
{
    createUI();
}

MainMenuApplicationItemFull::MainMenuApplicationItemFull(INdePanelPlugin *plugin, const XdgDesktopFile &desktopFile, QWidget *parent) :
    QWidget(parent),
    mPlugin(plugin)
{
    createUI();
    load(desktopFile);
}

MainMenuApplicationItemFull::MainMenuApplicationItemFull(INdePanelPlugin *plugin, const XdgDesktopFile *desktopFile, QWidget *parent) :
    QWidget(parent),
    mPlugin(plugin)
{
    createUI();
    load(*desktopFile);
}

MainMenuApplicationItemFull::MainMenuApplicationItemFull(INdePanelPlugin *plugin, const QString &desktopFileName, QWidget *parent) :
    QWidget(parent),
    mPlugin(plugin)
{
    createUI();
    XdgDesktopFile df;
    df.load(desktopFileName);
    load(df);
}

MainMenuApplicationItemFull::MainMenuApplicationItemFull(INdePanelPlugin *plugin, const MainMenuApplicationItemFull &other, QWidget *parent) :
    QWidget(parent),
    mPlugin(plugin)
{
    createUI();
    load(other.mDesktopFile);
}

MainMenuApplicationItemFull::~MainMenuApplicationItemFull()
{
    delete  m_labelIcon;
    delete  m_labelText;
}

MainMenuApplicationItemFull &MainMenuApplicationItemFull::operator=(
        const MainMenuApplicationItemFull &other)
{
    load(other.mDesktopFile);
    return *this;
}

bool MainMenuApplicationItemFull::isValid() const
{
    return mDesktopFile.isValid();
}

void MainMenuApplicationItemFull::updateIcon()
{
    QIcon icon = mDesktopFile.icon().isNull() ? XdgIcon::fromTheme(QLatin1String("application-x-executable")) : mDesktopFile.icon();

    // Some themes may lack the "application-x-executable" icon; checking null prevents
    // infinite recursion (setIcon->dataChanged->updateIcon->setIcon
    if (!icon.isNull()) {
        m_labelIcon->setPixmap(icon.pixmap(m_labelIcon->size()));
    }
}

void MainMenuApplicationItemFull::onAddToFixed()
{
    QString fileman = this->desktopFile().fileName();
    QDBusInterface* interface = new QDBusInterface(QString::fromLatin1("org.nde.systemsettings.Taskbar"),
                                                     QString::fromLatin1("/org/nde/CommonMenu"),
                                                     QString::fromLatin1("org.nde.systemsettings.CommonMenu"),
                                                     QDBusConnection::sessionBus());
    interface->call("commonAppFixedChanged", fileman);
    delete interface;

    AppManager::getIns()->commonAppFixed(this->desktopFile());
}

void MainMenuApplicationItemFull::paintEvent(QPaintEvent *event)
{
    QStyleOption opt;
    opt.init(this);
    QPainter p(this);
    style()->drawPrimitive(QStyle::PE_Widget, &opt, &p, this);
}

void MainMenuApplicationItemFull::mousePressEvent(QMouseEvent *event)
{
    //鼠标按下开始拖拽
    if(event->button() == Qt::LeftButton)
    {
        mDragStartPt = event->pos();
    }
    QWidget::mousePressEvent(event);
}

void MainMenuApplicationItemFull::mouseReleaseEvent(QMouseEvent *event)
{
    Q_EMIT signalClicked();
    QWidget::mouseReleaseEvent(event);
}

void MainMenuApplicationItemFull::mouseMoveEvent(QMouseEvent *event)
{
    if (!(event->buttons() & Qt::LeftButton))
    {
        return;
    }

    if ((event->pos() - mDragStartPt).manhattanLength() < QApplication::startDragDistance())
    {
        return;
    }

    //这里需要隐藏菜单界面
    Q_EMIT signalStartDrag();

    QList<QUrl> urls;
    urls << QUrl::fromLocalFile(mDesktopFile.fileName());

    QMimeData *mimeData = new QMimeData();
    mimeData->setUrls(urls);

    QDrag *drag = new QDrag(this);
    QIcon icon = mDesktopFile.icon().isNull() ? XdgIcon::fromTheme(QLatin1String("application-x-executable")) : mDesktopFile.icon();
    drag->setPixmap(icon.pixmap(m_labelIcon->size()));
    drag->setMimeData(mimeData);
    drag->exec(Qt::CopyAction | Qt::LinkAction);

    QWidget::mouseMoveEvent(event);
}

void MainMenuApplicationItemFull::keyPressEvent(QKeyEvent *event)
{
    if(event->key() == Qt::Key_Left)
    {
        Q_EMIT signalFocusPrvi();
    }
    else if(event->key() == Qt::Key_Up)
    {
        Q_EMIT signalFocusPrviLine();
    }
    else if(event->key() == Qt::Key_Right)
    {
        Q_EMIT signalFocusNext();
    }
    else if(event->key() == Qt::Key_Down)
    {
        Q_EMIT signalFocusNextLine();
    }
    QWidget::keyPressEvent(event);
}

void MainMenuApplicationItemFull::keyReleaseEvent(QKeyEvent *event)
{
    if(event->key() == Qt::Key_Enter || event->key() == Qt::Key_Space ||
            event->key() == Qt::Key_Return)
    {
        Q_EMIT signalClicked();
    }
    else if(event->key() >= Qt::Key_A && event->key() <= Qt::Key_Z)
    {
        Q_EMIT signalFocusSearch();
    }
    else if(event->key() >= Qt::Key_0 && event->key() <= Qt::Key_9)
    {
        Q_EMIT signalFocusSearch();
    }
    QWidget::keyReleaseEvent(event);
}

void MainMenuApplicationItemFull::contextMenuEvent(QContextMenuEvent *event)
{
    //右键菜单
    QMenu menu;
    menu.setWindowFlag(Qt::FramelessWindowHint);
    menu.setAttribute(Qt::WA_TranslucentBackground);
    if(mPlugin->panel()->compositorOpen())
    {
        menu.setStyleSheet("QMenu{background-color:white;color:black;padding-left:24px;border: none;border-radius:4px;margin:2px;padding:2px;}"
                           "QMenu::item{background-color:white;color:#222222;}"
                           "QMenu::item:selected{background-color:#2c9dfc;color:white;}");
    }
    else {
        menu.setStyleSheet("QMenu{background-color:rgba(255,255,255,1);color:black;border: 1px solid #d8d8d8;padding-left:24px;margin:0px;padding:2px;}"
                           "QMenu::item{background-color:white;color:#222222;}"
                           "QMenu::item:selected{background-color:#2c9dfc;color:white;}");
    }
    menu.addAction(tr("Open"), [this](){this->runConmmand();});
    if(AppManager::getIns()->menuCommonShow())
    {
        QString fixedStr = AppManager::getIns()->isCommonApp(mDesktopFile) ?  tr("UnFixed"):tr("Fixed To Common");
        menu.addAction(fixedStr, this, &MainMenuApplicationItemFull::onAddToFixed);
    }
    menu.addAction(tr("Add To Desktop"), [this](){Q_EMIT signalAddToDesktop(this->desktopFile());});
    menu.addAction(tr("Add To QuickLaunch"), [this](){Q_EMIT signalAddToQuickLaunch(this->desktopFile());});

    menu.exec(QCursor::pos());
}

void MainMenuApplicationItemFull::runConmmand() const
{
    if (mDesktopFile.isValid()) {
        mDesktopFile.startDetached();

        //需要新启动一个线程。原因：统计app时会重新加载常用列表，导致this被删除。
//        XdgDesktopFile tmpDF = mDesktopFile;
//        QTimer::singleShot(0,[tmpDF](){
//            AppManager::getIns()->commonAppCount(tmpDF);
//        });
    }
}

void MainMenuApplicationItemFull::createUI()
{
    this->setObjectName("MainMenuApplicationItemFull");

    this->setFocusPolicy(Qt::StrongFocus);

    QHBoxLayout * iconLayout = new QHBoxLayout;
    m_labelIcon = new QLabel(this);
    m_labelIcon->setObjectName("label_icon");
    m_labelIcon->setAlignment(Qt::AlignCenter);
    m_labelIcon->setFixedSize(64,64);
    iconLayout->addStretch();
    iconLayout->addWidget(m_labelIcon);
    iconLayout->addStretch();

    m_labelText = new QLabel(this);
    m_labelText->setObjectName("label_text");
    m_labelText->setFixedHeight(28);
    m_labelText->setFixedWidth(92);
    m_labelText->setTextFormat(Qt::RichText);
    QFont font = m_labelText->font();
    font.setPixelSize(14);
    m_labelText->setFont(font);
    m_labelText->setWordWrap(false);
    m_labelText->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
    m_labelText->setAlignment(Qt::AlignCenter);

    m_layout = new QVBoxLayout;
    m_layout->setMargin(0);
    m_layout->setSpacing(4);
    m_layout->setContentsMargins(2,2,2,2);
    m_layout->addLayout(iconLayout);
    m_layout->addWidget(m_labelText);

    this->setFixedSize(96,100);
    this->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);

    this->setLayout(m_layout);
}

void MainMenuApplicationItemFull::load(const XdgDesktopFile &desktopFile)
{
    mDesktopFile = desktopFile;
    if (mDesktopFile.isValid())
    {
        // & is reserved for mnemonics
        QString text = mDesktopFile.name().replace(QLatin1Char('&'), QLatin1String("&&"));
        QFontMetrics fontMetrics = QFontMetrics(m_labelText->font());
        QString elideText = fontMetrics.elidedText(text, Qt::ElideRight, m_labelText->width()-6);
        m_labelText->setText(elideText);
//        m_labelText->setText(text);

        QIcon icon = mDesktopFile.icon().isNull() ? XdgIcon::fromTheme(QLatin1String("application-x-executable")) : mDesktopFile.icon();
        // Some themes may lack the "application-x-executable" icon; checking null prevents
        // infinite recursion (setIcon->dataChanged->updateIcon->setIcon
         if (!icon.isNull()) {
             m_labelIcon->setPixmap(icon.pixmap(m_labelIcon->size()));
         }

         setToolTip(mDesktopFile.comment());

         connect(this, SIGNAL(signalClicked()), this, SLOT(runConmmand()));
    }
    else
    {
        m_labelText->setText(QString());
        setToolTip(QString());
        m_labelIcon->setPixmap(QPixmap());
    }
}
