#ifndef MAINMENUAPPLICATIONITEMFULL_H
#define MAINMENUAPPLICATIONITEMFULL_H

#include <QToolButton>
#include <XdgDesktopFile>
#include <QLabel>
#include <QHBoxLayout>
#include "../panel/ndepanel.h"

class MainMenuApplicationItemFull : public QWidget
{
    Q_OBJECT
public:
    explicit MainMenuApplicationItemFull(INdePanelPlugin * plugin, QWidget *parent = nullptr);
    explicit MainMenuApplicationItemFull(INdePanelPlugin * plugin, const XdgDesktopFile& desktopFile, QWidget *parent = nullptr);
    explicit MainMenuApplicationItemFull(INdePanelPlugin * plugin, const XdgDesktopFile* desktopFile, QWidget *parent = nullptr);
    explicit MainMenuApplicationItemFull(INdePanelPlugin * plugin, const QString& desktopFileName, QWidget *parent = nullptr);
    explicit MainMenuApplicationItemFull(INdePanelPlugin * plugin, const MainMenuApplicationItemFull& other, QWidget *parent = nullptr);

    /// Destroys the object and frees allocated resources.
    virtual ~MainMenuApplicationItemFull();
    MainMenuApplicationItemFull& operator=(const MainMenuApplicationItemFull& other);

    //! Returns true if the XdgAction is valid; otherwise returns false.
    bool isValid() const;

    const XdgDesktopFile& desktopFile() const { return mDesktopFile; }

Q_SIGNALS:
    void signalClicked();
    void signalStartDrag();
    void signalFocusNext();
    void signalFocusNextLine();
    void signalFocusPrvi();
    void signalFocusPrviLine();
    void signalFocusSearch();

    void signalAddToDesktop(XdgDesktopFile desktopFile);
    void signalAddToQuickLaunch(XdgDesktopFile desktopFile);

public Q_SLOTS:
    void updateIcon();// nouse
    void onAddToFixed();

protected:
    virtual void paintEvent(QPaintEvent *event);
    virtual void mousePressEvent(QMouseEvent *event);
    virtual void mouseReleaseEvent(QMouseEvent *event);
    virtual void mouseMoveEvent(QMouseEvent *event);
    virtual void keyPressEvent(QKeyEvent *event);
    virtual void keyReleaseEvent(QKeyEvent *event);
    virtual void contextMenuEvent(QContextMenuEvent *event);

private Q_SLOTS:
    void runConmmand() const;

private:
    void createUI();
    void load(const XdgDesktopFile& desktopFile);

    INdePanelPlugin * mPlugin;

    XdgDesktopFile mDesktopFile;

    QLabel * m_labelIcon;
    QLabel * m_labelText;
    QVBoxLayout * m_layout;

    QPoint mDragStartPt; //开始拖拽的点
};

#endif // MAINMENUAPPLICATIONITEMFULL_H
