/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)LGPL2+
 *
 * Nde - a lightweight, Qt based, desktop toolset
 * https://lxqt.org
 *
 * Copyright: 2010-2011 Razor team
 * Authors:
 *   Alexander Sokoloff <sokoloff.a@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */


#ifndef NDE_MAINMENU_H
#define NDE_MAINMENU_H

#include "../panel/indepanelplugin.h"
#include <XdgMenu>
#include <QLabel>
#include <QToolButton>
#include <QDomElement>
#include <QAction>
#include <QKeySequence>
#include "displaywidget.h"

namespace Nde {
class PowerManager;
class ScreenSaver;
}

namespace GlobalKeyShortcut
{
class Action;
}

class NdeMainMenu : public QObject, public INdePanelPlugin
{
    Q_OBJECT
public:
    NdeMainMenu(const INdePanelPluginStartupInfo &startupInfo);
    ~NdeMainMenu();

    QString themeId() const { return "MainMenu"; }
    virtual INdePanelPlugin::Flags flags() const { return SingleInstance ; }

    QWidget *widget() { return &mButton; }
//    QDialog *configureDialog();
    virtual void realign();

    bool isSeparate() const { return true; }

protected:
    bool eventFilter(QObject *obj, QEvent *event);

private:
    void setButtonIcon();

private:
    QToolButton mButton;
    QString mLogDir;

    GlobalKeyShortcut::Action *mShortcut1;
    GlobalKeyShortcut::Action *mShortcut2;
    XdgMenu mXdgMenu;
    QString mMenuFile;

    DisplayWidget * mDisplay;
    INdePanel::Position mCurPanelPos;
    QRect mScreenRect;

protected Q_SLOTS:
    virtual void readXDGMenuFile();
    void buildMenu();

public Q_SLOTS:
    void showHideMenu();
};

class NdeMainMenuPluginLibrary: public QObject, public INdePanelPluginLibrary
{
    Q_OBJECT
    // Q_PLUGIN_METADATA(IID "lxqt.org/Panel/PluginInterface/3.0")
    Q_INTERFACES(INdePanelPluginLibrary)
public:
    INdePanelPlugin *instance(const INdePanelPluginStartupInfo &startupInfo) const { return new NdeMainMenu(startupInfo);}
};

#endif
