#include "pinyinhandle.h"
#include <QTextCodec>
#include <QDebug>

bool In(int min, int max, int v)
{
    return (v >= min && v <= max);
}

char convert(int n)
{
    /*
         * GB2312码范围 (B0A1-F7FE)
         * HiByte (B0-F7) LoByte (A1-FE)
         *
         * GBK编码范围 (8140-FEFE)
         * 包含三大部分
         * 1.汉字区
         *      a. GB2312汉字区。即GBK/2
         *      b. GB13000.1扩充汉字区。包括GBK/3(CJK汉字)和GBK/4(CJK汉字和增补汉字)
         * 2.图形符号区
         *      a. GB2312非汉字区。即GBK/1 A1A1-A9FE。还有10个小写罗马数字和GB12345增补符号
         *      b. GB13000.1扩充非汉字区。即GBK/5 A840-A9A0非汉字符号、结构符...
         * 3.用户自定义区
         *
         */

    if (In(0xB0A1,0xB0C4,n)) return 'A';
    if (In(0XB0C5,0XB2C0,n)) return 'B';
    if (In(0xB2C1,0xB4ED,n)) return 'C';
    if (In(0xB4EE,0xB6E9,n)) return 'D';
    if (In(0xB6EA,0xB7A1,n)) return 'E';
    if (In(0xB7A2,0xB8C0,n)) return 'F';
    if (In(0xB8C1,0xB9FD,n)) return 'G';
    if (In(0xB9FE,0xBBF6,n)) return 'H';
    if (In(0xBBF7,0xBFA5,n)) return 'J';
    if (In(0xBFA6,0xC0AB,n)) return 'K';
    if (In(0xC0AC,0xC2E7,n)) return 'L';
    if (In(0xC2E8,0xC4C2,n)) return 'M';
    if (In(0xC4C3,0xC5B5,n)) return 'N';
    if (In(0xC5B6,0xC5BD,n)) return 'O';
    if (In(0xC5BE,0xC6D9,n)) return 'P';
    if (In(0xC6DA,0xC8BA,n)) return 'Q';
    if (In(0xC8BB,0xC8F5,n)) return 'R';
    if (In(0xC8F6,0xCBF9,n)) return 'S';
    if (In(0xCBFA,0xCDD9,n)) return 'T';
    if (In(0xCDDA,0xCEF3,n)) return 'W';
    if (In(0xCEF4,0xD1B8,n)) return 'X';
    if (In(0xD1B9,0xD4D0,n)) return 'Y';
    if (In(0xD4D1,0xD7F9,n)) return 'Z';
    if (In(0x00, 0x7f, n)) return n;
    return '\0';
}


PinyinHandle::PinyinHandle(QObject *parent) : QObject(parent)
{

}

QString PinyinHandle::ConvertToPinyin(const QString zhStr)
{
    return zhStr;
}

QString PinyinHandle::ConvertToQuanpin(const QString zhStr)
{
    QString firstLetters;
    for (int i=0; i<zhStr.length(); i++)
    {
        QString str = zhStr.at(i);
        QTextCodec* pCodec = QTextCodec::codecForName("gb2312");
        if(!pCodec) return QChar(' ');
        QByteArray arr = pCodec->fromUnicode(str);
        wchar_t wchr = 0;

        if(arr.size() == 1)
        {
            wchr = arr.at(0) & 0xff;
        }
        else if(arr.size() == 2)
        {
            wchr = (arr.at(0) & 0xff) << 8;
            wchr |= (arr.at(1) & 0xff);
        }
        else
        {
            //qDebug() << "unknown word";
        }

        char c = convert(wchr);
        if(c != 0)
        {
            firstLetters.append(c);
        }
    }
    return firstLetters;
}

bool PinyinHandle::isZh(const QChar zh)
{
    ushort uni = zh.unicode();
    return (uni >= 0x4E00 && uni <= 0x9FA5);
}

QString PinyinHandle::removeAllSpace(QString str)
{
    return str.remove(QRegExp("\\s"));
}

QString PinyinHandle::escape(QString string)
{
    return string.replace(QLatin1Char('&'), QLatin1String("&&"));
}

bool PinyinHandle::checkSearchFit(const XdgDesktopFile deskFile, const QString &filter)
{
    QString nospaceFileter = PinyinHandle::removeAllSpace(filter);
    if(nospaceFileter.isEmpty()) {
        return true;
    }

    QString nospaceName = PinyinHandle::removeAllSpace(deskFile.name());
    QString appNameFirstLetter = PinyinHandle::ConvertToQuanpin(nospaceName);

    //检测应用名称
    if(nospaceName.contains(nospaceFileter, Qt::CaseInsensitive)) {
        return true;
    }

    //检查首字母
    if(appNameFirstLetter.contains(nospaceFileter, Qt::CaseInsensitive)) {
        return true;
    }

    return false;
}

