#include "powermenu.h"
#include <QCoreApplication>
#include <QDBusConnection>
#include <QDBusInterface>
#include <QSettings>
#include <QDebug>
#include <QTime>

#define POWERBTNSIZE QSize(55,32)

PowerMenu::PowerMenu(QWidget *parent) :
    QWidget(parent),
    mExtend(false)
{
    m_power = new Nde::Power(this);

    this->setObjectName("PowerMenu");

    mLayout = new QVBoxLayout;
    mLayout->setMargin(0);
    mLayout->setContentsMargins(0,0,0,0);
    mLayout->setSpacing(0);

    QList<ToolButton*> btnList;
    mBtnSleep = new ToolButton(this);
    mBtnSleep->setToolButtonStyle(Qt::ToolButtonTextOnly);
    mBtnSleep->setText(tr("Sleep"));//睡眠
    mBtnSleep->setFixedSize(POWERBTNSIZE);
    mBtnSleep->setGeometry(QRect(QPoint(0,0), POWERBTNSIZE));
    connect(mBtnSleep, &ToolButton::clicked, this, &PowerMenu::onComputerSleep);
    mLayout->addWidget(mBtnSleep);
    mBtnSleep->hide();
    btnList.append(mBtnSleep);

    mBtnSwitch = new ToolButton(this);
    mBtnSwitch->setToolButtonStyle(Qt::ToolButtonTextOnly);
    mBtnSwitch->setText(tr("Switch"));//切换
    mBtnSwitch->setFixedSize(POWERBTNSIZE);
    mBtnSwitch->setGeometry(QRect(QPoint(0,0), POWERBTNSIZE));
    connect(mBtnSwitch, &ToolButton::clicked, this, &PowerMenu::onComputerSwitch);
    mLayout->addWidget(mBtnSwitch);
    mBtnSwitch->hide();
    btnList.append(mBtnSwitch);

    mBtnLogout = new ToolButton(this);
    mBtnLogout->setToolButtonStyle(Qt::ToolButtonTextOnly);
    mBtnLogout->setText(tr("Logout"));//注销
    mBtnLogout->setFixedSize(POWERBTNSIZE);
    mBtnLogout->setGeometry(QRect(QPoint(0,0), POWERBTNSIZE));
    connect(mBtnLogout, &ToolButton::clicked, this, &PowerMenu::onComputerLogout);
    mLayout->addWidget(mBtnLogout);
    mBtnLogout->hide();
    btnList.append(mBtnLogout);

    mBtnReboot = new ToolButton(this);
    mBtnReboot->setToolButtonStyle(Qt::ToolButtonTextOnly);
    mBtnReboot->setText(tr("Reboot"));//重启
    mBtnReboot->setFixedSize(POWERBTNSIZE);
    mBtnReboot->setGeometry(QRect(QPoint(0,0), POWERBTNSIZE));
    connect(mBtnReboot, &ToolButton::clicked, this, &PowerMenu::onComputerReboot);
    mLayout->addWidget(mBtnReboot);
    mBtnReboot->hide();
    btnList.append(mBtnReboot);

    mBtnPowerOff = new ToolButton(this);
    mBtnPowerOff->setToolButtonStyle(Qt::ToolButtonTextOnly);
    mBtnPowerOff->setText(tr("PowerOff"));//关机
    mBtnPowerOff->setFixedSize(POWERBTNSIZE);
    mBtnPowerOff->setGeometry(QRect(QPoint(0,0), POWERBTNSIZE));
    connect(mBtnPowerOff, &ToolButton::clicked, this, &PowerMenu::onComputerPowerOff);
    mLayout->addWidget(mBtnPowerOff);
    mBtnPowerOff->hide();
    btnList.append(mBtnPowerOff);

    mAnimGroup = new QParallelAnimationGroup();
    connect(mAnimGroup, &QPropertyAnimation::finished, this, &PowerMenu::onAnimFinished);

    for(int i = 0; i < btnList.count(); i++)
    {
        ToolButton * btn = btnList.at(i);

        QPropertyAnimation * mBtnAnim = new QPropertyAnimation(btn, "pos", btn);
        mBtnAnim->setDuration(200);
        mBtnAnim->setStartValue(QPoint(0,POWERBTNSIZE.height()*btnList.count()));
        mBtnAnim->setEndValue(QPoint(0,POWERBTNSIZE.height()*i));
        mBtnAnims.insert(btn, mBtnAnim);

        //由于style中的HighlightedText颜色默认为黑色，这里设置其默认颜色为白色
        QPalette pa = btn->palette();
        pa.setColor(QPalette::HighlightedText, QColor(Qt::white));
        btn->setPalette(pa);
        mAnimGroup->addAnimation(mBtnAnim);
    }

    this->setLayout(mLayout);
}

void PowerMenu::expend(bool extend)
{
    //
    mExtend = extend;

    if(mAnimGroup->state() == QPropertyAnimation::Running)
    {
        mAnimGroup->pause();
    }
    mAnimGroup->setDirection(extend ? QPropertyAnimation::Forward : QPropertyAnimation::Backward);
    mAnimGroup->start();
}

QList<ToolButton *> PowerMenu::getBtns()
{
    QList<ToolButton *> btnList;
    //虚拟机屏蔽休眠按钮
    // if(!checkVirtualMachine() && m_power->canAction(Nde::Power::PowerHibernate))
    // {
    //     btnList.append(mBtnSleep);
    // }
    //zte版本屏蔽切换用户按钮,所有版本屏蔽切换用户按钮
//    if(!checkZteVersion() && m_power->canAction(Nde::Power::PowerSwitchUser))
//    {
//        btnList.append(mBtnSwitch);
//    }

    //sssd_be进程被占用时会导致调用m_power->canAction()函数延迟
//    if(m_power->canAction(Nde::Power::PowerLogout))
//    {
//        btnList.append(mBtnLogout);
//    }
//    if(m_power->canAction(Nde::Power::PowerReboot))
//    {
//        btnList.append(mBtnReboot);
//    }
//    if(m_power->canAction(Nde::Power::PowerShutdown))
//    {
//        btnList.append(mBtnPowerOff);
//    }
    btnList.append(mBtnLogout);
    btnList.append(mBtnReboot);
    btnList.append(mBtnPowerOff);
    return btnList;
}

void PowerMenu::restorePowerMenu()
{
    QList<ToolButton *> btnList = getBtns();

    while (mAnimGroup->animationCount()) {
        mAnimGroup->takeAnimation(0);
    }
    mBtnSleep->setVisible( btnList.contains(mBtnSleep));
    mBtnSwitch->setVisible( btnList.contains(mBtnSwitch));
    mBtnLogout->setVisible( btnList.contains(mBtnLogout));
    mBtnReboot->setVisible( btnList.contains(mBtnReboot));
    mBtnPowerOff->setVisible( btnList.contains(mBtnPowerOff));
    for(int i = 0; i < btnList.count(); i++)
    {
        ToolButton * btn = btnList.at(i);
        QPropertyAnimation * mBtnAnim = mBtnAnims.value(btn);

        mBtnAnim->setStartValue(QPoint(0,POWERBTNSIZE.height()*btnList.count()));
        mBtnAnim->setEndValue(QPoint(0,POWERBTNSIZE.height()*i));
        mAnimGroup->addAnimation(mBtnAnim);
    }

    this->setFixedSize(POWERBTNSIZE.width(), POWERBTNSIZE.height()*btnList.count());
}

void PowerMenu::onAnimFinished()
{
    if(!mExtend) this->hide();

    signalAnimFinished();
}

void PowerMenu::onComputerSleep()
{
    QProcess::startDetached("nde-leave --hibernate");
    this->window()->hide();
}

void PowerMenu::onComputerSwitch()
{
    //nde-leave
    QProcess::startDetached("nde-leave --switchUser");
    this->window()->hide();
}

void PowerMenu::onComputerLogout()
{
    //nde-leave --logout
    QProcess::startDetached("nde-leave --logout");
    this->window()->hide();
}

void PowerMenu::onComputerReboot()
{
    QProcess::startDetached("nde-leave --reboot");
    this->window()->hide();
}

void PowerMenu::onComputerPowerOff()
{
    QProcess::startDetached("nde-leave --shutdown");
    this->window()->hide();
}

bool PowerMenu::checkVirtualMachine()
{
    QProcess p;
    QStringList options;

    //检测是否是虚拟机vbox vmware，通过读取scsi文件中的设备判断虚拟机
    //    options << "-c" << "cat /proc/scsi/scsi | grep -i -e vbox -e vmware -e virtualbox";
    //读取cpuinfo，cpuinfo中包含hypervisor字段说明是虚拟机
    options << "-c" << "grep -q \"^flags.*\\ hypervisor\\ \" /proc/cpuinfo";
    p.start("/bin/bash", options);
    p.waitForFinished();

    //exitCode=0 说明命令执行成功，是虚拟机;exitCode=1表示命令执行失败，是实体机
    int exitCode = p.exitCode();
    return exitCode ? false:true;
}

bool PowerMenu::checkZteVersion()
{
    QSettings setting("/etc/os-release", QSettings::IniFormat, this);
    QString version = setting.value("VERSION").toString();
    QString versionId = setting.value("VERSION_ID").toString();
    return (version.contains("zte", Qt::CaseInsensitive) || versionId.contains("zte", Qt::CaseInsensitive));
}
