#ifndef NOTIFICATIONBAR_H
#define NOTIFICATIONBAR_H

#include <QApplication>
#include <QDesktopWidget>
#include <QFrame>
#include <QLineEdit>
#include <QToolButton>
#include <QPropertyAnimation>

#include "../panel/ndepanel.h"
#include "notifydisplaywidget.h"
#include "notifypopup.h"


//
class NotificationBarWidget : public QFrame
{
    Q_OBJECT

public:
    explicit NotificationBarWidget(INdePanelPlugin * plugin, QWidget *parent = nullptr);
    ~NotificationBarWidget();

    void realign();

private Q_SLOTS:
    void onDisplayLostFocus();
    void onNotifiesCountChanged();
    void onBtnClicked();
    void onAnimationFinished();
    void onPopupTriggered();

private:
    //icon btn in panel
    QToolButton * mBtn;

    bool mIsDisplayShow;
    //width of display widget
    int mDisplayWidth;
    //display widget of notify
    NotifyDisplayWidget * mDisplay;
    //popup the notify
    NotifyPopup * mPopup;
    //animation of display widget
    QPropertyAnimation * mAnimation;

    QRect mScreenRect;
    INdePanel::Position mPanelPos;
    INdePanelPlugin * mPlugin;
};


class NotificationBar : public QObject, public INdePanelPlugin
{
    Q_OBJECT
public:
    NotificationBar(const INdePanelPluginStartupInfo &startupInfo);
    ~NotificationBar();

    virtual QWidget *widget() { return mWidget; }
    virtual QString themeId() const { return "NotificationBar"; }
    virtual INdePanelPlugin::Flags flags() const { return SingleInstance|NeedsHandle; }

    void realign();

    virtual void settingsChanged();

private:
    NotificationBarWidget *mWidget;
    bool mVisible;
};

class NotificationBarLibrary: public QObject, public INdePanelPluginLibrary
{
    Q_OBJECT
    Q_PLUGIN_METADATA(IID "lxqt.org/Panel/PluginInterface/3.0")
    Q_INTERFACES(INdePanelPluginLibrary)
public:
    INdePanelPlugin *instance(const INdePanelPluginStartupInfo &startupInfo) const
    {
        return new NotificationBar(startupInfo);
    }
};

#endif // NOTIFICATIONBAR_H
