/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)LGPL2+
 *
 * Nde - a lightweight, Qt based, desktop toolset
 * https://lxqt.org
 *
 * Copyright: 2012 Razor team
 * Authors:
 *   Alec Moskvin <alecm@gmx.com>
 *   Petr Vanek <petr@scribus.info>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include "notifyd.h"

#include <QDebug>

Notifyd::Notifyd(NotifyDisplayWidget *displayWgt, NotifyPopup * popup,  QObject* parent)
    : QObject(parent),
      mId(0),
      m_displayWgt(displayWgt),
      m_popup(popup)
{
//    _timer = new QTimer(this);
//    _timer->setSingleShot(true);
//    _timer->setInterval(9*1000);
//    _timer->start();

//    connect(m_popup, &NotifyPopup::actionShowPopup, this, [this]{
//        _timer->stop();
//    });
//    connect(m_popup, &NotifyPopup::actionHidePopup, this, [this]{
//        _timer->setInterval(9*1000);
//        _timer->start();
//    });

//    connect(_timer, &QTimer::timeout, this, [this](){
//        m_popup->hide();
//        m_popup->startTimer();
//    });


    connect(m_displayWgt, &NotifyDisplayWidget::notifyClosed,
            this, &Notifyd::NotificationClosed);
    connect(m_displayWgt, &NotifyDisplayWidget::actionTriggered,
            this, &Notifyd::ActionInvoked);
    connect(m_popup,  &NotifyPopup::actionTriggered,
            this, &Notifyd::ActionInvoked);
}

Notifyd::~Notifyd()
{

}

void Notifyd::CloseNotification(uint id)
{
    m_displayWgt->onRmoveNotify(id, 3);
    Q_EMIT m_popup->NotificationClosed(id);
}

QStringList Notifyd::GetCapabilities()
{
    QStringList caps;
    caps
            << QSL("actions")
               // << "action-icons"
            << QSL("body")
               //             << QSL("body-hyperlinks")
               //             << QSL("body-images")
               //             << QSL("body-markup")
               // << "icon-multi"
               // << "icon-static"
            << QSL("persistence")
               // << "sound"
               ;
    return caps;
}

QString Notifyd::GetServerInformation(QString &vendor, QString &version, QString &spec_version)
{
    spec_version = QSL("1.2");
    version = QSL("1.0.0");
    vendor = QSL("newstartos.org");
    return QSL("nde-notificationbar");
}

uint Notifyd::Notify(const QString& app_name,
                     uint replaces_id,
                     const QString& app_icon,
                     const QString& summary,
                     const QString& body,
                     const QStringList& actions,
                     const QVariantMap& hints,
                     int expire_timeout,
                     bool noSave)
{
    uint ret;
    if (replaces_id == 0)
    {
        mId++;
        ret = mId;
    }
    else
    {
        ret = replaces_id;
    }

    NotifyMessage nm;
    nm.setMessage(app_name, ret, app_icon, summary, body, actions, hints,  expire_timeout, noSave);

    m_displayWgt->onAddNotify(nm);
    m_popup->setValues(nm);
    //调用通知弹窗功能,延迟主要为了处理系统启动时右下角的花屏现象。
//    QTimer::singleShot(200, [=](){
//        m_popup->setValues(nm);
//        _timer->start();
//    });

    return ret;
}
