#include "notifydisplaywidget.h"
#include "ui_notifydisplaywidget.h"

#include <KWindowEffects>
#include <QProcess>
#include <QDBusConnection>
#include <QDBusInterface>
#include <QStyle>
#include <QDebug>

NotifyDisplayWidget::NotifyDisplayWidget(INdePanelPlugin *plugin, QWidget *parent) :
    QWidget(parent),
    m_ui(new Ui::NotifyDisplayWidget),
    mPlugin(plugin),
    mCompositorOpen(false)
{
    m_ui->setupUi(this);
    setObjectName("NotifyDisplayWidget");
    this->setFocusPolicy(Qt::ClickFocus);
    this->setWindowFlags(Qt::FramelessWindowHint |
                         Qt::WindowStaysOnTopHint |
                         Qt::Tool |
                         Qt::X11BypassWindowManagerHint);

    this->setAttribute(Qt::WA_TranslucentBackground);
    this->setAttribute(Qt::WA_NoSystemBackground, false);

    KWindowEffects::enableBlurBehind(this->winId(), true);

    connect(m_ui->btn_clear, &QToolButton::clicked,this,&NotifyDisplayWidget::onClear);
    connect(m_ui->btn_remote, &QToolButton::clicked,this,&NotifyDisplayWidget::onRemote);
    connect(m_ui->btn_setting, &QToolButton::clicked,this,&NotifyDisplayWidget::onSetting);
    connect(m_ui->btn_screen_shot, &QToolButton::clicked,this,&NotifyDisplayWidget::onScreenShot);
    connect(m_ui->btn_multi_screen, &QToolButton::clicked,this,&NotifyDisplayWidget::onMultiScreen);

    m_notifyLayout = new QVBoxLayout();
    m_notifyLayout->setMargin(0);
    m_notifyLayout->setContentsMargins(0,0,0,0);
    m_notifyLayout->setSpacing(20);
    m_notifyLayout->addStretch();
    m_ui->scrollAreaContents_notify->setLayout(m_notifyLayout);

    mCompositorOpen = mPlugin->panel()->compositorOpen();
    this->setProperty("compositor", mCompositorOpen);

    //由于该功能未实现，因此先隐藏
    m_ui->btn_remote->hide();
    m_ui->btn_screen_shot->hide();
    m_ui->btn_multi_screen->hide();
    m_ui->btn_setting->hide();
}

NotifyDisplayWidget::~NotifyDisplayWidget()
{
    delete m_ui;
}

void NotifyDisplayWidget::realign()
{
    if(mCompositorOpen != mPlugin->panel()->compositorOpen())
    {
        mCompositorOpen =  mPlugin->panel()->compositorOpen();
        this->setProperty("compositor", mCompositorOpen);
        this->style()->unpolish(this);
        this->style()->polish(this);
    }
}

void NotifyDisplayWidget::onAddNotify(NotifyMessage nm)
{
    //
    int groupWidth = NOTIFICATIONBAR_WIDTH-10-this->layout()->contentsMargins().left() -this->layout()->contentsMargins().right();

    if(m_groups.contains(nm.m_id))
    {
        m_groups.value(nm.m_id)->addNotifyItem(nm);
    }
    else
    {
        NotifyGroup * group = new NotifyGroup(groupWidth, nm, m_ui->scrollAreaContents_notify);
        connect(group, &NotifyGroup::signalGroupClose, this, &NotifyDisplayWidget::onGroupClose);
        connect(group, &NotifyGroup::actionTriggered, this, &NotifyDisplayWidget::actionTriggered);

        m_notifyLayout->insertWidget(0,group);
        if(this->isVisible()) group->show();

        m_groups.insert(group->applicationID(), group);
    }

    Q_EMIT actionNewNotify();
}

void NotifyDisplayWidget::onRmoveNotify(uint id, uint reason)
{
    for(QHash<uint, NotifyGroup*>::iterator it = m_groups.begin(); it != m_groups.end(); it++)
    {
        if(it.key() == id)
        {
            NotifyGroup * group = it.value();
            group->hide();
            group->deleteLater();

            m_groups.remove(id);
            break;
        }
    }
}

bool NotifyDisplayWidget::event(QEvent *event)
{
    if(event->type() == QEvent::ActivationChange)
    {
        if(!this->isActiveWindow())
        {
            Q_EMIT lostFocus();
        }
    }
    return QWidget::event(event);
}

void NotifyDisplayWidget::resizeEvent(QResizeEvent *event)
{
    for(int i = 0; i < m_groups.keys().count(); i++)
    {
        uint key = m_groups.keys().at(i);
        NotifyGroup * group = m_groups.value(key);
        if(group)
        {
            int itemWidth = m_ui->scrollArea_notify->maximumViewportSize().width() -10;
            group->setFixedWidth(itemWidth);
        }
    }
    QWidget::resizeEvent(event);
}

void NotifyDisplayWidget::onGroupClose()
{
    NotifyGroup * group = qobject_cast<NotifyGroup*>(sender());
    if(group)
    {
        notifyClosed(group->applicationID(), 2);
        m_groups.remove(group->applicationID());
    }
}

void NotifyDisplayWidget::onClear()
{
    while(m_groups.count())
    {
        NotifyGroup * group = m_groups.begin().value();
        m_groups.erase(m_groups.begin());

        notifyClosed(group->applicationID(), 2);
        if(group->animationFinished()) group->onGroupClose();
    }
}

void NotifyDisplayWidget::onSetting()
{
    QProcess::startDetached("nde-systemsettings");
    Q_EMIT lostFocus();
}

void NotifyDisplayWidget::onRemote()
{
    //
    Q_EMIT lostFocus();
}

void NotifyDisplayWidget::onScreenShot()
{
    //
    Q_EMIT lostFocus();
}

void NotifyDisplayWidget::onMultiScreen()
{
    //
    Q_EMIT lostFocus();
}
