#ifndef NOTIFYGROUP_H
#define NOTIFYGROUP_H

#include <QWidget>
#include "notifyitem.h"
#include <QTimer>
#include "notifygrouptitle.h"

class NotifyGroup : public QWidget
{
    Q_OBJECT
    Q_PROPERTY(uint applicationID READ applicationID)
    Q_PROPERTY(int fixedHeight READ fixedHeight WRITE setFixedHeight)
public:
    explicit NotifyGroup(int width, NotifyMessage nm, QWidget *parent = nullptr);

    inline uint applicationID() const {return m_appID;}

    void addNotifyItem(NotifyMessage nm);
    bool animationFinished() const;

    //property
    inline int fixedHeight(){return this->height();}

Q_SIGNALS:
    void signalGroupClose(uint id);
    void actionTriggered(uint id, const QString &actionKey);

public Q_SLOTS:
    void onItemExpend(bool extend);
    void onItemClose();
    void onGroupClose();
    void onGroupExtend(bool extend);

protected:
    virtual void resizeEvent(QResizeEvent *event);

private Q_SLOTS:
    void slotAnimationCloseFinished();
    void slotAnimationExtendFinished();

private:
    int calculateHeight(bool extend);

private:
    bool m_extend;
    uint m_appID;
    QList<NotifyItem*> m_items;

    //界面元素
    NotifyGroupTitle * m_title;
    QVBoxLayout * m_layout;
    QWidget * m_maskWgt;

    //动画
    QPropertyAnimation * m_animationClose;
    QPropertyAnimation * m_animationExtend;
};

#endif // NOTIFYGROUP_H

