#include "notifygrouptitle.h"
#include "ui_notifygrouptitle.h"
#include <KWindowEffects>
#include <QStyleOption>
#include <QPainter>

#include <QDebug>

NotifyGroupTitle::NotifyGroupTitle(int width, NotifyMessage nm, QWidget *parent) :
    QWidget(parent),
    ui(new Ui::NotifyGroupTitle),
    m_extend(false)
{
    ui->setupUi(this);

    this->setObjectName("NotifyGroupTitle");
    this->setFocusPolicy(Qt::NoFocus);
    this->setAttribute(Qt::WA_TranslucentBackground);
    this->setAttribute(Qt::WA_NoSystemBackground, false);

    m_appID = nm.m_id;

    QFont f = ui->label_application->font();
    f.setBold(true);
    f.setPixelSize(14);
    ui->label_application->setFont(f);
    QFontMetrics fontMetrics = QFontMetrics(ui->label_application->font());
    QString elideName = fontMetrics.elidedText(nm.m_name, Qt::ElideRight, ui->label_application->width());
    ui->label_application->setText(elideName);

    f = ui->label_notice->font();
    f.setPixelSize(14);
    ui->label_notice->setFont(f);

    ui->label_icon->setFixedSize(NOTIFY_ICON_SIZE);
    if(!nm.m_icon.isNull())
    {
        ui->label_icon->setPixmap(nm.m_icon);
        ui->label_icon->show();
    }
    else {
        ui->label_icon->hide();
    }

    setNotifyCount(0);

    this->setFixedSize(width, this->sizeHint().height());

    ui->btn_close->hide();
    connect(ui->btn_close, &QToolButton::clicked, this, &NotifyGroupTitle::signalGroupClose);
}

NotifyGroupTitle::~NotifyGroupTitle()
{
    delete ui;
}

void NotifyGroupTitle::setNotifyCount(int count)
{
    ui->label_notice->setText(tr("%1 notifies contained.").arg(count));
}

void NotifyGroupTitle::paintEvent(QPaintEvent *event)
{
    QStyleOption opt;
    opt.init(this);
    QPainter p(this);
    style()->drawPrimitive(QStyle::PE_Widget, &opt, &p, this);
}

void NotifyGroupTitle::mousePressEvent(QMouseEvent *event)
{
    QWidget::mousePressEvent(event);
}

void NotifyGroupTitle::mouseReleaseEvent(QMouseEvent *event)
{
    m_extend = !m_extend;
    Q_EMIT signalGroupExtend(m_extend);
    QWidget::mouseReleaseEvent(event);
}

void NotifyGroupTitle::enterEvent(QEvent *event)
{
    ui->btn_close->show();
    QWidget::enterEvent(event);
}

void NotifyGroupTitle::leaveEvent(QEvent *event)
{
    ui->btn_close->hide();
    QWidget::leaveEvent(event);
}
