#include "notifyitem.h"
#include "../panel/ndepanel.h"

#include <KWindowEffects>
#include <QStyleOption>
#include <QPaintEvent>
#include <QPainter>

#include <QDebug>

NotifyItem::NotifyItem(const int itemWidth,
                       NotifyMessage nm,
                       QWidget *parent)
    : QWidget(parent)
{
    this->setObjectName("NotifyItem");
    this->setFocusPolicy(Qt::NoFocus);
    this->setAttribute(Qt::WA_TranslucentBackground);
    this->setAttribute(Qt::WA_NoSystemBackground, false);

    m_item_width = itemWidth;

    //main layout
    m_layout = new QHBoxLayout;
    m_layout->setContentsMargins(12,16,12,16);
    m_layout->setSpacing(10);

    //icon layout
    m_labelIcon = new QLabel(this);
    m_labelIcon->setObjectName("label_icon");
    m_labelIcon->setFixedSize(NOTIFY_ICON_SIZE);

    m_layout_icon = new QVBoxLayout;
    m_layout_icon->setMargin(0);
    m_layout_icon->setContentsMargins(0,0,0,0);
    m_layout_icon->setSpacing(0);
    m_layout_icon->addWidget(m_labelIcon);
    m_layout_icon->addStretch();

    //btn layout
    m_btn_close = new QToolButton(this);
    m_btn_close->setObjectName("btn_close");
    m_btn_close->setFixedSize(28,28);
    m_btn_close->setIconSize(QSize(28, 28));
    m_btn_close->hide();

    m_btn_extend = new QToolButton(this);
    m_btn_extend->setObjectName("btn_extend");
    m_btn_extend->setFixedSize(28,28);
    m_btn_extend->setCheckable(true);
    m_btn_extend->setIconSize(QSize(28, 28));
    m_btn_extend->setProperty("extend", false);
    m_btn_extend->hide();

    m_layout_btn = new QVBoxLayout;
    m_layout_btn->setMargin(0);
    m_layout_btn->setSpacing(0);
    m_layout_btn->setContentsMargins(0,0,0,0);
    m_layout_btn->addWidget(m_btn_close);
    m_layout_btn->addWidget(m_btn_extend);
    m_layout_btn->addStretch(0);

    //actions btns layout
    m_layout_actions = new QGridLayout;
    m_layout_actions->setContentsMargins(0,10,0,0);
    m_layout_actions->setSpacing(5);
    m_frame_actions = new QFrame(this);
    m_frame_actions->setLayout(m_layout_actions);

    //计算文字label的宽度
    int width = getLayoutTextWidth();

    //text layout
    m_labelApplication = new QLabel(this);
    m_labelApplication->setObjectName("label_application");
    m_labelApplication->setFixedWidth(width);
    QFont f = m_labelApplication->font();
    f.setBold(true);
    f.setPixelSize(14);
    m_labelApplication->setFont(f);
    m_labelApplication->setTextFormat(Qt::PlainText);

    m_labelSummary = new QLabel(this);
    m_labelSummary->setObjectName("label_summary");
    m_labelSummary->setFixedWidth(width);
    f = m_labelSummary->font();
    f.setBold(true);
    f.setPixelSize(14);
    m_labelSummary->setFont(f);
    m_labelSummary->setTextFormat(Qt::RichText);
    m_labelSummary->setWordWrap(true);

    m_labelBody = new ExtendLabel(this);
    m_labelBody->setObjectName("label_body");
    m_labelBody->setFixedWidth(width);
    f = m_labelBody->font();
    f.setBold(true);
    f.setPixelSize(14);
    m_labelBody->setFont(f);
    m_labelBody->setTextFormat(Qt::RichText);
    m_labelBody->setWordWrap(true);
    m_labelBody->setOpenExternalLinks(true);
    m_labelBody->hide();

    m_layout_text = new QVBoxLayout;
    m_layout_text->setMargin(0);
    m_layout_text->setSpacing(5);
    m_layout_text->setContentsMargins(0,0,0,0);
    m_layout_text->addWidget(m_labelApplication);
    m_layout_text->addWidget(m_labelSummary);
    m_layout_text->addWidget(m_labelBody);
    m_layout_text->addWidget(m_frame_actions);
    m_layout_text->addStretch(0);

    m_layout->addLayout(m_layout_icon);
    m_layout->addLayout(m_layout_text);
    m_layout->addLayout(m_layout_btn);
    m_layout->addStretch(0);
    this->setLayout(m_layout);

    setValues(nm);
    this->setFixedSize(m_item_width, getHeight(false));

    //animation
    m_animationClose = new QPropertyAnimation(this, "geometry", this);
    m_animationClose->setDuration(200);
    m_animationClose->setEasingCurve(QEasingCurve::InOutCubic);
    connect(m_animationClose, &QPropertyAnimation::finished, this, &NotifyItem::slotAnimationCloseFinished);

    m_animationExtend = new QPropertyAnimation(this, "fixedHeight", this);
    m_animationExtend->setDuration(200);

    m_animationExtendLabelBody = new QPropertyAnimation(m_labelBody, "fixedHeight", m_labelBody);
    m_animationExtendLabelBody->setDuration(200);

    m_animationExtendGroup = new QParallelAnimationGroup(this);
    m_animationExtendGroup->addAnimation(m_animationExtend);
    m_animationExtendGroup->addAnimation(m_animationExtendLabelBody);
    connect(m_animationExtendGroup, &QPropertyAnimation::finished, this, &NotifyItem::slotAnimationExtendFinished);

    m_animationMove = new QPropertyAnimation(this, "pos", this);
    m_animationMove->setDuration(200);
    connect(m_animationMove, &QPropertyAnimation::finished, this, &NotifyItem::slotAnimationMoveFinished);

    connect(m_btn_close, &QToolButton::clicked, this, &NotifyItem::slotClose);
    connect(m_btn_extend, &QToolButton::clicked,this,&NotifyItem::slotExtend);
}

void NotifyItem::setValues(NotifyMessage nm)
{
    m_notifyMsg = nm;

    m_labelIcon->setPixmap(nm.m_icon);

    //应用名
    QFontMetrics fontMetrics = QFontMetrics(m_labelApplication->font());
    QString elideApplication = fontMetrics.elidedText(nm.m_name, Qt::ElideRight, m_labelApplication->width());
    m_labelApplication->setText(elideApplication);

    //简单描述
    QFontMetrics fontMetrics1 = QFontMetrics(m_labelSummary->font());
    QString elideSummary = fontMetrics1.elidedText(nm.m_summary, Qt::ElideRight, m_labelSummary->width());
    m_labelSummary->setText(elideSummary);

    //详细描述
    QString formatted(nm.m_body);
    m_labelBody->setText(formatted.replace(QL1C('\n'), QStringLiteral("<br/>")));

    //动作
    //解析actions中的动作(actions<<acion<<desc，删除动作的描述
    int row = 0;
    int col = 0;
    for (int i = 0; i < nm.m_actions.count(); i += 2)
    {
        if(col > 1)
        {
            col = 0;
            row += 1;
        }

        QString key = nm.m_actions[i].first;
        QString desc = nm.m_actions[i].second;

        QToolButton * btn = new QToolButton(this);
        btn->setFixedSize(QSize((this->getLayoutTextWidth()- m_layout_actions->spacing())/2, 24));
        connect(btn, &QToolButton::clicked, this, &NotifyItem::slotActionBtnClicked);
        btn->setText(desc);
        btn->setProperty("action_key", key);
        m_layout_actions->addWidget(btn, row, col);

        col++;

        m_btn_actions.append(btn);
    }
    m_frame_actions->setVisible(m_btn_actions.count() ? true: false);
    m_frame_actions->setFixedHeight(getActionFrameHeight());
}

bool NotifyItem::animationFinished()
{
    return  m_animationClose->state() != QPropertyAnimation::Running &&
            m_animationExtendGroup->state() != QPropertyAnimation::Running;
}

int NotifyItem::getHeight()
{
    bool extend = m_btn_extend->isChecked();
    return getHeight(extend);
}

void NotifyItem::slotClose()
{
    if(m_animationClose->state() == QPropertyAnimation::Running)
    {
        return;
    }
    m_animationClose->setStartValue(this->geometry());
    m_animationClose->setEndValue(this->geometry().adjusted(-this->width(),0,-this->width(),0));
    m_animationClose->start();
}

void NotifyItem::slotExtend()
{
    bool extend = m_btn_extend->isChecked();

    if(m_animationExtendGroup->state() == QPropertyAnimation::Running)
    {
        m_animationExtendGroup->pause();
    }
    m_animationExtend->setStartValue(getHeight(!extend));
    m_animationExtend->setEndValue(getHeight(extend));
    int bodyHeight = m_labelBody->heightForWidth(m_labelBody->width());
    if(bodyHeight <= 0) bodyHeight = 22;
    m_animationExtendLabelBody->setStartValue(extend ? 0:bodyHeight);
    m_animationExtendLabelBody->setEndValue(extend ? bodyHeight:0);
    m_animationExtendGroup->start();

    if(extend)
    {
        m_labelBody->setVisible(extend);
        signalExpend(true);
    }

    m_btn_extend->setProperty("extend", extend);
    m_btn_extend->style()->unpolish(m_btn_extend);
    m_btn_extend->style()->polish(m_btn_extend);
}

void NotifyItem::slotMove(bool moveDown, QPoint startP, QPoint endP)
{
    this->setVisible(true);

    if(m_animationMove->state() == QPropertyAnimation::Running)
    {
        m_animationMove->pause();
    }

    m_animationMove->setStartValue(startP);
    m_animationMove->setEndValue(endP);
    m_animationMove->setDirection(moveDown ? QPropertyAnimation::Forward : QPropertyAnimation::Backward);
    m_animationMove->start();
}

void NotifyItem::paintEvent(QPaintEvent *event)
{
    QStyleOption opt;
    opt.init(this);
    QPainter p(this);
    style()->drawPrimitive(QStyle::PE_Widget, &opt, &p, this);
}

void NotifyItem::enterEvent(QEvent *event)
{
    m_btn_close->show();
    m_btn_extend->show();
    QWidget::enterEvent(event);
}

void NotifyItem::leaveEvent(QEvent *event)
{
    m_btn_close->hide();
    m_btn_extend->hide();
    QWidget::leaveEvent(event);
}

void NotifyItem::slotAnimationCloseFinished()
{
    QWidget * w = this->parentWidget();
    if(w && w->layout())
    {
        w->layout()->removeWidget(this);
    }
    this->hide();
    this->deleteLater();

    Q_EMIT signalClose();
}
void NotifyItem::slotAnimationShowFinished()
{
    this->parentWidget()->update();
}

void NotifyItem::slotAnimationExtendFinished()
{
    bool extend = m_btn_extend->isChecked();
    if(!extend)
    {
        m_labelBody->setVisible(false);
        signalExpend(false);
    }
}

void NotifyItem::slotAnimationMoveFinished()
{
    bool moveDown = (m_animationMove->direction() == QPropertyAnimation::Forward);
    this->setVisible(moveDown);
}

void NotifyItem::slotActionBtnClicked()
{
    QToolButton *btn = qobject_cast<QToolButton *>(sender());
    if(btn )
    {
        QString key = btn->property("action_key").toString();
        if(!key.isEmpty()) Q_EMIT actionTriggered(m_notifyMsg.m_id, key);
        qWarning() << m_notifyMsg.m_id << key;
    }
}

int NotifyItem::getHeight(bool extend)
{
    int applicationHeight = m_labelApplication->heightForWidth(m_labelApplication->width());
    if(applicationHeight <= 0) applicationHeight = 22;
    int summaryHeight = m_labelSummary->heightForWidth(m_labelSummary->width());
    if(summaryHeight <= 0) summaryHeight = 22;
    int bodyHeight = m_labelBody->heightForWidth(m_labelBody->width());
    if(bodyHeight <= 0) bodyHeight = 22;
    int actionsHeight = getActionFrameHeight();

    int height =  m_layout->contentsMargins().top() +
            applicationHeight +
            m_layout_text->spacing() +
            summaryHeight +
            m_layout_text->spacing() +
            actionsHeight +
            m_layout->contentsMargins().bottom();

    height += extend ? bodyHeight+m_layout_text->spacing():0;

    return height;
}

int NotifyItem::getActionFrameHeight()
{
    int actionsHeight = m_layout_actions->contentsMargins().top() +
            m_layout_actions->contentsMargins().bottom() +
            m_layout_actions->rowCount() * 24 +
            (m_layout_actions->rowCount() - 1) * m_layout_actions->spacing();

    if(m_btn_actions.count() <= 0)
        actionsHeight = 0;

    return actionsHeight;
}

int NotifyItem::getLayoutTextWidth()
{
    int width = m_item_width - m_layout->contentsMargins().left() -
            m_layout->contentsMargins().right() - m_layout->spacing()*2 -
            m_labelIcon->width() -  m_btn_close->width();
    return width;
}

//QString NotifyItem::getBtnExtendStyleSheet()
//{
//    QString btnExtendStyleSheet = QString("QToolButton#btn_extend{"
//                                          "image:url(:/res/%1-normal.png);"
//                                          "background-color: none;"
//                                          "}"
//                                          "QToolButton#btn_extend:hover{"
//                                          "image:url(:/res/%2-hover.png);"
//                                          "background-color: none;"
//                                          "}"
//                                          "QToolButton#btn_extend:pressed{"
//                                          "image:url(:/res/%3-click.png);"
//                                          "background-color: none;"
//                                          "}"
//                                          );

//    if(m_btn_extend->isChecked())
//    {
//        return btnExtendStyleSheet.arg("pull").arg("pull").arg("pull");
//    }
//    return btnExtendStyleSheet.arg("extend").arg("extend").arg("extend");
//}
