#ifndef NOTIFYITEM_H
#define NOTIFYITEM_H

#include <QWidget>
#include <QPixmap>
#include <QPropertyAnimation>
#include <QParallelAnimationGroup>
#include <QLabel>
#include <QFrame>
#include <QHBoxLayout>
#include <QToolButton>

#include "notifymessage.h"

#define NOTIFICATIONBAR_WIDTH 320

class ExtendLabel : public QLabel
{
    Q_OBJECT
    Q_PROPERTY(int fixedHeight READ fixedHeight WRITE setFixedHeight)
public:
    explicit ExtendLabel(QWidget *parent = nullptr):QLabel(parent){}
    inline int fixedHeight(){return this->height();}
};

class NotifyItem : public QWidget
{
    Q_OBJECT
    Q_PROPERTY(uint applicationID READ applicationID )
    Q_PROPERTY(int fixedHeight READ fixedHeight WRITE setFixedHeight)
public:
    explicit NotifyItem(const int itemWidth,
                        NotifyMessage nm,
                        QWidget *parent = nullptr);

    //
    void setValues(NotifyMessage nm);

    uint applicationID() const {return m_notifyMsg.m_id;}
    //property
    inline int fixedHeight(){return this->height();}

    bool animationFinished();

    int getHeight() ;

Q_SIGNALS:
    void signalExpend(bool expend);
    void signalClose();
    void actionTriggered(uint id, const QString &actionKey);

public Q_SLOTS:
    void slotClose();
    void slotExtend();
    void slotMove(bool moveDown, QPoint startP, QPoint endP);

protected:
    virtual void paintEvent(QPaintEvent *event);
    virtual void enterEvent(QEvent *event);
    virtual void leaveEvent(QEvent *event);

private Q_SLOTS:
    void slotAnimationCloseFinished();
    void slotAnimationShowFinished();
    void slotAnimationExtendFinished();
    void slotAnimationMoveFinished();
    void slotActionBtnClicked();

private:
    int getHeight( bool extend );
    int getActionFrameHeight();
    int getLayoutTextWidth();
//    QString getBtnExtendStyleSheet();

private:
    NotifyMessage m_notifyMsg;

    //界面尺寸
    int m_item_width;
    //界面元素
    QHBoxLayout * m_layout;
    QLabel * m_labelIcon;
    QVBoxLayout * m_layout_icon;
    QLabel * m_labelApplication;
    QLabel * m_labelSummary;
    ExtendLabel * m_labelBody;
    QVBoxLayout * m_layout_text;
    QToolButton * m_btn_close;
    QToolButton * m_btn_extend;
    QVBoxLayout * m_layout_btn;

    QFrame * m_frame_actions;
    QList<QToolButton*> m_btn_actions;
    QGridLayout * m_layout_actions;

    QPropertyAnimation * m_animationClose;
    QPropertyAnimation * m_animationShow;
    QPropertyAnimation * m_animationExtend;
    QPropertyAnimation * m_animationExtendLabelBody;
    QParallelAnimationGroup * m_animationExtendGroup;
    QPropertyAnimation * m_animationMove;
};

#endif // NOTIFYITEM_H
