#include "notifypopup.h"

#include <QHBoxLayout>
#include <QEvent>
#include <KF5/KWindowSystem/KWindowEffects>
#include <QApplication>
#include <QDesktopWidget>
#include <QDBusConnection>
#include <QDBusInterface>
#include <QLineEdit>
#include <QDebug>
#include <QListWidget>
#include <QScreen>
#include <QScrollBar>

#define WIDTH 322
#define MAXHEIGHT 432

NotifyPopup::NotifyPopup(INdePanelPlugin *plugin, QWidget *parent) :
    QWidget(parent),
    mPlugin(plugin),
    mCompositorOpen(false)
{
    this->setObjectName("NotifyPopup");
    this->setWindowFlags(Qt::FramelessWindowHint |
                         Qt::WindowStaysOnTopHint |
                         Qt::Tool |
                         Qt::X11BypassWindowManagerHint);
    this->setAttribute(Qt::WA_TranslucentBackground);
    this->setAttribute(Qt::WA_NoSystemBackground, false);
    KWindowEffects::enableBlurBehind(this->winId(), true);

    m_closeBtn = new QToolButton(this);
    m_closeBtn->setEnabled(true);
    m_closeBtn->setFixedSize(30,30);
    m_closeBtn->setObjectName("btn_close");
    m_closeBtn->setIconSize(QSize(30,30));
    connect(m_closeBtn, &QToolButton::clicked, this, &NotifyPopup::slotClose);

    QHBoxLayout * closeLayout = new QHBoxLayout;
    closeLayout->setMargin(0);
    closeLayout->setContentsMargins(0,0,0,0);
    closeLayout->setSpacing(0);
    closeLayout->addStretch();
    closeLayout->addWidget(m_closeBtn);

    QRect availableRect = qApp->primaryScreen()->availableGeometry();
    this->setMaximumSize(qMin(availableRect.width(), WIDTH), qMin(availableRect.height(), MAXHEIGHT));

    m_scrollAreaWidgetLayout = new QVBoxLayout;
    m_scrollAreaWidgetLayout->setMargin(0);
    m_scrollAreaWidgetLayout->setContentsMargins(5,0,10,0);
    m_scrollAreaWidgetLayout->setSpacing(5);

    m_scrollAreaWidget = new QWidget(this);
    m_scrollAreaWidget->setObjectName("m_scrollAreaWidget");
    m_scrollAreaWidget->setLayout(m_scrollAreaWidgetLayout);

    m_scrollArea = new QScrollArea(this);
    m_scrollArea->setObjectName("m_scrollarea");
    m_scrollArea->setWidget(m_scrollAreaWidget);
    m_scrollArea->setWidgetResizable(true);
    m_scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    m_scrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);

    mBackground = new QWidget(this);
    mBackground->setObjectName("NotifyPopupBackgroundWidget");

    m_cbox_interval = new TimerComboBox;
    m_cbox_interval->setFixedHeight(35);
    m_cbox_interval->setObjectName("ComboBox_timer");
    QStringList slist = {tr("Always show"), tr("Show after 1 hour"), tr("Show after 2 hours"), tr("Show after 4 hours")/*, tr("Never")*/};
    m_cbox_interval->addItems(slist);
    connect(m_cbox_interval, SIGNAL(currentIndexChanged(int)), this, SLOT(on_m_cbox_interval_indexChanged(int)));
    connect(m_cbox_interval, &TimerComboBox::actionShowPopup, this, &NotifyPopup::actionShowPopup);
    connect(m_cbox_interval, &TimerComboBox::actionHidePopup, this, &NotifyPopup::actionHidePopup);

    connect(m_cbox_interval, &TimerComboBox::actionShowPopup, this, [this]{ m_isComboxOpen = true;});
    connect(m_cbox_interval, &TimerComboBox::actionHidePopup, this, [this]{
        m_isComboxOpen = false;
        if(m_scrollAreaWidgetLayout->count() == 0)
            slotClose();
    });

    m_scrollAreaLayout = new QVBoxLayout;
    m_scrollAreaLayout->setContentsMargins(0,0,0,0);
    m_scrollAreaLayout->setSpacing(0);
    m_scrollAreaLayout->addWidget(m_scrollArea);

    //main layout
    m_vlayout = new QVBoxLayout;
    m_vlayout->setContentsMargins(0,0,0,0);
    m_vlayout->setSpacing(0);
    m_vlayout->addLayout(closeLayout);
    m_vlayout->addLayout(m_scrollAreaLayout);
    m_vlayout->addWidget(m_cbox_interval);
    mBackground->setLayout(m_vlayout);

    QHBoxLayout * layout = new QHBoxLayout;
    layout->setContentsMargins(0,0,0,0);
    layout->setSpacing(0);
    layout->addWidget(mBackground);
    this->setLayout(layout);

    mCompositorOpen = mPlugin->panel()->compositorOpen();
    mBackground->setProperty("compositor", mCompositorOpen);

    m_timerInterval = new QTimer;
    m_fixedHeight = m_cbox_interval->height() + m_closeBtn->height() + m_scrollAreaWidgetLayout->contentsMargins().top() +
            m_scrollAreaWidgetLayout->contentsMargins().bottom();

    m_height += m_fixedHeight;

    m_setting = new Nde::Settings(QLatin1String("panel"), this);
    connect(m_setting, &Nde::Settings::settingsChanged, this, &NotifyPopup::onSettingsChanged);
}

NotifyPopup::~NotifyPopup()
{

}

void NotifyPopup::setValues(NotifyMessage nm)
{
    m_notifyMsg = nm;
    bool isScreensaver = false;
    QDBusMessage message = QDBusMessage::createMethodCall(QString::fromLatin1("org.nde.ScreenSaver"),
                                                          QString::fromLatin1("/ScreenSaver"),
                                                          QString::fromLatin1("org.nde.ScreenSaver"),
                                                          QString::fromLatin1("getLockState"));
    QDBusMessage reply = QDBusConnection::sessionBus().call(message);
    if(reply.arguments().count() > 0)
    {
        isScreensaver = reply.arguments().first().toBool();
    }

    if(isScreensaver) return;

    this->layout()->setEnabled(false);
    NotifyPopupItem * npitem = new NotifyPopupItem(nm);
    connect(npitem, &NotifyPopupItem::actionClose, this, &NotifyPopup::onClearItem);
    connect(npitem, &NotifyPopupItem::actionTriggered, this, &NotifyPopup::actionTriggered);
    m_scrollAreaWidgetLayout->insertWidget(0, npitem);
    m_notifyPopupItems.append(npitem);

    (m_scrollAreaWidgetLayout->count() == 1) ? m_closeBtn->hide() : m_closeBtn->show();

    m_height = m_height + npitem->height() + m_scrollAreaWidgetLayout->spacing();

    this->layout()->setEnabled(true);

    mPlugin->panel()->willShowWindow(this);

    if(m_isShow)
    {
        if(!m_isCurrentPopup)
            m_cbox_interval->setCurrentIndex(0);
        this->show();
    }

    m_scrollArea->verticalScrollBar()->setSliderPosition(0);
    this->setFixedHeight(qMin(MAXHEIGHT,m_height));
    adjustPosition();
}

void NotifyPopup::realign()
{
    if(mCompositorOpen != mPlugin->panel()->compositorOpen())
    {
        mCompositorOpen = mPlugin->panel()->compositorOpen();
        mBackground->setProperty("compositor", mCompositorOpen);
    }
}

int NotifyPopup::getTimeInterval(int index)
{
    //Always, 1h ,2h , 4h, Never
    const int n = 5;
    static int intervals[n] = { 1, 60*60, 120*60, 240*60, 0};
    return index < n ? intervals[index] : intervals[0];
}

bool NotifyPopup::event(QEvent *event)
{
    if(event->type() == QEvent::ActivationChange)
    {
        if(!this->isActiveWindow())
        {
            this->hide();
            startTimer();
        }
    }
    return QWidget::event(event);
}

void NotifyPopup::slotClose()
{
    QList<NotifyPopupItem*> items = m_scrollAreaWidget->findChildren<NotifyPopupItem*>();
    for (NotifyPopupItem * item : items) {
        m_scrollAreaWidgetLayout->removeWidget(item);
        item->deleteLater();
    }

    this->setFixedHeight( m_fixedHeight + 30);

    if(!m_isComboxOpen)
    {
        this->hide();
        startTimer();
    }

    m_notifyPopupItems.clear();
    m_height = m_fixedHeight;
}

void NotifyPopup::on_m_cbox_interval_indexChanged(int index)
{
    //最后一个是永不显示
//    if(index == m_cbox_interval->count() - 1)
//    {
//        m_setting->beginGroup("notificationbar");
//        m_setting->setValue("isShowpopup", false);
//        m_setting->endGroup();
//        m_isShow = false;
//    }else
//    {
        m_timeIndex = index;
//    }
}

void NotifyPopup::onSettingsChanged()
{
    m_setting->beginGroup("notificationbar");
    if(m_setting->contains("isShowpopup"))
        m_isShow = m_setting->value("isShowpopup").toBool();
    m_setting->endGroup();
}

void NotifyPopup::onClearItem()
{
    NotifyPopupItem* item = qobject_cast<NotifyPopupItem*>(sender());
    m_height = m_height - item->height() - m_scrollAreaWidgetLayout->spacing();

    m_scrollAreaWidgetLayout->takeAt(m_scrollAreaWidgetLayout->indexOf(item));

    if(m_scrollAreaWidgetLayout->count() == 0)
    {
        slotClose();
    }
    else {
        this->setFixedHeight(qMin(MAXHEIGHT,m_height));
    }

    (m_scrollAreaWidgetLayout->count() == 1) ? m_closeBtn->hide() : m_closeBtn->show();

    m_notifyPopupItems.removeOne(item);
    adjustPosition();
}

void NotifyPopup::adjustPosition()
{
    //计算位置
    QRect rect = qApp->desktop()->screenGeometry();
    int pos_x,pos_y;
    switch (mPlugin->panel()->position())
    {
    case INdePanel::PositionTop:
        rect.adjust(0, mPlugin->panel()->globalGeometry().height(),0,0);
        pos_x = rect.width()-this->width();
        pos_y = mPlugin->panel()->globalGeometry().height()+4;
        break;
    case INdePanel::PositionBottom:
        rect.adjust(0,0,0,-mPlugin->panel()->globalGeometry().height());
        pos_x = rect.width()-this->width();
        pos_y = rect.height()-this->height()-4;
        break;

    case INdePanel::PositionLeft:
        rect.adjust(mPlugin->panel()->globalGeometry().width(),0,0,0);
        pos_x = mPlugin->panel()->globalGeometry().width()+4;
        pos_y = rect.height()-this->height();
        break;

    case INdePanel::PositionRight:
        rect.adjust(0,0,-mPlugin->panel()->globalGeometry().width(),0);
        pos_x = rect.width()-this->width()-4;
        pos_y = rect.height()-this->height();
        break;
    }
    this->move(pos_x,pos_y);
}

void NotifyPopup::startTimer()
{
    if(m_timeIndex == 0)
        return;

//    if(m_timeIndex == m_cbox_interval->count() - 1)
//    {
//        m_setting->beginGroup("notificationbar");
//        m_setting->setValue("isShowpopup", false);
//        m_setting->endGroup();
//        return;
//    }
    m_isShow = false;
    m_isCurrentPopup = false;

    int interval = getTimeInterval(m_timeIndex);
    m_timerInterval->singleShot(interval * 1000, this ,[this]{
        m_isShow = true;
        m_isCurrentPopup = true;
        m_cbox_interval->setCurrentIndex(0);
    });
}

void NotifyPopup::NotificationClosed(uint id)
{
    for(int i = 0; i < m_notifyPopupItems.count(); i++)
    {
        NotifyPopupItem * item = m_notifyPopupItems.at(i);
        if(item->getNotifyMessage().m_id == id)
            item->slotClose();
    }
}
void TimerComboBox::hidePopup()
{
    QComboBox::hidePopup();
    Q_EMIT actionHidePopup();
}

void TimerComboBox::showPopup()
{
    QComboBox::showPopup();
    Q_EMIT actionShowPopup();
}
