#include "notifypopupitem.h"
#include <QDBusConnection>
#include <QDBusInterface>
#include <QTimer>
#include <QStyleOption>
#include <QPainter>
#include <QDebug>

#define DEFAUT_CLOSE_TIME 9 * 1000

NotifyPopupItem::NotifyPopupItem(NotifyMessage msg, QWidget *parent) :
    QWidget(parent)
{
    this->setObjectName("NotifyPopupItem");
    m_labelIcon = new QLabel(this);
    m_labelIcon->setObjectName("label_icon");
    m_labelIcon->setFixedSize(NOTIFY_ICON_SIZE);

    m_layout_icon = new QVBoxLayout;
    m_layout_icon->setMargin(0);
    m_layout_icon->setContentsMargins(0,0,0,0);
    m_layout_icon->setSpacing(0);
    m_layout_icon->addWidget(m_labelIcon);
    m_layout_icon->addStretch();

    //actions btns layout
    m_layout_actions = new QGridLayout;
    m_layout_actions->setContentsMargins(0,10,0,0);
    m_layout_actions->setSpacing(5);
    m_frame_actions = new QFrame(this);
    m_frame_actions->setLayout(m_layout_actions);

    //text layout
    m_labelApplication = new QLabel(this);
    m_labelApplication->setObjectName("label_application");
    QFont f = m_labelApplication->font();
    f.setBold(true);
    f.setPixelSize(14);
    m_labelApplication->setFont(f);
    m_labelApplication->setTextFormat(Qt::PlainText);
    m_labelApplication->setMinimumWidth(200);

    m_labelSummary = new QLabel(this);
    m_labelSummary->setObjectName("label_summary");
    f = m_labelSummary->font();
    f.setBold(true);
    f.setPixelSize(14);
    m_labelSummary->setFont(f);
    m_labelSummary->setTextFormat(Qt::RichText);
    m_labelSummary->setWordWrap(true);
    m_labelSummary->setFixedWidth(200);

    m_labelBody = new QLabel(this);
    m_labelBody->setObjectName("label_body");
    f = m_labelBody->font();
    f.setBold(true);
    f.setPixelSize(14);
    m_labelBody->setFont(f);
    m_labelBody->setTextFormat(Qt::RichText);
    m_labelBody->setWordWrap(true);
    m_labelBody->setOpenExternalLinks(true);
    m_labelBody->setFixedWidth(200);

    m_layout_text = new QVBoxLayout;
    m_layout_text->setMargin(0);
    m_layout_text->setSpacing(5);
    m_layout_text->setContentsMargins(0,0,0,0);
    m_layout_text->addWidget(m_labelApplication);
    m_layout_text->addWidget(m_labelSummary);
    m_layout_text->addWidget(m_labelBody);
    m_layout_text->addWidget(m_frame_actions);
    m_frame_actions->hide();
    m_frame_actions->setAttribute(Qt::WA_TranslucentBackground);

    //btn layout
    m_btn_close = new QToolButton(this);
    m_btn_close->setObjectName("btn_close");
    m_btn_close->setFixedSize(28,28);
    m_btn_close->setIconSize(QSize(28, 28));
    connect(m_btn_close, &QToolButton::clicked, this, &NotifyPopupItem::slotClose);

    m_layout_btn = new QVBoxLayout;
    m_layout_btn->setMargin(0);
    m_layout_btn->setSpacing(0);
    m_layout_btn->setContentsMargins(0,0,0,0);
    m_layout_btn->addWidget(m_btn_close);
    m_layout_btn->addStretch(0);

//    //main layout
    m_layout = new QHBoxLayout;
    m_layout->setContentsMargins(12,16,12,16);
    m_layout->setSpacing(10);
    m_layout->addLayout(m_layout_icon);
    m_layout->addLayout(m_layout_text);
    m_layout->addLayout(m_layout_btn);

    this->setLayout(m_layout);

    setValues(msg);
}

void NotifyPopupItem::paintEvent(QPaintEvent *event)
{
    Q_UNUSED(event)
    QStyleOption opt;
    opt.init(this);
    QPainter p(this);
    style()->drawPrimitive(QStyle::PE_Widget, &opt, &p, this);
}

void NotifyPopupItem::setValues(NotifyMessage nm)
{
    m_notifyMsg = nm;
    int height = 0;
    m_labelIcon->setPixmap(nm.m_icon);
    m_labelApplication->setText(nm.m_name);

    //简单描述
    m_labelSummary->setText(nm.m_summary);

    //详细描述
    QString formatted(nm.m_body);
    m_labelBody->setText(formatted.replace(QLatin1Char('\n'), QStringLiteral("<br/>")));

    //动作
    //解析actions中的动作(actions<<acion<<desc，删除动作的描述
    while(m_btn_actions.count())
    {
        QToolButton * btn = m_btn_actions.takeFirst();
        m_layout_actions->removeWidget(btn);
        delete  btn;
    }
    int row = 0;
    int col = 0;
    for (int i = 0; i < nm.m_actions.count(); i++)
    {
        int btnHeight = 24;
        if(col > 1)
        {
            col = 0;
            row += 1;
            height += btnHeight;
        }

        QString key = nm.m_actions[i].first;
        QString desc = nm.m_actions[i].second;
        QToolButton * btn = new QToolButton(this);
        btn->setFixedSize(QSize(100, btnHeight));
        connect(btn, &QToolButton::clicked, this, &NotifyPopupItem::slotActionBtnClicked);
        btn->setText(desc);
        btn->setProperty("action_key", key);
        m_layout_actions->addWidget(btn, row, col);

        col++;

        m_btn_actions.append(btn);
        m_frame_actions->show();
    }
    m_frame_actions->setVisible(m_btn_actions.count());

    int timeout = nm.m_timeout;
    timeout = ((nm.m_timeout == -1) ? DEFAUT_CLOSE_TIME : nm.m_timeout);

    if(timeout)
        QTimer::singleShot(timeout,this,&NotifyPopupItem::slotClose);

    if(!nm.m_actions.isEmpty())
        height += m_frame_actions->height() + m_layout_actions->contentsMargins().top() + m_layout_actions->contentsMargins().bottom();

    //计算高度
    height += m_layout->contentsMargins().top() + m_layout->contentsMargins().bottom()+
            m_labelApplication->height() + (m_layout_text->count() - 1) * m_layout_text->spacing() +
            m_labelSummary->height() + m_labelBody->heightForWidth(200);

    this->setFixedHeight(height);
}

void NotifyPopupItem::slotClose()
{
    Q_EMIT actionClose();
    this->hide();
}
void NotifyPopupItem::slotActionBtnClicked()
{
    QToolButton *btn = qobject_cast<QToolButton *>(sender());
    if(btn)
    {
        QString key = btn->property("action_key").toString();
        if(!key.isEmpty()) Q_EMIT actionTriggered(m_notifyMsg.m_id, key);
    }
}

NotifyMessage NotifyPopupItem::getNotifyMessage()
{
    return m_notifyMsg;
}

