/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)LGPL2+
 *
 * Nde - a lightweight, Qt based, desktop toolset
 * https://lxqt.org
 *
 * Copyright: 2010-2012 Razor team
 * Authors:
 *   Petr Vanek <petr@scribus.info>
 *   Kuzma Shapran <kuzma.shapran@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include "ndequicklaunch.h"
#include "quicklaunchbutton.h"
#include "quicklaunchaction.h"
#include "../panel/indepanelplugin.h"
#include <QDesktopServices>
#include <QDragEnterEvent>
#include <QFileIconProvider>
#include <QFileInfo>
#include <QLabel>
#include <QMessageBox>
#include <QToolButton>
#include <QUrl>
#include <QDebug>
#include <XdgDesktopFile>
#include <XdgIcon>
#include <Nde/GridLayout>
#include <QStandardPaths>
#include "../panel/pluginsettings.h"
#include <QApplication>
#include <QProcess>

NdeQuickLaunch::NdeQuickLaunch(INdePanelPlugin *plugin, QWidget* parent) :
    QFrame(parent),
    mPlugin(plugin),
    mPlaceHolder(0)
{
    QBoxLayout * layout = new QBoxLayout(QBoxLayout::LeftToRight);
    layout->setSpacing(0);
    layout->setContentsMargins(0,0,0,0);
    layout->setMargin(0);
    setLayout(layout);

    //快捷应用按钮layout
    mBtnsLayout = new Nde::GridLayout();
    //快捷应用按钮widget
    mBtnsWidget = new QWidget(this);
    mBtnsWidget->setMinimumSize(QSize(3,3));
    mBtnsWidget->setLayout(mBtnsLayout);
    mBtnsWidget->setAcceptDrops(true);
    mBtnsWidget->installEventFilter(this);

    //翻页按钮相关
    int size = getPanelSize();
    mPrviBtn = new StatusNotifierArrow(this);
    mPrviBtn->setObjectName("QuickLaunch_prvi_btn");
    mPrviBtn->setFixedSize(QSize(size/2,size/2));
    mPrviBtn->setToolButtonStyle(Qt::ToolButtonIconOnly);
    mPrviBtn->setOrigin(Qt::TopLeftCorner);
    connect(mPrviBtn, &QToolButton::clicked,this,&NdeQuickLaunch::slotPrviBtnClicked);

    mNextBtn = new StatusNotifierArrow(this);
    mNextBtn->setObjectName("QuickLaunch_next_btn");
    mNextBtn->setFixedSize(QSize(size/2,size/2));
    mNextBtn->setToolButtonStyle(Qt::ToolButtonIconOnly);
    mPrviBtn->setOrigin(Qt::TopLeftCorner);
    connect(mNextBtn, &QToolButton::clicked,this,&NdeQuickLaunch::slotNextBtnClicked);

    //分割线
    mLine = new QFrame(this);
    mLine->setObjectName("line");
    mLine->setFrameShape(QFrame::VLine);
    mLine->setFrameShadow(QFrame::Sunken);
    mLine->setLineWidth(1);
    mLine->setMidLineWidth(0);
    mLine->setFixedSize(17,size); //这里的17计算方法:qss中的margin-left+margin-right

    //翻页按钮Layout
    mPageLayout = new QBoxLayout(QBoxLayout::TopToBottom);
    mPageLayout->setSpacing(0);
    mPageLayout->setContentsMargins(0,0,0,0);
    mPageLayout->setMargin(2);
    mPageLayout->addWidget(mPrviBtn);
    mPageLayout->addWidget(mNextBtn);

    //翻页按钮widget
    mPageWidget = new QWidget(this);
    mPageWidget->setMinimumSize(QSize(5,5));
    mPageWidget->setLayout(mPageLayout);

    layout->addWidget(mBtnsWidget);
    layout->addWidget(mPageWidget);
    layout->addWidget(mLine);

    //每一页的快速启动应用数量
    mCurrentPage = 0;
    mPageCount = mPlugin->settings()->value("page_count", 5).toInt();

    rebuildButtonQuick();
}


NdeQuickLaunch::~NdeQuickLaunch()
{
}


int NdeQuickLaunch::indexOfButton(QuickLaunchButton* button) const
{
    return mBtnsLayout->indexOf(button);
}


int NdeQuickLaunch::countOfButtons() const
{
    return mBtnsLayout->count();
}


void NdeQuickLaunch::realign()
{
    mBtnsLayout->setEnabled(false);
    INdePanel *panel = mPlugin->panel();

    QBoxLayout * layout = qobject_cast<QBoxLayout*>(this->layout());
    if (mPlaceHolder)
    {
        mBtnsLayout->setColumnCount(1);
        mBtnsLayout->setRowCount(1);
        mPageWidget->hide();
        mLine->show();

        if (panel->isHorizontal())
        {
            layout->setDirection(QBoxLayout::LeftToRight);

            mLine->setFrameShape(QFrame::VLine);
            mLine->setFixedSize(17,getPanelSize());
            mPlaceHolder->setAlignment(Qt::AlignLeft|Qt::AlignVCenter);
            QString text = tr("Drop application\nicons here");
            mPlaceHolder->setText(text);
        }
        else
        {
            layout->setDirection(QBoxLayout::TopToBottom);

            mLine->setFrameShape(QFrame::HLine);
            mLine->setFixedSize(getPanelSize(),17);
            mPlaceHolder->setAlignment(Qt::AlignTop|Qt::AlignHCenter);
            QString text = tr("Drop application\nicons here");
            text = text.split("").join("\n");
            if(text.startsWith("\n")) text.remove(0,1);
            if(text.endsWith("\n")) text.remove(-1,1);
            mPlaceHolder->setText(text);
        }
    }
    else
    {
        int pages = getPages();
        int curPageItemNums = mPageCount;
        //是最后一页,则获取最后一页的应用数量
        if(pages - 1  == mCurrentPage)
        {
            curPageItemNums = mBtnsLayout->count() - mCurrentPage * mPageCount;
        }
        //查看panel的位置
        if (panel->isHorizontal())
        {
            layout->setDirection(QBoxLayout::LeftToRight);

            mBtnsLayout->setRowCount(1);
            mBtnsLayout->setColumnCount(curPageItemNums);

            mPageLayout->setDirection(QBoxLayout::TopToBottom);
            mPrviBtn->setOrigin(Qt::TopLeftCorner);
            mNextBtn->setOrigin(Qt::TopLeftCorner);

            mLine->setFrameShape(QFrame::VLine);
            mLine->setFixedSize(17,getPanelSize());

            //若页数大于1，显示翻页按钮。否则显示一条线
            if(pages > 1)
            {
                //
                mPageWidget->show();
                mLine->hide();
            }
            else
            {
                //
                mPageWidget->hide();
                mLine->show();
            }
        }
        else
        {
            layout->setDirection(QBoxLayout::TopToBottom);

            mBtnsLayout->setColumnCount(1);
            mBtnsLayout->setRowCount(curPageItemNums);

            mPageLayout->setDirection(QBoxLayout::LeftToRight);
            mPrviBtn->setOrigin(Qt::BottomLeftCorner);
            mNextBtn->setOrigin(Qt::BottomLeftCorner);

            mLine->setFrameShape(QFrame::HLine);
            mLine->setFixedSize(getPanelSize(),17);

            //若页数大于1，显示翻页按钮。否则显示一条线
            if(pages > 1)
            {
                mPageWidget->show();
                mLine->hide();
            }
            else
            {
                mPageWidget->hide();
                mLine->show();
            }
        }

        //计算应该显示的应用的序号
        int beginShowIndex = mCurrentPage * mPageCount;
        int endShowIndex = (mCurrentPage+1) * mPageCount - 1;
        for(int i = 0; i < mBtnsLayout->count() ;i++)
        {
             QLayoutItem *item = mBtnsLayout->itemAt(i);
             if(item)
             {
                 if(i <= endShowIndex && i >= beginShowIndex){
                     item->widget()->show();
                 }
                 else {
                     item->widget()->hide();
                 }
             }
             else {
                 break;
             }
        }
    }
    for(int i = 0; i < mBtnsLayout->count() ;i++)
    {
        QuickLaunchButton *btn = qobject_cast<QuickLaunchButton*>(mBtnsLayout->itemAt(i)->widget());
        if(!btn)
            continue;
        btn->setIconSize(QSize(mPlugin->panel()->iconSize(), mPlugin->panel()->iconSize()));
    }
    mBtnsLayout->setEnabled(true);
}

void NdeQuickLaunch::addButtonQuick(XdgDesktopFile xdg)
{
    if (!xdg.isSuitable())
    {
        qDebug() << "XdgDesktopFile" << xdg.fileName() << "is not applicable";
        return;
    }

    saveDesktopFile(xdg);
    addButton(new QuickLaunchAction(&xdg, this));

    //添加应用切换到最后一页
    int pages = getPages();
    mCurrentPage = pages - 1;
    realign();

    //
    saveSettings();
}

void NdeQuickLaunch::rebuildButtonQuick()
{
    while(mBtnsLayout->count())
    {
        QuickLaunchButton *btn = qobject_cast<QuickLaunchButton*>(mBtnsLayout->takeAt(0)->widget());
        if(btn)
        {
            mBtnsLayout->removeWidget(btn);
            btn->deleteLater();
        }
    }

    const auto apps = mPlugin->settings()->readArray("apps");
    for (const QMap<QString, QVariant> &app : apps)
    {
        QString desktop = app.value("desktop", "").toString();
        if (!desktop.isEmpty())
        {
            XdgDesktopFile xdg;
            if (!xdg.load(desktop))
            {
                qDebug() << "XdgDesktopFile" << desktop << "is not valid";
                continue;
            }
            if (!xdg.isSuitable())
            {
                qDebug() << "XdgDesktopFile" << desktop << "is not applicable";
                continue;
            }

            //检测application目录中是否存在对应的desktop文件
            QString appSystemPath = "/usr/share/applications/";
            QFileInfo desktopFile(desktop);
            QString appDesktopIcon = appSystemPath+desktopFile.fileName();

            if(!QFile::exists(appDesktopIcon))
            {
                qDebug() << "XdgDesktopFile" << desktop << "is removed,not exists!";
                continue;
            }


            addButton(new QuickLaunchAction(&xdg, this));
        }
    } // for

    saveSettings();

    //如果没有项目，就显示提示，否则删除提示
    if (mBtnsLayout->count() <= 0)
        showPlaceHolder();

    int pages = getPages();
    if(mCurrentPage > pages-1)
        mCurrentPage = pages-1;

    realign();
}

bool NdeQuickLaunch::eventFilter(QObject *watched, QEvent *event)
{
    if(watched == mBtnsWidget)
    {
        if(event->type() == QEvent::DragMove)
        {
            //
            QDragMoveEvent * e = static_cast<QDragMoveEvent *>(event);
            if (e->mimeData()->hasUrls())
            {
                e->acceptProposedAction();
            }
            else if (e->source() && e->source()->parent() == this)
            {
                e->acceptProposedAction();
            }
        }
        else if(event->type() == QEvent::DragEnter)
        {
            //
            QDragEnterEvent * e = static_cast<QDragEnterEvent *>(event);
            if (e->mimeData()->hasUrls())
            {
                e->acceptProposedAction();
            }
            else if (e->source() && e->source()->parent() == this)
            {
                e->acceptProposedAction();
            }
        }
        else if(event->type() == QEvent::Drop)
        {
            //
            QDropEvent * e = static_cast<QDropEvent *>(event);
            const auto urls = e->mimeData()->urls().toSet();
            for (const QUrl &url : urls)
            {
                QString fileName(url.isLocalFile() ? url.toLocalFile() : url.url());
                QFileInfo fi(fileName);
                XdgDesktopFile xdg;

                if (xdg.load(fileName))
                {
                    if (xdg.isSuitable())
                    {
                        saveDesktopFile(xdg);
                        addButton(new QuickLaunchAction(&xdg, this));

                        //添加应用切换到最后一页
                        int pages = getPages();
                        mCurrentPage = pages - 1;
                        realign();
                    }
                }
                //只能拖动desktop文件到快速启动栏
//                else if (fi.exists() && fi.isExecutable() && !fi.isDir())
//                {
//                    addButton(new QuickLaunchAction(fileName, fileName, "", this));
//                }
//                else if (fi.exists())
//                {
//                    addButton(new QuickLaunchAction(fileName, this));
//                }
                else
                {
                    qWarning() << "XdgDesktopFile" << fileName << "is not valid";
                    QMessageBox::information(this, tr("Drop Error"),
                                      tr("File/URL '%1' cannot be embedded into QuickLaunch for now").arg(fileName)
                                    );
                }
                //由于拖拽添加后，部分按钮的hover状态不会被释放，这里手动释放一下
                for (int i = 0; i < mBtnsLayout->count(); ++i)
                {
                    QuickLaunchButton *b = qobject_cast<QuickLaunchButton*>(mBtnsLayout->itemAt(i)->widget());
                    if (!b)
                        continue;
                    QMouseEvent event(QEvent::Leave, QPoint(1, 1) ,Qt::LeftButton,Qt::LeftButton,Qt::NoModifier);
                    QApplication::sendEvent(b, &event);
                }
            }
            saveSettings();
        }
        return true;
    }
    return QFrame::eventFilter(watched, event);
}


void NdeQuickLaunch::addButton(QuickLaunchAction* action)
{
    mBtnsLayout->setEnabled(false);

    //判断是否重复添加
    for (int i = 0; i < mBtnsLayout->count(); ++i)
    {
        QuickLaunchButton *b = qobject_cast<QuickLaunchButton*>(mBtnsLayout->itemAt(i)->widget());
        if (!b)
            continue;

        if(action->settingsMap().contains("desktop"))
        {
            QString desktop = action->settingsMap().value("desktop");
            QString existDesktop = b->settingsMap().value("desktop");
            if(existDesktop.compare(desktop, Qt::CaseInsensitive) == 0)
            {
                //重复添加
                delete  action;
                return;
            }
            //desktopFile Name is same.
            QString dName = desktop.split("/").takeLast();
            QString eDName = existDesktop.split("/").takeLast();
            if(dName.compare(eDName, Qt::CaseInsensitive) == 0)
            {
                //重复添加
                delete  action;
                return;
            }
        }
        //只能拖动desktop文件到快速启动栏
//        else if(action->settingsMap().contains("file"))
//        {
//            QString file = action->settingsMap().value("file");
//            QString existFile = b->settingsMap().value("file");
//            if(existFile.compare(file, Qt::CaseInsensitive) == 0)
//            {
//                //重复添加
//                delete  action;
//                return;
//            }
//        }
//        else
//        {
//            QString name = action->settingsMap().value("name");
//            QString exec = action->settingsMap().value("exec");
//            QString existName = b->settingsMap().value("name");
//            QString existExec = b->settingsMap().value("exec");
//            if(existName.compare(name, Qt::CaseInsensitive) == 0 &&
//                    existExec.compare(exec, Qt::CaseInsensitive) == 0)
//            {
//                //重复添加
//                delete  action;
//                return;
//            }
//        }
    }

    QuickLaunchButton* btn = new QuickLaunchButton(action, mPlugin, this,this);
    btn->setIconSize(QSize(mPlugin->panel()->iconSize(),mPlugin->panel()->iconSize()));
    mBtnsLayout->addWidget(btn);

    connect(btn, SIGNAL(switchButtons(QuickLaunchButton*,QuickLaunchButton*)), this, SLOT(switchButtons(QuickLaunchButton*,QuickLaunchButton*)));
    connect(btn, SIGNAL(buttonDeleted()), this, SLOT(buttonDeleted()));
    connect(btn, SIGNAL(movedLeft()), this, SLOT(buttonMoveLeft()));
    connect(btn, SIGNAL(movedRight()), this, SLOT(buttonMoveRight()));

    //删除提示
    mBtnsLayout->removeWidget(mPlaceHolder);
    delete mPlaceHolder;
    mPlaceHolder = 0;

    mBtnsLayout->setEnabled(true);
    realign();
}

void NdeQuickLaunch::switchButtons(QuickLaunchButton *button1, QuickLaunchButton *button2)
{
    if (button1 == button2)
        return;

    int n1 = mBtnsLayout->indexOf(button1);
    int n2 = mBtnsLayout->indexOf(button2);

    int l = qMin(n1, n2);
    int m = qMax(n1, n2);

    mBtnsLayout->moveItem(l, m, true);
    mBtnsLayout->moveItem(m-1, l, true);
    saveSettings();
}


void NdeQuickLaunch::buttonDeleted()
{
    QuickLaunchButton *btn = qobject_cast<QuickLaunchButton*>(sender());
    if (!btn)
        return;

    //删除保存的desktop文件
    QString filePath = btn->settingsMap()["desktop"];
    rmDesktopFile(filePath);

    mBtnsLayout->removeWidget(btn);
    btn->deleteLater();
    saveSettings();

    //如果没有项目，就显示提示，否则删除提示
    if (mBtnsLayout->count() <= 0)
        showPlaceHolder();

    int pages = getPages();
    if(mCurrentPage > pages-1)
        mCurrentPage = pages-1;

    realign();
}


void NdeQuickLaunch::buttonMoveLeft()
{
    QuickLaunchButton *btn = qobject_cast<QuickLaunchButton*>(sender());
    if (!btn)
        return;

    int index = indexOfButton(btn);
    if (index > 0)
    {
        mBtnsLayout->moveItem(index, index - 1);
        saveSettings();
    }
}


void NdeQuickLaunch::buttonMoveRight()
{
    QuickLaunchButton *btn1 = qobject_cast<QuickLaunchButton*>(sender());
    if (!btn1)
        return;

    int index = indexOfButton(btn1);
    if (index < countOfButtons() - 1)
    {
        mBtnsLayout->moveItem(index, index + 1);
        saveSettings();
    }
}

void NdeQuickLaunch::slotPrviBtnClicked()
{
    //
    if(mCurrentPage > 0)
    {
        mCurrentPage -= 1;
        realign();
    }
}

void NdeQuickLaunch::slotNextBtnClicked()
{
    //
    int pages = getPages();
    if(pages-1 > mCurrentPage)
    {
        mCurrentPage += 1;
        realign();
    }
}


void NdeQuickLaunch::saveSettings()
{
    PluginSettings *settings = mPlugin->settings();
    settings->remove("apps");

    QList<QMap<QString, QVariant> > hashList;
    int size = mBtnsLayout->count();
    for (int j = 0; j < size; ++j)
    {
        QuickLaunchButton *b = qobject_cast<QuickLaunchButton*>(mBtnsLayout->itemAt(j)->widget());
        if (!b)
            continue;

        // convert QHash<QString, QString> to QMap<QString, QVariant>
        QMap<QString, QVariant> map;
        QHashIterator<QString, QString> it(b->settingsMap());
        while (it.hasNext())
        {
            it.next();
            map[it.key()] = it.value();
        }
        hashList << map;
    }

    settings->setArray("apps", hashList);
}


void NdeQuickLaunch::showPlaceHolder()
{
    if (!mPlaceHolder)
    {
        mPlaceHolder = new QLabel(this);
        mPlaceHolder->setAlignment(Qt::AlignCenter);
        mPlaceHolder->setObjectName("QuickLaunchPlaceHolder");
        mPlaceHolder->setText(tr("Drop application\nicons here"));
    }

    mBtnsLayout->addWidget(mPlaceHolder);
}

int NdeQuickLaunch::getPages()
{
    int pages = mBtnsLayout->count()/mPageCount;
    //如果有余量，就添加一页
    if(mBtnsLayout->count()%mPageCount > 0)
    {
        pages += 1;
    }
    return pages;
}

int NdeQuickLaunch::getPanelSize()
{
    int size = mPlugin->panel()->iconSize();
    if(static_cast<NdePanel*>( mPlugin->panel()))
    {
        size = static_cast<NdePanel*>( mPlugin->panel())->panelSize();
    }
    return size;
}

QString NdeQuickLaunch::getSaveDesktopFilePath()
{
    QDir tmpDir = QStandardPaths::writableLocation(QStandardPaths::ConfigLocation);

    if(!QDir(tmpDir.absolutePath() + "/nde/nde-quicklaunch").exists())
    {
        tmpDir.mkpath("nde/nde-quicklaunch");
    }
    tmpDir.cd("nde/nde-quicklaunch");

    return  tmpDir.absolutePath();
}

void NdeQuickLaunch::saveDesktopFile(XdgDesktopFile &xdg)
{
    QString name = xdg.fileName().split('/').last();
    QString path = getSaveDesktopFilePath();
    //QString filePath = path +"/"+ XdgDesktopFile::id( xdg.fileName(),false);
    //XdgDesktopFile::id() 该函数获取桌面上Desktop文件时为空，原因未知
    QString filePath = path +"/"+ name;
    //直接保存destop文件到桌面可能会遇到无法打开的问题。
    xdg.save(filePath);
    xdg.load(filePath);

//    QString tmp = QString("cp %1 %2").arg(xdg.fileName()).arg(filePath);
//    QProcess::startDetached(tmp);
}

void NdeQuickLaunch::rmDesktopFile(const QString fileName)
{
    QString name = fileName.split('/').last();
    QString path = getSaveDesktopFilePath();
    QDir dir(path);
    if(dir.exists(name))
    {
        dir.remove(name);
    }
}
