/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)LGPL2+
 *
 * Nde - a lightweight, Qt based, desktop toolset
 * https://lxqt.org
 *
 * Copyright: 2010-2012 Razor team
 * Authors:
 *   Petr Vanek <petr@scribus.info>
 *   Kuzma Shapran <kuzma.shapran@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */


#ifndef NDEQUICKLAUNCH_H
#define NDEQUICKLAUNCH_H

#include "../panel/ndepanel.h"
#include <QHash>
#include <QString>
#include "../plugin-statusnotifier/statusnotifier.h"

class XdgDesktopFile;
class QuickLaunchAction;
class QDragEnterEvent;
class QuickLaunchButton;
class QSettings;
class QLabel;
class QToolButton;
class QBoxLayout;

namespace Nde {
class GridLayout;
}


/*! \brief Loader for "quick launcher" icons in the panel.
\author Petr Vanek <petr@scribus.info>
*/
class NdeQuickLaunch : public QFrame
{
    Q_OBJECT

public:
    NdeQuickLaunch(INdePanelPlugin *plugin, QWidget* parent = nullptr);
    ~NdeQuickLaunch();

    int indexOfButton(QuickLaunchButton* button) const;
    int countOfButtons() const;

    void realign();

    //从其他地方添加快速启动应用，比如开始菜单
    void addButtonQuick(XdgDesktopFile xdg);

    //若有应用卸载,需要重新加载
    void rebuildButtonQuick();

    virtual bool eventFilter(QObject *watched, QEvent *event);

private:
    //快速启动应用按钮widget
    QWidget* mBtnsWidget;
    //快速启动应用按钮Layout
    Nde::GridLayout *mBtnsLayout;
    INdePanelPlugin *mPlugin;
    QLabel *mPlaceHolder;

    void saveSettings();
    void showPlaceHolder();

    //每一页的快速启动应用数量
    int mPageCount;
    //当前页
    int mCurrentPage;
    //下一页按钮
    StatusNotifierArrow * mPrviBtn;
    //上一页按钮
    StatusNotifierArrow * mNextBtn;
    //包含上下翻页按钮的layout
    QBoxLayout * mPageLayout;
    //包含翻页按钮的widget
    QWidget* mPageWidget;
    //分割线
    QFrame * mLine;

private:
    int getPages();
    int getPanelSize();
    QString getSaveDesktopFilePath();
    void saveDesktopFile(XdgDesktopFile & xdg);
    void rmDesktopFile(const QString fileName);
private Q_SLOTS:
    void addButton(QuickLaunchAction* action);
    void switchButtons(QuickLaunchButton *button1, QuickLaunchButton *button2);
    void buttonDeleted();
    void buttonMoveLeft();
    void buttonMoveRight();
    void slotPrviBtnClicked();
    void slotNextBtnClicked();
};

#endif
