#include "showdesktopwidget.h"
#include <QShortcut>
#include "showdesktop.h"
#include <QDBusConnection>
#include <QDBusMessage>
#include <QDBusPendingCall>
#include <QSignalMapper>
#include <QSettings>
#include "../panel/pluginsettings.h"
#include <QDebug>


ShowDesktopWidget::ShowDesktopWidget(QWidget *parent) : QToolButton(parent)
{
    mSignalMapperTileWindow = new QSignalMapper(this);
    connect(mSignalMapperTileWindow, static_cast<void (QSignalMapper::*)(int)>(&QSignalMapper::mapped), this, &ShowDesktopWidget::activateTask);
    QTimer::singleShot(0, this, &ShowDesktopWidget::registerShortcuts);
}

void ShowDesktopWidget::invokeShortcut(const QString &shortcut) {
    auto msg = QDBusMessage::createMethodCall(
        QStringLiteral("org.kde.kglobalaccel"),
        QStringLiteral("/component/kwin"),
        QStringLiteral("org.kde.kglobalaccel.Component"),
        QStringLiteral("invokeShortcut"));
    msg.setArguments(QList<QVariant>{shortcut});
    QDBusConnection::sessionBus().asyncCall(msg);
}

void ShowDesktopWidget::registerShortcuts()
{
    GlobalKeyShortcut::Action * gshortcut;
    QString path;
    QString description;

    for (int i = 1; i <= 4; ++i)
    {
        path = QString("/panel/%1/function_%2").arg("tile_window").arg(i);
        description = tr("Window Quick Tile Function%1").arg(i);

        gshortcut = GlobalKeyShortcut::Client::instance()->addAction(QStringLiteral(), path, description, this);

        if (nullptr != gshortcut)
        {
            mKeys << gshortcut;
            connect(gshortcut, &GlobalKeyShortcut::Action::registrationFinished, this, &ShowDesktopWidget::shortcutRegistered);
            connect(gshortcut, &GlobalKeyShortcut::Action::activated, mSignalMapperTileWindow, static_cast<void (QSignalMapper::*)()>(&QSignalMapper::map));
            mSignalMapperTileWindow->setMapping(gshortcut, i);
        }
    }
}

void ShowDesktopWidget::shortcutRegistered()
{
    QString key;
    GlobalKeyShortcut::Action * const shortcut = qobject_cast<GlobalKeyShortcut::Action*>(sender());

    disconnect(shortcut, &GlobalKeyShortcut::Action::registrationFinished, this, &ShowDesktopWidget::shortcutRegistered);

    const int i = mKeys.indexOf(shortcut);
    Q_ASSERT(-1 != i);

    if (shortcut->shortcut().isEmpty())
    {
        const int j = (i +1)  % 5;
        switch(j)
        {
        case SC_TYPE_TOP:
            key = "Up";
            break;
        case SC_TYPE_BOTTOM:
            key = "Down";
            break;
        case SC_TYPE_LEFT:
            key = "Left";
            break;
        case SC_TYPE_RIGHT:
            key = "Right";
            break;
        default:
            key = "Up";
            break;
        }
        shortcut->changeShortcut(QStringLiteral("Meta+%1").arg(key));
    }
}

void ShowDesktopWidget::activateTask(int pos)
{
    auto currentWindowId = KWindowSystem::activeWindow();

    QString direction;
    switch(pos)
    {

    case SC_TYPE_TOP:
        direction = "Top";
        invokeShortcut(QStringLiteral("Window Maximize"));
        break;
    case SC_TYPE_BOTTOM:
        direction = "Bottom";
        invokeShortcut(QStringLiteral("Window Minimize"));
        break;
    case SC_TYPE_LEFT:
        direction = "Left";
        invokeShortcut(QStringLiteral("Window Quick Tile To %1").arg(direction));
        break;
    case SC_TYPE_RIGHT:
        direction = "Right";
        invokeShortcut(QStringLiteral("Window Quick Tile To %1").arg(direction));
        break;
    default:
        direction = "";
        break;
    }
}
