#include "statusnotifierpopup.h"
#include <QDragEnterEvent>
#include <QMimeData>
#include <QDebug>
#include <QtMath>
#include <KWindowEffects>
#include <QPainter>
#include <QStyleOption>
#include "../panel/indepanelplugin.h"
StatusNotifierPopup::StatusNotifierPopup(INdePanelPlugin *plugin, QWidget *parent)
    : StatusNotifierWidget(plugin, "popupOrder", parent),
      mPlugin(plugin),
      mCompositorOpen(false)
{
    setObjectName("StatusNotifierPopup");
    setAcceptDrops(true);
    setWindowFlags(Qt::FramelessWindowHint  | Qt::X11BypassWindowManagerHint | Qt::WindowStaysOnTopHint | Qt::Tool);
    setAttribute(Qt::WA_TranslucentBackground);
    //setAttribute(Qt::WA_NoSystemBackground, false);
    KWindowEffects::enableBlurBehind(winId(), true);

    mCompositorOpen = mPlugin->panel()->compositorOpen();
    this->setProperty("compositor", mCompositorOpen);
}

void StatusNotifierPopup::realign()
{
    Nde::GridLayout *layout = qobject_cast<Nde::GridLayout*>(this->layout());
    int columnCount = qCeil(qSqrt(layout->count()));
    columnCount = columnCount < 1 ? 1 : columnCount;
    int rowCount = qCeil(layout->count() / (qreal)columnCount);
    rowCount = rowCount < 1 ? 1 : rowCount;
    layout->setEnabled(false);
    layout->setColumnCount(columnCount);
    layout->setRowCount(rowCount);

    int width = 42 * columnCount;
    int height = 42 * rowCount;
    resize(width, height);
    layout->setEnabled(true);

    if(mCompositorOpen == mPlugin->panel()->compositorOpen()) return ;
    mCompositorOpen = mPlugin->panel()->compositorOpen();

    this->setProperty("compositor", mCompositorOpen);
    this->style()->unpolish(this);
    this->style()->polish(this);
}

bool StatusNotifierPopup::event(QEvent *event)
{
    if(this->geometry().contains(QCursor::pos()))
        return QWidget::event(event);
    if(event->type() == QEvent::ActivationChange)
    {
        if(!this->isActiveWindow())
        {
            this->hide();
        }
    }
    return QWidget::event(event);
}

void StatusNotifierPopup::itemActivated()
{
    this->hide();
}

void StatusNotifierPopup::paintEvent(QPaintEvent *event)
{
    QStyleOption opt;
    opt.init(this);
    QPainter painter(this);
    opt.initFrom(this);
    opt.rect = rect();
    style()->drawPrimitive(QStyle::PE_Widget, &opt, &painter, this);
    QWidget::paintEvent(event);
}
