#ifndef STATUSNOTIFIERTRAY_H
#define STATUSNOTIFIERTRAY_H

#include <QWidget>
#include <Nde/Settings>
#include "../panel/indepanelplugin.h"
#include "statusnotifierwidget.h"

class StatusNotifierPopup;
class StatusNotifierWatcher;
class PluginSettings;
class SniAsync;

struct SystemIconData
{
    QString ServiceAndPath;
    QString id;
    bool Display;
};

class StatusNotifierArrow : public QToolButton
{
    Q_OBJECT
    Q_PROPERTY(Qt::Corner origin READ origin WRITE setOrigin)
public:
    Qt::Corner origin() const;
    void setOrigin(Qt::Corner);
    explicit StatusNotifierArrow(QWidget *parent = nullptr);
    static QString mimeDataFormat() { return QLatin1String("nde/StatusNotifierButton"); }
Q_SIGNALS:
    void dropArrow(QString & serviceAndPath);

protected:
    void dropEvent(QDropEvent * event);
    void dragEnterEvent(QDragEnterEvent * event);
    void paintEvent(QPaintEvent * event);
private:
    Qt::Corner mOrigin;
    QPixmap mPixmap;
    bool mDrawPixmap;
};

class StatusNotifierTray : public StatusNotifierWidget
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "com.nde.panel.statusnotifiertray")
public:
    explicit StatusNotifierTray(INdePanelPlugin *plugin, QWidget *parent = nullptr);
    virtual ~StatusNotifierTray();
    void realign();
public Q_SLOTS:
    void recalculatePopupPosition();
    void onHideOrShowSystemIconCheange(QString command, bool isDisplay);

protected:
    void leaveEvent(QEvent *event);
private:
    void hideArrow();
    void ItemUnregistered(const QString & serviceAndPath);
    void addystemIconBtn(const SystemIconData & item);
    
    StatusNotifierPopup *mPopup;
    StatusNotifierWatcher *mWatcher;
    StatusNotifierArrow *mArrow;
    PluginSettings *mSettings;
    INdePanel::Position mCurPanelPos;
    QRect mScreenRect;
    QTimer *mWatchCursor;
    Nde::Settings * mSystemIconSetting;
    QMap<QString, SystemIconData> mItemData;
};

#endif // STATUSNOTIFIERTRAY_H
