/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)LGPL2+
 *
 * Nde - a lightweight, Qt based, desktop toolset
 * https://lxqt.org
 *
 * Copyright: 2015 Nde team
 * Authors:
 *  Balázs Béla <balazsbela[at]gmail.com>
 *  Paulo Lieuthier <paulolieuthier@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include "statusnotifierwatcher.h"
#include <QDebug>
#include <QDBusConnectionInterface>
#include <QtConcurrent/QtConcurrent>

StatusNotifierWatcher::StatusNotifierWatcher(QObject *parent) : QObject(parent)
{
    qRegisterMetaType<IconPixmap>("IconPixmap");
    qDBusRegisterMetaType<IconPixmap>();
    qRegisterMetaType<IconPixmapList>("IconPixmapList");
    qDBusRegisterMetaType<IconPixmapList>();
    qRegisterMetaType<ToolTip>("ToolTip");
    qDBusRegisterMetaType<ToolTip>();

    QDBusConnection dbus = QDBusConnection::sessionBus();
    if (!dbus.registerService("org.kde.StatusNotifierWatcher"))
        qDebug() << QDBusConnection::sessionBus().lastError().message();
    if (!dbus.registerObject("/StatusNotifierWatcher", this, QDBusConnection::ExportScriptableContents))
        qDebug() << QDBusConnection::sessionBus().lastError().message();

    mWatcher = new QDBusServiceWatcher(this);
    mWatcher->setConnection(dbus);
    mWatcher->setWatchMode(QDBusServiceWatcher::WatchForUnregistration);

    connect(mWatcher, &QDBusServiceWatcher::serviceUnregistered, this, &StatusNotifierWatcher::serviceUnregistered);
    m_settings = new Nde::Settings(QLatin1String("panel"), this);
    m_settings->beginGroup("statusnotifier");
    QStringList services = m_settings->value("trayLast").toStringList();
    m_settings->setValue("trayLast", QStringList());
    m_settings->endGroup();
    QtConcurrent::run([this, services, dbus](){
        while (!dbus.isConnected()) {
            QThread::msleep(1000);
        }
        QStringList items = RegisteredStatusNotifierItems();
        QString path = "/StatusNotifierItem";
        for(QString tmp : services){
            bool b = false;
            for (QString item : items) {
                if(item == tmp)
                {
                    b = true;
                    break;
                }
            }
            if(!b)
            {
                QString service;
//                if(tmp.startsWith(":"))
//                {
//                    service = tmp;
//                }
//                else
                    service = tmp.remove(path);
                QString cmd = QString("dbus-send --session --print-reply --dest=org.kde.StatusNotifierWatcher /StatusNotifierWatcher "
                                      "org.kde.StatusNotifierWatcher.RegisterStatusNotifierItem "
                                      "string:'%1'").arg(service);
                system(cmd.toLocal8Bit().data());
            }
        }
    });
}

StatusNotifierWatcher::~StatusNotifierWatcher()
{
    QDBusConnection::sessionBus().unregisterService("org.kde.StatusNotifierWatcher");
}

void StatusNotifierWatcher::RegisterStatusNotifierItem(const QString &serviceOrPath)
{
    qWarning() << "Service" << serviceOrPath << "registered";
    QString service = serviceOrPath;
    QString path = "/StatusNotifierItem";

    // workaround for sni-qt
    if (service.startsWith('/'))
    {
        path = service;
        service = message().service();
    }

    QString notifierItemId;
//    if(serviceOrPath.startsWith(":"))
//    {
//        service = serviceOrPath.mid(0, serviceOrPath.indexOf('/'));
//        notifierItemId = service + path;
//    }
//    else
        notifierItemId = service + path;
    if (QDBusConnection::sessionBus().interface()->isServiceRegistered(service).value()
        && !mServices.contains(notifierItemId))
    {
        mServices << notifierItemId;
        mWatcher->addWatchedService(service);
        m_settings->beginGroup("statusnotifier");
        m_settings->setValue("trayLast", mServices);
        m_settings->endGroup();
        Q_EMIT StatusNotifierItemRegistered(notifierItemId);
    }
}

void StatusNotifierWatcher::RegisterStatusNotifierHost(const QString &service)
{
    if (!mHosts.contains(service))
    {
        mHosts.append(service);
        mWatcher->addWatchedService(service);
    }
}

void StatusNotifierWatcher::serviceUnregistered(const QString &service)
{
    qDebug() << "Service" << service << "unregistered";

    mWatcher->removeWatchedService(service);

    if (mHosts.contains(service))
    {
        mHosts.removeAll(service);
        return;
    }

    QString match = service + '/';
    QStringList::Iterator it = mServices.begin();
    while (it != mServices.end())
    {
        if (it->startsWith(match))
        {
            QString name = *it;
            it = mServices.erase(it);
            Q_EMIT StatusNotifierItemUnregistered(name);
        }
        else
            ++it;
    }
    m_settings->beginGroup("statusnotifier");
    m_settings->setValue("trayLast", mServices);
    m_settings->endGroup();
}
