/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)LGPL2+
 *
 * Nde - a lightweight, Qt based, desktop toolset
 * https://lxqt.org
 *
 * Copyright: 2015 Nde team
 * Authors:
 *  Balázs Béla <balazsbela[at]gmail.com>
 *  Paulo Lieuthier <paulolieuthier@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#ifndef STATUSNOTIFIERWIDGET_H
#define STATUSNOTIFIERWIDGET_H

#include <QDir>

#include <Nde/GridLayout>
#include "statusnotifierbutton.h"

class StatusNotifierWidget : public QWidget
{
    Q_OBJECT

public:
    StatusNotifierWidget(INdePanelPlugin *plugin, QString name, QWidget *parent = nullptr);
    ~StatusNotifierWidget();

    INdePanelPlugin *plugin() const {return mPlugin;}
    int count() const {return mLayout->count();}
    static QString mimeDataFormat() { return QLatin1String("nde/StatusNotifierButton"); }
    QString orderName() const {return mName;}
    QString historyOrderName() const {return mName + "_history";}
    int index(const QString & serviceAndPath) const;
    QString trayId(QString service) const;
    bool contains(const QString & serviceAndPath){return mServices.contains(serviceAndPath); }
    void saveOrder();

Q_SIGNALS:
    void dropButton(const QString & serviceAndPath, int index = -1);
protected:
    void dropEvent(QDropEvent * event);
    void dragEnterEvent(QDragEnterEvent * event);
    void dragLeaveEvent(QDragLeaveEvent *event);
public Q_SLOTS:
    void itemAdded(QString serviceAndPath, SniAsync *interface, int index = -1);
    void layoutAdded(const QString & serviceAndPath, StatusNotifierButton *button, int index = -1);
    void itemRemoved(const QString &serviceAndPath);
    StatusNotifierButton * layoutRemoved(const QString & serviceAndPath);
    void buttonMove(StatusNotifierButton * dst, StatusNotifierButton * src, QPoint const & pos);
    void buttonDropped(StatusNotifierButton * dst, StatusNotifierButton * src, QPoint const & pos, QString &serviceAndPath);
    void arrowDropped(QString & serviceAndPath);
    virtual void itemActivated();
private:
    Nde::GridLayout *mLayout;
    INdePanelPlugin *mPlugin;
    QHash<QString, StatusNotifierButton*> mServices;
    QString mName;
};

#endif // STATUSNOTIFIERWIDGET_H
