/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)LGPL2+
 *
 * Nde - a lightweight, Qt based, desktop toolset
 * https://lxqt.org
 *
 * Copyright: 2011 Razor team
 *            2014 Nde team
 * Authors:
 *   Alexander Sokoloff <sokoloff.a@gmail.com>
 *   Maciej Płaza <plaza.maciej@gmail.com>
 *   Kuzma Shapran <kuzma.shapran@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */


#ifndef NDETASKBAR_H
#define NDETASKBAR_H

#include "../panel/indepanel.h"
#include "../panel/indepanelplugin.h"
#include "ndetaskbarconfiguration.h"
#include "ndetaskgroup.h"
#include "ndetaskbutton.h"
#include "../plugin-statusnotifier/statusnotifier.h"
#include "../panel/ndepanel.h"

#include <QFrame>
#include <QBoxLayout>
#include <QMap>
#include <Nde/GridLayout>
#include <nde-globalkeys.h>
#include "../panel/indepanel.h"
#include <KWindowSystem/KWindowSystem>
#include <KWindowSystem/KWindowInfo>
#include <KWindowSystem/NETWM>

class QSignalMapper;
class NdeTaskButton;
class ElidedButtonStyle;
class XdgDesktopFile;
namespace Nde {
class GridLayout;
}

class NdeTaskBar : public QFrame
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "com.nde.panel.ndetaskbar")

public:
    explicit NdeTaskBar(INdePanelPlugin *plugin, QWidget* parent = nullptr);
    virtual ~NdeTaskBar();

    void realign();

    ////图标模式值ToolButtonIconOnly，平铺模式为ToolButtonTextOnly或ToolButtonTextBesideIcon
    Qt::ToolButtonStyle buttonStyle() const { return mButtonStyle; }
    int buttonWidth() const { return mButtonStyle == Qt::ToolButtonIconOnly ? mButtonIconWidth:mButtonTileWidth;}
    bool closeOnMiddleClick() const { return mCloseOnMiddleClick; }
    bool raiseOnCurrentDesktop() const { return mRaiseOnCurrentDesktop; }
//    bool isShowOnlyOneDesktopTasks() const { return mShowOnlyOneDesktopTasks; }
    //这里强制设置为true，强制只显示当前桌面的应用
    bool isShowOnlyOneDesktopTasks() const { return true; }
    int showDesktopNum() const { return mShowDesktopNum; }
//    bool isShowOnlyCurrentScreenTasks() const { return mShowOnlyCurrentScreenTasks; }
//    bool isShowOnlyMinimizedTasks() const { return mShowOnlyMinimizedTasks; }
    //这里强制设置为true,强制显示当前屏幕的应用
    bool isShowOnlyCurrentScreenTasks() const { return true; }
    //这里强制显示设置为false，强制显示所有应用
    bool isShowOnlyMinimizedTasks() const { return false; }
    bool isAutoRotate() const { return (mButtonStyle != Qt::ToolButtonIconOnly); }
    bool isGroupingEnabled() const { return mGroupingEnabled; }
    bool isShowGroupOnHover() const { return mShowGroupOnHover; }
    bool isIconByClass() const { return mIconByClass; }
    inline INdePanel * panel() const { return mPlugin->panel(); }
    inline INdePanelPlugin * plugin() const { return mPlugin; }
	void addProcessPid(const QString &fileName, const qint64 &pid);
//    void removeGroupButton(const QString execStr);
    void panelSizeChanged();
    QString executeLinuxCmd(QString strCmd);            //执行shell指令或者shell脚本
public Q_SLOTS:
    void settingsChanged();
    //提供给其他应用调用的信息提示dbus接口
    bool taskbarInformationPrompt(unsigned long long appWindowId);

Q_SIGNALS:
    void buttonRotationRefreshed(bool autoRotate, INdePanel::Position position);
    void buttonStyleRefreshed(Qt::ToolButtonStyle buttonStyle);
    void refreshIconGeometry();
    void showOnlySettingChanged();
    void iconByClassChanged();
    void popupShown(NdeTaskGroup* sender);

protected:
    virtual void dragEnterEvent(QDragEnterEvent * event);
    virtual void dragMoveEvent(QDragMoveEvent * event);
	void dropEvent(QDropEvent *event);

private Q_SLOTS:
    void refreshTaskList();
    void refreshButtonRotation();
    void refreshPlaceholderVisibility();
    void groupBecomeEmptySlot();
    void onWindowChanged(WId window, NET::Properties prop, NET::Properties2 prop2);
    void onWindowAdded(WId window);
    void onWindowRemoved(WId window);
    void onActiveWindowChanged(WId window);
    void onDesktopChanged(int number);
    void registerShortcuts();
    void shortcutRegistered();
    void activateTask(int pos);
    //恢复任务栏信息提示状态
    void taskbarPromptsRecovery(WId Id);
    //翻页相关
    void slotPrviBtnClicked();
    void slotNextBtnClicked();

    //进程预览相关
    void onWindowDamaged(WId id, const uint8_t responseType);

private:
    typedef QMap<WId, NdeTaskGroup*> windowMap_t;

private:
    void addWindow(WId window);
    windowMap_t::iterator removeWindow(windowMap_t::iterator pos);
    void buttonMove(NdeTaskGroup * dst, NdeTaskGroup * src, QPoint const & pos);
//	void addButton(const XdgDesktopFile &xdg);

    //获取panel的尺寸
    int getPanelSize();
    //重新计算没页按钮的数量
    void reCalPageTotalButtons();
    //重新计算页面布局
    void reCalPage(bool toCurrentPage = false);

    void resetLayout();

    void showCurrentPage();
private:
    QMap<WId, NdeTaskGroup*> mKnownWindows; //!< Ids of known windows (mapping to buttons/groups)
    QList<WId> mKnownTileModeOrder;
    QList<QList<WId>> mKnownIconModeOrder;
    QBoxLayout *mLayout;
    QList<GlobalKeyShortcut::Action*> mKeys;
    QSignalMapper *mSignalMapper;
    //有信息提示的窗口
    QVector<WId> mTaskbarInformationWId;
    //应用布局
    QWidget* mTasksWidget;
    Nde::GridLayout *mTasksLayout;

    //当前方向
    INdePanel::Position mCurrentPosition;
    //总的页数
    int mCountPage;
    //当前页
    int mCurrentPage;
    //下一页按钮
    StatusNotifierArrow * mPrviBtn;
    //上一页按钮
    StatusNotifierArrow * mNextBtn;
    //包含上下翻页按钮的layout
    QBoxLayout * mPageLayout;
    //包含翻页按钮的widget
    QWidget* mPageWidget;

    // Settings
    Qt::ToolButtonStyle mButtonStyle; //图标模式值ToolButtonIconOnly，平铺模式为ToolButtonTextOnly或ToolButtonTextBesideIcon
    int mButtonIconWidth;
    int mButtonTileWidth;
    int mButtonHeight;
    int mPageTotalButtons; //每页按钮最多个数
    bool mCloseOnMiddleClick;
    bool mRaiseOnCurrentDesktop;
    bool mShowOnlyOneDesktopTasks;
    int mShowDesktopNum;
    bool mShowOnlyCurrentScreenTasks;
    bool mShowOnlyMinimizedTasks;
//    bool mAutoRotate;
    bool mGroupingEnabled;
    bool mShowGroupOnHover;
    bool mIconByClass;
    bool mCycleOnWheelScroll; //!< flag for processing the wheelEvent

    bool acceptWindow(WId window) const;
    void setButtonStyle(Qt::ToolButtonStyle buttonStyle);

    void wheelEvent(QWheelEvent* event);
    void changeEvent(QEvent* event);
    void resizeEvent(QResizeEvent *event);

    QMap<qint64 , QString> mProcessPid;
//    QMap<QString, NdeTaskGroup*> mFixedApp;
    INdePanelPlugin *mPlugin;
    LeftAlignedTextStyle *mStyle;
    bool loadactive{false};
};

#endif // NDETASKBAR_H
