/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)LGPL2+
 *
 * Nde - a lightweight, Qt based, desktop toolset
 * https://lxqt.org
 *
 * Copyright: 2011 Razor team
 *            2014 Nde team
 * Authors:
 *   Alexander Sokoloff <sokoloff.a@gmail.com>
 *   Kuzma Shapran <kuzma.shapran@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */


#ifndef NDETASKBUTTON_H
#define NDETASKBUTTON_H

#include <QToolButton>
#include <QProxyStyle>
#include <XdgDesktopFile>
#include <QMimeData>
#include "../panel/indepanel.h"

class QPainter;
class QPalette;
class QMimeData;
class NdeTaskGroup;
class NdeTaskBar;

class LeftAlignedTextStyle : public QProxyStyle
{
    using QProxyStyle::QProxyStyle;
public:

    virtual void drawItemText(QPainter * painter, const QRect & rect, int flags
            , const QPalette & pal, bool enabled, const QString & text
            , QPalette::ColorRole textRole = QPalette::NoRole) const override;
};

class NdeTaskButton : public QToolButton
{
    Q_OBJECT

    Q_PROPERTY(Qt::Corner origin READ origin WRITE setOrigin)

public:
//	enum TASK_BUTTON_STATUS{
//        BUTTON_FIXED,
//        BUTTON_FIXED_AND_OPENED,
//        BUTTON_UNFIXED
//    };
    explicit NdeTaskButton(const WId window, NdeTaskBar * taskBar, bool autoRotation, QWidget *parent = nullptr);
    explicit NdeTaskButton(const XdgDesktopFile &xdg,  NdeTaskBar * taskbar, bool autoRotation, QWidget *parent = nullptr);
	virtual ~NdeTaskButton();

    bool isApplicationHidden() const;
    bool isApplicationActive() const;
    WId windowId() const { return mWindow; }

    bool hasUrgencyHint() const { return mUrgencyHint; }
    void setUrgencyHint(bool set);

    bool isOnDesktop(int desktop) const;
    bool isOnCurrentScreen() const;
    bool isMinimized() const;
    void updateText();

    //获取该btn在panel中显示在哪一页
    int getCurrentPanelPage() { return currentPanelPage;}
    //更新在panel 中显示的页面
    void setCurrentPanelPage(const int page){currentPanelPage = page;}

    Qt::Corner origin() const;
    virtual void setAutoRotation(bool autoRotation, INdePanel::Position position);
    void updateButtonStyle(bool autoRotation, int position);
    NdeTaskBar * parentTaskBar() const {return mParentTaskBar;}

    void refreshIconGeometry(QRect const & geom);
    static QString mimeDataFormat() { return QLatin1String("nde/ndetaskbutton"); }
    /*! \return true if this buttom received DragEnter event (and no DragLeave event yet)
     * */
    bool hasDragAndDropHover() const;
//    void setButtonStatus(TASK_BUTTON_STATUS status);
//    TASK_BUTTON_STATUS buttonStatus() const;
//    bool isFixed() const;
    inline INdePanelPlugin * plugin() const { return mPlugin; }
    void setWindowId(WId wid) {mWindow = wid;}
public Q_SLOTS:
    void raiseApplication();
    void minimizeApplication();
    void maximizeApplication();
    void deMaximizeApplication();
    void shadeApplication();
    void unShadeApplication();
    void closeApplication();
    void moveApplicationToDesktop();
    void moveApplication();
    void resizeApplication();
    void setApplicationLayer();

    void setOrigin(Qt::Corner);

    void updateIcon();
//    void unpinApp();
protected:
    virtual void dragEnterEvent(QDragEnterEvent *event);
    virtual void dragMoveEvent(QDragMoveEvent * event);
    virtual void dragLeaveEvent(QDragLeaveEvent *event);
    virtual void dropEvent(QDropEvent *event);
    void mousePressEvent(QMouseEvent *event);
    void mouseReleaseEvent(QMouseEvent *event);
    void mouseMoveEvent(QMouseEvent *event);
    virtual void contextMenuEvent(QContextMenuEvent *event);
    void paintEvent(QPaintEvent *);

    virtual QMimeData * mimeData();
    static bool sDraggging;
    bool mPreventPopup;
    bool isMoved;
    QString name() const;
    void initButton();
protected:
    WId mWindow;
    bool mAutoRotation;
    bool mUrgencyHint;
    QPoint mDragStartPosition;
    Qt::Corner mOrigin;
    QPixmap mPixmap;
    bool mDrawPixmap;
    NdeTaskBar * mParentTaskBar;
    INdePanelPlugin * mPlugin;
    int currentPosition;
    int currentPanelPage;
    // Timer for when draggind something into a button (the button's window
    // must be activated so that the use can continue dragging to the window
    QTimer * mDNDTimer;
    XdgDesktopFile mXdg;
//    TASK_BUTTON_STATUS mButtonStatus;
private Q_SLOTS:
    void activateWithDraggable();

Q_SIGNALS:
    void dragging(QObject * dragSource, QPoint const & pos);

private:
    QTimer* m_mouseLeaseTimer;
};

typedef QHash<WId,NdeTaskButton*> NdeTaskButtonHash;

#endif // NDETASKBUTTON_H
