#include "ndethumbnailbutton.h"
#include "ndetaskbar.h"
#include <QLabel>
#include <QDebug>
#include <QStyleOption>
#include <QPainter>
#include <QMouseEvent>
#include <KWindowSystem/NETWM>
#include <QX11Info>
#include <QApplication>
#include <QDesktopWidget>
#include <QToolTip>
#include "../panel/thumbnailmanager.h"
#include "windowthumbnail.h"
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#define TITLE_HEIGHT 22

NdeThumbnailButton::NdeThumbnailButton(const WId window, NdeTaskBar * taskBar, QWidget *parent) :
    QFrame(parent),
    mWindow(window),
    mParentTaskBar(taskBar)
{
    setObjectName("thumbnailButton");
    QVBoxLayout *layout = new QVBoxLayout;
    closeButton = new QToolButton(this);
    closeButton->setObjectName("closeButton");
    closeButton->setFixedHeight(TITLE_HEIGHT);
    connect(closeButton, &QToolButton::clicked, this, &NdeThumbnailButton::closeApplication);
    windowTitle = new QLabel;
    windowTitle->setObjectName("windowTitle");
//    thumbnailPixmap = new QLabel;
//    thumbnailPixmap->setScaledContents(true);
    qmlRegisterType<Plasma::WindowThumbnail>("org.kde.plasma", 2, 0, "WindowThumbnail");
    m_thumbnailWidget = new QQuickWidget(this);
    m_thumbnailWidget->setResizeMode(QQuickWidget::ResizeMode::SizeRootObjectToView);
    m_thumbnailWidget->setSource(QUrl("qrc:/windowthumbnail.qml"));
    QQuickItem* root = m_thumbnailWidget->rootObject();
    if(root)
    {
        Plasma::WindowThumbnail* item = static_cast<Plasma::WindowThumbnail*>(root);
        item->setProperty("winId", mWindow);
    }

    windowTitle->setFixedHeight(TITLE_HEIGHT);

    closeButton->hide();
    QHBoxLayout *titleLayout = new QHBoxLayout;
    QHBoxLayout *controlLayout = new QHBoxLayout;
    titleLayout->addWidget(windowTitle, 1, Qt::AlignLeft | Qt::AlignVCenter);
    titleLayout->addWidget(closeButton, 0, Qt::AlignRight | Qt::AlignVCenter);
    titleLayout->setContentsMargins(0, 0, 0, 0);

    layout->addItem(titleLayout);
//    layout->addWidget(thumbnailPixmap, 1, Qt::AlignHCenter | Qt::AlignVCenter);
    layout->addWidget(m_thumbnailWidget, 1, Qt::AlignHCenter | Qt::AlignVCenter);
    layout->addItem(controlLayout);

    setLayout(layout);

    updateTitle();

    setBordeState(false);

}
NdeThumbnailButton::~NdeThumbnailButton()
{

}
bool NdeThumbnailButton::isApplicationActive() const
{
    return KWindowSystem::activeWindow() == mWindow;
}
bool NdeThumbnailButton::isOnDesktop(int desktop) const
{
    return KWindowInfo(mWindow, NET::WMDesktop).isOnDesktop(desktop);
}
void NdeThumbnailButton::closeApplication()
{
    NETRootInfo(QX11Info::connection(), NET::CloseWindow).closeWindowRequest(mWindow);
}
void NdeThumbnailButton::setUrgencyHint(bool set)
{
    if (mUrgencyHint == set)
        return;

    if (!set)
        KWindowSystem::demandAttention(mWindow, false);

    mUrgencyHint = set;
    setProperty("urgent", set);
    style()->unpolish(this);
    style()->polish(this);
    update();
}
void NdeThumbnailButton::raiseApplication()
{
    KWindowInfo info(mWindow, NET::WMDesktop | NET::WMState | NET::XAWMState);
    if (parentTaskBar()->raiseOnCurrentDesktop() && info.isMinimized())
    {
        KWindowSystem::setOnDesktop(mWindow, KWindowSystem::currentDesktop());
    }
    else
    {
        int winDesktop = info.desktop();
        if (KWindowSystem::currentDesktop() != winDesktop)
            KWindowSystem::setCurrentDesktop(winDesktop);
    }
    KWindowSystem::activateWindow(mWindow);

    setUrgencyHint(false);
}
bool NdeThumbnailButton::isOnCurrentScreen() const
{
    return QApplication::desktop()->screenGeometry(parentTaskBar()).intersects(KWindowInfo(mWindow, NET::WMFrameExtents).frameGeometry());
}
void NdeThumbnailButton::updateTitle()
{
    KWindowInfo info(mWindow, NET::WMVisibleName | NET::WMName);
    QString title = info.visibleName().isEmpty() ? info.name() : info.visibleName();
    windowTitle->setText(title.replace("&", "&&"));
    setToolTip(title);
}

QString NdeThumbnailButton::title() const
{
    return windowTitle->text();
}
void NdeThumbnailButton::addThumbnail(void* pixmap)
{
//    thumbnailPixmap->setPixmap(pixmap);
    m_image = pixmap;
    QQuickItem* root = m_thumbnailWidget->rootObject();
    if(root)
    {
        //最小化时X11不渲染窗口，获取不到窗口的渲染数据，使用截图
        Plasma::WindowThumbnail* item = static_cast<Plasma::WindowThumbnail*>(root);
        QMetaObject::invokeMethod(item, "setImage", Q_ARG(void*, m_image));
    }
}
void NdeThumbnailButton::enterEvent(QEvent *event)
{
    ThumbnailManager::getIns()->previewWindow(mWindow);
    closeButton->show();
    QFrame::enterEvent(event);
}

void NdeThumbnailButton::leaveEvent(QEvent *event)
{
    ThumbnailManager::getIns()->quitPreviewWindow();
    closeButton->hide();
    QFrame::enterEvent(event);
}
void NdeThumbnailButton::mousePressEvent(QMouseEvent* event)
{
    const Qt::MouseButton b = event->button();

    if (Qt::MidButton == b && parentTaskBar()->closeOnMiddleClick())
        closeApplication();
    QFrame::mousePressEvent(event);
}
void NdeThumbnailButton::mouseReleaseEvent(QMouseEvent* event)
{
    //需要在activate前恢复窗口的layer
    ThumbnailManager::getIns()->quitPreviewWindow(true);
    //点击了关闭按钮不要触发click事件
    if(closeButton && closeButton->isVisible() && closeButton->geometry().contains(event->pos()))
    {
        //do nothing
    }
    else
    {
        if (event->button() == Qt::LeftButton)
        {
            raiseApplication();
        }
        Q_EMIT clicked();
    }
    QFrame::mouseReleaseEvent(event);
}

bool NdeThumbnailButton::event(QEvent *e)
{
    switch (e->type()) {
    case QEvent::ToolTip://防止鼠标移到tooltip上失去焦点触发leaveevent
        QToolTip::showText(this->mapToGlobal(QPoint(this->width() / 3, this->height() + 50)), title());
        e->accept();
        return true;
    default:
        break;
    }
    return QFrame::event(e);
}
void NdeThumbnailButton::minimizeApplication()
{
    KWindowSystem::minimizeWindow(mWindow);
}
void NdeThumbnailButton::setAutoRotation(bool value, INdePanel::Position position)
{
    if (value)
    {
        switch (position)
        {
        case INdePanel::PositionTop:
        case INdePanel::PositionBottom:
            setOrigin(Qt::TopLeftCorner);
            break;

        case INdePanel::PositionLeft:
            setOrigin(Qt::BottomLeftCorner);
            break;

        case INdePanel::PositionRight:
            setOrigin(Qt::TopRightCorner);
            break;
        }
    }
    else
        setOrigin(Qt::TopLeftCorner);
}

bool NdeThumbnailButton::onWindowDamaged(const uint8_t responseType)
{
    QQuickItem* root = m_thumbnailWidget->rootObject();
    if(root)
    {
        Plasma::WindowThumbnail* item = static_cast<Plasma::WindowThumbnail*>(root);
        QMetaObject::invokeMethod(item, "windowDamaged", Q_ARG(uint8_t, responseType));
    }
    return false;
}

void NdeThumbnailButton::paintEvent(QPaintEvent *)
{
    if(bordeState){
        QPainter painter(this);
        //填充色
        QColor color;
        color.setRgb(225,129,2,100);
        QBrush brush(color);
        painter.setBrush(brush);
        painter.drawRect(this->rect());
    }

}

void NdeThumbnailButton::setOrigin(Qt::Corner newOrigin)
{
    if (mOrigin != newOrigin)
    {
        mOrigin = newOrigin;
        update();
    }
}
