/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)LGPL2+
 *
 * Nde - a lightweight, Qt based, desktop toolset
 * https://lxqt.org
 *
 * Copyright: 2012 Razor team
 * Authors:
 *   Johannes Zellner <webmaster@nebulon.de>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#ifndef NDEVOLUME_H
#define NDEVOLUME_H

#include "../panel/indepanelplugin.h"
#include <QToolButton>
#include <QSlider>
#include <QPointer>

class VolumeButton;
class AudioEngine;
class AudioDevice;
namespace Nde {
class Notification;
}
namespace GlobalKeyShortcut
{
class Action;
}

class NdeVolumeConfiguration;

class NdeVolume : public QObject, public INdePanelPlugin
{
    Q_OBJECT
public:
    NdeVolume(const INdePanelPluginStartupInfo &startupInfo);
    ~NdeVolume();

    virtual QWidget *widget();
    virtual QString themeId() const { return "Volume"; }
    virtual INdePanelPlugin::Flags flags() const { return PreferRightAlignment | HaveConfigDialog ; }
    void realign();
    QDialog *configureDialog();

    void setAudioEngine(AudioEngine *engine);
protected Q_SLOTS:
    virtual void settingsChanged();
    void handleSinkListChanged();
    void handleShortcutVolumeUp();
    void handleShortcutVolumeDown();
    void handleShortcutVolumeMute();
    void shortcutRegistered();
    void showNotification(bool forceShow) const;

private:
    AudioEngine *m_engine;
    VolumeButton *m_volumeButton;
    int m_defaultSinkIndex;
    AudioDevice *m_defaultSink;
    GlobalKeyShortcut::Action *m_keyVolumeUp;
    GlobalKeyShortcut::Action *m_keyVolumeDown;
    GlobalKeyShortcut::Action *m_keyMuteToggle;
    Nde::Notification *m_notification;
    QPointer<NdeVolumeConfiguration> m_configDialog;
    bool m_allwaysShowNotifications;
};


class NdeVolumePluginLibrary: public QObject, public INdePanelPluginLibrary
{
    Q_OBJECT
    Q_PLUGIN_METADATA(IID "lxqt.org/Panel/PluginInterface/3.0")
    Q_INTERFACES(INdePanelPluginLibrary)
public:
    INdePanelPlugin *instance(const INdePanelPluginStartupInfo &startupInfo) const
    {
        return new NdeVolume(startupInfo);
    }
};

#endif // NDEVOLUME_H
