#include "calendar.h"
#include "ui_calendar.h"
#include <QBitmap>
#include <QPainter>
#include <QStyleOption>
#include <QDateTime>
#include <QTimer>
#include <QDebug>
#include <KWindowEffects>
#include <QScreen>
#include <QRect>
#include <QWheelEvent>

extern bool timemode_24;

//static const QString s_strWeekl[] = {
//    QObject::tr("Mon"), QObject::tr("Tue"),QObject::tr("Wed"),
//    QObject::tr("Thu"), QObject::tr("Fri"),QObject::tr("Sat"),QObject::tr("Sun"),
//};

Calendar::Calendar(INdePanelPlugin *plugin, QWidget *parent) :
    QWidget(parent),
    mPlugin(plugin),
    timer(nullptr),
    mCompositorOpen(-1),
    ui(new Ui::Calendar)
{
    this->setObjectName("CalendarWidget");
    ui->setupUi(this);

    weekList << tr("Mon") << tr("Tue") << tr("Wed") << tr("Thu") << tr("Fri") << tr("Sat") << tr("Sun");

    monthList << tr("Jan") << tr("Feb") << tr("Mar") << tr("Apr") << tr("May") << tr("Jun")
              << tr("Jul") << tr("Aug") << tr("Sep") << tr("Oct") << tr("Nov") << tr("Dec");

    this->setProperty("NormalSize", true);
    mCompositorOpen =  mPlugin->panel()->compositorOpen();
    this->setProperty("compositor", mCompositorOpen);
    this->style()->polish(this);
    update();
    for (int i = 0; i < 7; i++){
        QLabel *l = qobject_cast<QLabel *>(ui->gridLayout->itemAtPosition(0,i%7)->widget());
        l->setObjectName("labelWeek");
        listLabelWeek << l;

        l->setText(weekList.at(i));
    }


    this->setWindowFlags(Qt::FramelessWindowHint |
                         Qt::Tool |
                         Qt::X11BypassWindowManagerHint);
    this->setAttribute(Qt::WA_TranslucentBackground);

    KWindowEffects::enableBlurBehind(this->winId(), true);
    initDateTime();
    initDateToolBar();
    initCalendarWidget();

    ui->stackedWidget->setCurrentIndex(PAGE_CALENDAR);

    for (int i = 0; i < 16; i++){
        QPushButton *b = qobject_cast<QPushButton *>(ui->gridLayout_2->itemAtPosition(i/4,i%4)->widget());
        b->setObjectName("selectButton");
        listSelectButton << b;
        connect(b, SIGNAL(clicked()), this, SLOT(selectDate()));
    }
}
Calendar::~Calendar()
{
    delete ui;
}

bool Calendar::event(QEvent *e)
{
    if (e->type() == QEvent::ActivationChange){
        if (!this->isActiveWindow()){
            this->hide();
        }
    }
    return QWidget::event(e);
}

void Calendar::selectDate()
{
    QPushButton *b = qobject_cast<QPushButton *>(sender());
    //4位是年份
    if (b->text().count() == 4){
        mDate.setDate(b->text().toInt(), mDate.month(), mDate.day());
    }else {
        mDate.setDate(mDate.year(), (monthList.indexOf(b->text()) + 1), mDate.day());

    }

    ui->stackedWidget->setCurrentIndex(PAGE_CALENDAR);
    ui->labelRange->hide();
    showToolBar();

    dateChanged(mDate);
}

void Calendar::refreshSize(bool normal)
{
    if (!normal){
        this->setFixedSize(300,350);
        ui->horizontalLayout->setContentsMargins(5, 0, 5, 0);
        ui->horizontalLayout->setSpacing(8);
        this->setProperty("NormalSize", false);

        for (int i = 0; i < 42; i++) {
            listDateWidget.at(i)->refreshSize(false);
        }
    }else{
        this->setFixedSize(467,620);
        ui->horizontalLayout->setContentsMargins(20, 0, 20, 0);
        ui->horizontalLayout->setSpacing(20);
        this->setProperty("NormalSize", true);

        for (int i = 0; i < 42; i++) {
            listDateWidget.at(i)->refreshSize(true);
        }
    }

    this->style()->polish(this);
    this->style()->polish(ui->labelTime);
    this->style()->polish(ui->labelDate);
    update();
}

void Calendar::realign()
{
    if(mCompositorOpen != mPlugin->panel()->compositorOpen())
    {
        mCompositorOpen =  mPlugin->panel()->compositorOpen();
        this->setProperty("compositor", mCompositorOpen);
        this->style()->unpolish(this);
        this->style()->polish(this);
        update();
    }
}

void Calendar::setDateTime()
{
    QDateTime datetime = QDateTime::currentDateTime();
    QString time = datetime.time().toString();
    if(timemode_24){
        ui->labelTime->setText(time);
    } else {
        ui->labelTime->setText(datetime.time().toString("AP hh:mm:ss"));
    }

    QString date = datetime.date().toString("yyyy/MM/dd");
    int day = datetime.date().dayOfWeek();
    ui->labelDate->setText(date + "    " + weekList.at(day-1));
}

void Calendar::initDateTime()
{
    ui->labelTime->setObjectName("CurrentTimeLabel");
    ui->labelDate->setObjectName("CurrentDateLabel");

    setDateTime();

    if (nullptr == timer){
        timer = new QTimer;
        timer->setSingleShot(false);
        timer->start(1000);
        connect(timer,SIGNAL(timeout()),this,SLOT(setDateTime()));
    }
}

void Calendar::initDateToolBar()
{
    mDate = QDate::currentDate();
    ui->buttonYear->setText(QString::number(mDate.year()));
    if (mDate.month() < 10){
        ui->buttonMonth->setText("0"+ QString::number(mDate.month()));
    }else{
        ui->buttonMonth->setText(QString::number(mDate.month()));
    }

    ui->labelRange->hide();
}

void Calendar::initCalendarWidget()
{
    for (int i = 0; i < 42; i++){
        dateWidget *d = new dateWidget(ui->widget_2);
        ui->gridLayout->addWidget(d, i/7+1, i%7);
        listDateWidget << d;
    }

    setCalendarDate(mDate);
}

void Calendar::setCalendarDate(QDate date)
{
    int days = date.day();
    int weekday = date.dayOfWeek();
    int premonth_day;
    if (days < weekday) {
        premonth_day = weekday - days;
    } else {
         premonth_day = 7 - ((days-weekday) % 7);
    }

    if (premonth_day ==0 ){
        premonth_day = 7;
    }

    int from = 0-premonth_day-days+1;
    for (int i = 0 ,j = from; i < 42; i++,j++) {
        QDate d = date.addDays(j);
        listDateWidget.at(i)->setProperty("InThisMonth", d.month() == date.month());
        listDateWidget.at(i)->setDateText(d);
    }
}

void Calendar::refreshDate()
{
    on_buttonToday_clicked();
}

void Calendar::paintEvent(QPaintEvent *event)
{
    QBitmap bmp(this->size());
    bmp.fill();
    QPainter painter(&bmp);
    painter.setPen(Qt::NoPen);
    painter.setBrush(Qt::black);
    painter.setRenderHint(QPainter::Antialiasing);
    painter.setRenderHint(QPainter::SmoothPixmapTransform);
    painter.drawRoundedRect(bmp.rect(), 4, 4);
    setMask(bmp);

    QStyleOption opt;
    opt.init(this);
    QPainter painter1(this);
    style()->drawPrimitive(QStyle::PE_Widget, &opt, &painter1, this);
}

void Calendar::hideToolBar()
{
    ui->buttonYearPre->hide();
    ui->buttonYear->hide();
    ui->buttonYearNext->hide();

    ui->buttonMonthPre->hide();
    ui->buttonMonth->hide();
    ui->buttonMonthNext->hide();
}

void Calendar::showToolBar()
{
    ui->buttonYearPre->show();
    ui->buttonYear->show();
    ui->buttonYearNext->show();

    ui->buttonMonthPre->show();
    ui->buttonMonth->show();
    ui->buttonMonthNext->show();
}

void Calendar::on_buttonYear_clicked()
{
    ui->stackedWidget->setCurrentIndex(PAGE_SELECTYM);
    int beginYear = mDate.year() - 7;
    QString str = QString::number(beginYear) + "-" + QString::number(beginYear + 15);
    ui->labelRange->setText(str);

    ui->labelRange->show();
    hideToolBar();

    for (int i = 0; i < listSelectButton.count(); i++){
        listSelectButton.at(i)->setText(QString::number(beginYear + i));
        listSelectButton.at(i)->show();

        listSelectButton.at(i)->setProperty("Current", beginYear + i == QDate::currentDate().year());

        listSelectButton.at(i)->setProperty("Lighten", true);
//        if (i < 15){
//            listSelectButton.at(i)->setProperty("Lighten", true);
//        }else {
//            listSelectButton.at(i)->setProperty("Lighten", false);
//        }

        style()->polish(listSelectButton.at(i));
    }
    update();
}


void Calendar::on_buttonMonth_clicked()
{
    ui->stackedWidget->setCurrentIndex(PAGE_SELECTYM);
    QString str = QString::number(mDate.year()) + "-" + QString::number(mDate.month());
    ui->labelRange->setText(str);

    ui->labelRange->show();
    hideToolBar();

    for (int i = 0; i < listSelectButton.count(); i++){

        listSelectButton.at(i)->setText(QString::number(i%12+1));
        if (i < 12){
            listSelectButton.at(i)->setText(monthList.at(i));

            listSelectButton.at(i)->setProperty("Lighten", true);
            listSelectButton.at(i)->setProperty("Current", (i + 1) == QDate::currentDate().month() && mDate.year() == QDate::currentDate().year());
        }else {
              listSelectButton.at(i)->hide();
        }

        style()->polish(listSelectButton.at(i));
    }
    update();
}

void Calendar::wheelEvent(QWheelEvent *event)
{
    if (event->delta() > 0){
        mDate = mDate.addMonths(-1);
        dateChanged(mDate);
    }else{
        mDate = mDate.addMonths(1);
        dateChanged(mDate);
    }
}

void Calendar::on_buttonToday_clicked()
{
    ui->stackedWidget->setCurrentIndex(PAGE_CALENDAR);
    ui->labelRange->hide();
    showToolBar();

    mDate = QDate::currentDate();
    dateChanged(mDate);
}

void Calendar::on_buttonYearPre_clicked()
{
    mDate = mDate.addYears(-1);
    dateChanged(mDate);
}

void Calendar::dateChanged(QDate date)
{
    ui->buttonYear->setText(QString::number(date.year()));
    if (mDate.month() < 10){
        ui->buttonMonth->setText("0"+ QString::number(date.month()));
    }else{
        ui->buttonMonth->setText(QString::number(date.month()));
    }
    setCalendarDate(date);
}

void Calendar::on_buttonYearNext_clicked()
{
    mDate = mDate.addYears(1);
    dateChanged(mDate);
}

void Calendar::on_buttonMonthPre_clicked()
{
    mDate = mDate.addMonths(-1);
    dateChanged(mDate);
}

void Calendar::on_buttonMonthNext_clicked()
{
    mDate = mDate.addMonths(1);
    dateChanged(mDate);
}
