#ifndef CALENDAR_H
#define CALENDAR_H

#include <QWidget>
#include <QDate>
#include <QPushButton>
#include "datewidget.h"
#include <QLabel>
#include "../panel/indepanelplugin.h"

namespace Ui {
class Calendar;
}

typedef enum{
    PAGE_CALENDAR = 0,
    PAGE_SELECTYM
}E_PAGE;

class Calendar : public QWidget
{
    Q_OBJECT

public:
    explicit Calendar(INdePanelPlugin *plugin,  QWidget *parent = nullptr);
    ~Calendar();

    void paintEvent(QPaintEvent *event);
    void initDateTime();
    void initDateToolBar();
    void initCalendarWidget();

    void hideToolBar();
    void showToolBar();

    void setCalendarDate(QDate date);
    void realign();

    void refreshSize(bool normal);
    void refreshDate();

protected:
    bool event(QEvent *e);
    virtual void wheelEvent(QWheelEvent *event);

private Q_SLOTS:
    void selectDate();
    void dateChanged(QDate date);
    void setDateTime();

    void on_buttonYear_clicked();

    void on_buttonMonth_clicked();

    void on_buttonToday_clicked();

    void on_buttonYearPre_clicked();

    void on_buttonYearNext_clicked();

    void on_buttonMonthPre_clicked();

    void on_buttonMonthNext_clicked();

private:
    INdePanelPlugin *mPlugin;
    Ui::Calendar *ui;
    QTimer *timer;
    QDate mDate;
    bool mCompositorOpen;
    QList<dateWidget *> listDateWidget;
    QList<QLabel *> listLabelWeek;
    QList<QPushButton *> listSelectButton;
    QStringList weekList;
    QStringList monthList;
};

#endif // CALENDAR_H
