#include "datewidget.h"
#include "ui_datewidget.h"
#include <QDate>
#include <QFile>
#include <QJsonDocument>
#include <QJsonArray>
#include <QJsonObject>
#include <QStyle>
#include <QPainter>
#include <QStyleOption>

dateWidget * lastSelect = nullptr;

static const int springFestival[] = {
    130,217,206,                                // 1968,1969,1970
    127,215,203,123,211,131,218,207,128,216,    // 1971--1980
    205,125,213,202,220,209,219,217,206,127,    // 1981---1990
    215,204,123,210,131,219,207,128,216,205,    // 1991--2000
    124,212,201,122,209,129,218,207,126,214,    // 2001--2010
    203,123,210,131,219,208,128,216,205,125,    // 2011--2020                                       //2011--2020
    212,201,122,210,129,217,206,126,213,203,    // 2021--2030
    123,211,131,219,208,128,215,204,124,212     // 2031--2040
};

//16--18位表示闰几月，0--12位表示农历每月的数据，高位表示1月，低位表示12月（农历闰月就会多一个月）
static const int  nLunarData[] = {
    461653,1386,2413,                                            // 1968,1969,1970
    330077,1197,2637,268877,3365,531109,2900,2922,398042,2395,   // 1971--1980
    1179,267415,2635,661067,1701,1748,398772,2742,2391,330031,   // 1981---1990
    1175,1611,200010,3749,527717,1452,2742,332397,2350,3222,     // 1991--2000
    268949,3402,3493,133973,1386,464219,605,2349,334123,2709,    // 2001--2010
    2890,267946,2773,592565,1210,2651,395863,1323,2707,265877,   // 2011--2020
    1706,2773,133557,1206,398510,2638,3366,335142,3411,1450,     // 2021 -- 2030
    200042,2413,723293,1197,2637,399947,3365,3410,334676,2906    // 2031 -- 2040
};

static const char *s_pchDayName[] =  {
    ("*"),
    ("初一"), ("初二"), ("初三"), ("初四"), ("初五"),
    ("初六"), ("初七"), ("初八"), ("初九"), ("初十"),
    ("十一"), ("十二"), ("十三"), ("十四"), ("十五"),
    ("十六"), ("十七"), ("十八"), ("十九"), ("二十"),
    ("廿一"), ("廿二"), ("廿三"), ("廿四"), ("廿五"),
    ("廿六"), ("廿七"), ("廿八"), ("廿九"), ("三十")
};

/*农历月份名*/
static const char *s_pchMonName[] = {
    ("*"),
    ("正月"), ("二月"), ("三月"), ("四月"),
    ("五月"), ("六月"), ("七月"), ("八月"),
    ("九月"), ("十月"), ("冬月"), ("腊月")
};

/*公历每月前面的天数*/
int monthAdd[] = {
    0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334
};

Date::Date(QObject *parent) : QObject(parent)
{

}

/**
 * @brief Date::getLunarDate 计算农历
 * @param year  年
 * @param month 月
 * @param day 日
 * @return
 */
QString Date::getLunarDate (int year, int month, int day){
    int nTheDate,nIsEnd,nMonTemp,k,n,nBit;
    // 先获取公历节日
    QString strDate;

    /*现在计算农历：获得当年春节的公历日期（比如：2015年春节日期为（2月19日）），
            以此为分界点，2.19前面的农历是2014年农历（用2014年农历数据来计算），
            2.19以及之后的日期，农历为2015年农历（用2015年农历数据来计算）。*/
    nMonTemp = year - 1968;
    if ((year <= 1968) || (year >=2040)) {
        return "";
    }
    int springFestivalMonth = springFestival[nMonTemp] / 100;
    int springFestivalDaty = springFestival[nMonTemp] % 100;

    if(month < springFestivalMonth )
    {
        nMonTemp--;
        nTheDate = 365 * 1 + day + monthAdd[month - 1] - 31 * ((springFestival[nMonTemp] / 100) - 1) - springFestival[nMonTemp] % 100 + 1;

        if((!(year % 4)) && (month > 2))
            nTheDate = nTheDate + 1;

        if((!((year - 1) % 4)))
            nTheDate = nTheDate + 1;
    }
    else if (month == springFestivalMonth)
    {
        if (day < springFestivalDaty) {
            nMonTemp--;
            nTheDate = 365 * 1 + day + monthAdd[month - 1] - 31 * ((springFestival[nMonTemp] / 100) - 1) - springFestival[nMonTemp] % 100 + 1;

            if((!(year % 4)) && (month > 2))
                nTheDate = nTheDate + 1;

            if((!((year-1) % 4)))
                nTheDate = nTheDate + 1;
        }
        else {
            nTheDate = day + monthAdd[month - 1] - 31 * ((springFestival[nMonTemp] / 100) - 1) - springFestival[nMonTemp] % 100 + 1;

            if((!(year % 4)) && (month > 2))
                nTheDate = nTheDate + 1;
        }
    }else{
        nTheDate = day + monthAdd[month - 1] - 31 * ((springFestival[nMonTemp] / 100) - 1) - springFestival[nMonTemp] % 100 + 1;
        if((!(year % 4)) && (month > 2))
            nTheDate = nTheDate + 1;
    }
    nIsEnd = 0;

    while(nIsEnd != 1)  {
        if(nLunarData[nMonTemp] < 4095)
            k = 11;
        else
            k = 12;
        n = k;
        while(n >= 0)   {
            nBit = nLunarData[nMonTemp];

            nBit = nBit >> n;
            nBit = nBit % 2;
            if (nTheDate <= (29 + nBit))    {
                nIsEnd = 1;
                break;
            }

            nTheDate = nTheDate - 29 - nBit;
            n = n - 1;
        }

        if(nIsEnd)
            break;

        nMonTemp = nMonTemp + 1;
    }

    // 农历的年月日
    year = 1969 + nMonTemp -1;
    month = k - n + 1;

    day = nTheDate;
    if (k == 12) {
        if (month >= (nLunarData[nMonTemp] / 65536) + 1)
            month = month - 1;
    }
    if (day == 1) {
        strDate = tr(s_pchMonName[month]);
    } else {
        strDate = tr(s_pchDayName[day]);
    }


    return strDate;
}

DateConfig& DateConfig::instance()
{
    static DateConfig* _self = nullptr;
    if (!_self) {
        _self = new DateConfig();
    }
    return *_self;
}

int DateConfig::getDate(QDate date)
{
   QString key = date.toString("yyyy-MM-dd");
   auto iter = worklist.find(key);
   if (iter != worklist.end()){
        return iter.value();
   } else {
       return -1;
   }
}

void DateConfig::Init()
{
    worklist.clear();
    QFile loadFile("/etc/nde/holiday.json");
    if (loadFile.exists()){
        loadFile.open(QIODevice::ReadOnly | QIODevice::Text);

        QString strCfg = QString::fromLocal8Bit(loadFile.readAll());
        QJsonDocument loadDoc = QJsonDocument::fromJson(strCfg.toUtf8());

        QJsonArray array;
        if (loadDoc.isArray()){
            array = loadDoc.array();
            int size = array.size();
            for (int i =0; i< size; i++){
                QJsonObject ob = array.at(i).toObject();
                QString date = ob.value("date").toString();
                int flag = ob.value("flag").toInt();
                worklist.insert(date,flag);
            }
        }
    }
}

dateWidget::dateWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::dateWidget)
{
    ui->setupUi(this);

    labelAdd = new QLabel(this);
    labelAdd->setObjectName("labelAdd");
    labelAdd->setFixedSize(QSize(16,16));
    labelAdd->setAlignment(Qt::AlignCenter);

    QLocale locale;
    if (!(locale.language() == QLocale::Chinese || locale.language() == QLocale::China)){
        ui->labelOldDate->hide();
    }
}

dateWidget::~dateWidget()
{
    delete ui;
}

void dateWidget::refreshSize(bool normal)
{
    if (!normal){
        labelAdd->setFixedSize(QSize(12,12));
    }else{
        labelAdd->setFixedSize(QSize(16,16));
    }

    style()->polish(this);
    style()->polish(labelAdd);
    style()->polish(ui->labelNewDate);
    style()->polish(ui->labelOldDate);
    update();
}

void dateWidget::setDateText(QDate date)
{
    int ret = DateConfig::instance().getDate(date);
    if (ret == 1){
        labelAdd->show();
        labelAdd->setText(tr("OT"));
        setProperty("IsOverTime", true);
        setProperty("IsHoliday", false);
    }else if (ret == 0){
        labelAdd->show();
        labelAdd->setText(tr("OFF"));
        setProperty("IsHoliday", true);
        setProperty("IsOverTime", false);
    }else{
        setProperty("IsHoliday", false);
        setProperty("IsOverTime", false);
        labelAdd->hide();
    }

    ui->labelNewDate->setText(QString::number(date.day()));
    ui->labelOldDate->setText(Date::getLunarDate(date.year(),date.month(),date.day()));

    setProperty("CurrentDay", date == QDate::currentDate());
    setProperty("IsSelected", false);

    style()->polish(this);
    style()->polish(labelAdd);
    style()->polish(ui->labelNewDate);
    style()->polish(ui->labelOldDate);
    update();
}

void dateWidget::paintEvent(QPaintEvent *event)
{
    QStyleOption opt;
    opt.init(this);
    QPainter painter1(this);
    style()->drawPrimitive(QStyle::PE_Widget, &opt, &painter1, this);
}

void dateWidget::enterEvent(QEvent *event)
{
    setProperty("Hover", true);
    if (lastSelect)
        style()->polish(lastSelect);

    style()->polish(this);
    update();
}

void dateWidget::leaveEvent(QEvent *event)
{
    setProperty("Hover", false);
    style()->polish(this);
    if (lastSelect)
        style()->polish(lastSelect);
    update();
}

void dateWidget::mousePressEvent(QMouseEvent *event)
{
    if (lastSelect) {
        lastSelect->setProperty("IsSelected", false);
        style()->polish(lastSelect);
    }
    lastSelect = this;
    this->setProperty("IsSelected", true);
    style()->polish(this);
    update();
}
