/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)LGPL2+
 *
 * Nde - a lightweight, Qt based, desktop toolset
 * https://lxqt.org
 *
 * Copyright: 2012-2013 Razor team
 *            2014 Nde team
 * Authors:
 *   Kuzma Shapran <kuzma.shapran@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#ifndef NDE_PANEL_WORLDCLOCK_H
#define NDE_PANEL_WORLDCLOCK_H

#include <QTimeZone>

#include <QDialog>
#include <QLabel>
#include <QVBoxLayout>
#include <Nde/RotatedWidget>
#include <gio/gio.h>
#include "../panel/indepanelplugin.h"
#include "ndeworldclockconfiguration.h"
#include "calendar.h"
#include <QVBoxLayout>
class ActiveLabel;
class QTimer;
class NdeWorldClockPopup;


class NdeWorldClock : public QObject, public INdePanelPlugin
{
    Q_OBJECT
public:
    NdeWorldClock(const INdePanelPluginStartupInfo &startupInfo);
    ~NdeWorldClock();

    virtual QWidget *widget() { return mMainWidget; }
    virtual QString themeId() const { return QLatin1String("WorldClock"); }
    virtual INdePanelPlugin::Flags flags() const { return PreferRightAlignment | HaveConfigDialog ; }
    bool isSeparate() const { return true; }
    void activated(ActivationReason reason);
    void getTimeMode();

    virtual void settingsChanged();
    virtual void realign();
    QDialog *configureDialog();

private Q_SLOTS:

    void timeout();
    void wheelScrolled(int);
    void updateTimeText();

private:

    QWidget *mMainWidget;
    Nde::RotatedWidget* mRotatedWidget;
    ActiveLabel *mContent;
    Calendar * calendarWidget {nullptr};
    QTimer *mTimer;
    int mUpdateInterval;
    QVBoxLayout *borderLayout;
    QStringList mTimeZones;
    QMap<QString, QString> mTimeZoneCustomNames;
    QString mDefaultTimeZone;
    QString mActiveTimeZone;
    QString mFormat;
    INdePanel::Position mCurPanelPos;
    QRect mScreenRect;
    bool mAutoRotate;
    GSettings *clock_settings {nullptr};

    QDateTime mShownTime;
    bool mVisible{true};

    void restartTimer();

    void setTimeText();
    void updatetext();
    QString formatDateTime(const QDateTime &datetime, const QString &timeZoneName);
    bool formatHasTimeZone(QString format);
    QString preformat(const QString &format, const QTimeZone &timeZone, const QDateTime& dateTime);
};


class ActiveLabel : public QLabel
{
Q_OBJECT

public:
    explicit ActiveLabel(QWidget * = NULL);

Q_SIGNALS:
    void wheelScrolled(int);
    void leftMouseButtonClicked();
    void middleMouseButtonClicked();

protected:
    void wheelEvent(QWheelEvent *);
    void mouseReleaseEvent(QMouseEvent* event);
};

class NdeWorldClockLibrary: public QObject, public INdePanelPluginLibrary
{
    Q_OBJECT
    Q_PLUGIN_METADATA(IID "lxqt.org/Panel/PluginInterface/3.0")
    Q_INTERFACES(INdePanelPluginLibrary)
public:
    INdePanelPlugin *instance(const INdePanelPluginStartupInfo &startupInfo) const
    {
        return new NdeWorldClock(startupInfo);
    }
};

#endif // NDE_PANEL_WORLDCLOCK_H
